/*******************************************************************/
/*                                                                 */
/*   File name :  pulsar.h                                         */
/*                                                                 */
/*      Version:   2020-05-26                                      */
/*                                                                 */
/*   Copyright (c) 2020-     T.Kondo All Right Reserved            */
/*                                                                 */
/*******************************************************************/

//    2020-05-27  start development

#include <stdio.h>
#include <signal.h> 
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef __CYGWIN__
#ifndef linux
#define linux
#endif
#endif

#ifdef linux
#include <sys/time.h>
#endif

#ifndef WIN32
#include <unistd.h>
#endif

#include "g_head.h"    // header for graphics

#ifdef WIN32
  #define USE_FFTW    // use FFTW package
#endif

#ifdef USE_FFTW
 #ifdef WIN32
  #include "fftw3.h"     // use of FFTW package
 #else
  #include <fftw3.h>     // use of FFTW package
 #endif
#endif

#include "libk5.h"      // common header for IP-VLBI utilities

#include "libfft.h"

#ifdef WIN32
#define USE_ASM    // use inline assemble function
#endif

#include "libvssp32.h"
#include "libspeplot.h"


typedef struct runp     // run parameters for this
{
    char fname[128];  // data file name
    char comment[60]; // comment
    double tspan;     // 
    double soffset;   // start offset in floating second 
    int  mode2;        // axis mode
    int  mode;        // axis mode
    int  ymode;       //
    int amode;        // 
    int pmode;        // plot mode
    int flag_halt;
    int sleep_msec;
    int flag_power;   // 1: power plot mode     0: normal  2020-05-20
    double minpow;   // plot power range    2020-05-20
    double maxpow;   //     "
    double tave;     // averaging period for power mode  (default : equivalent to nfft)
    double period;   // pulsar period
    double phase;    // pulsar phase (deg) at reference time
    int iprt[5];     // pulsar reference time YYYY DDD HH MM SS  (default is time at beginning of data
                     //      use HH MM SS only
    double prtsec;   // reference time (second in a day)
    double dtime1st;  // data 1st time in sec
    double dtime;     // for work
    double *pdat;     // pulsar synchronaized period data
    int    *cdat;     // counts at each bin in pdat
    int npdat;        // number of elements of pdat
    int npulse;
    double tlength;   // time length to process
} runp_t;

void plotpulse(speana_param_t *st, runp_t *rp);
void plotpulse_a(speana_param_t *st,  runp_t *rp, int mch);
char *sectohhmmss(double dsec);

