/*******************************************************************/
/*                                                                 */
/*   Program Name :  skdchk                                        */
/*                                                                 */
/*      Version:  2.40  2020-05-13                                 */
/*                                                                 */
/*      Copyright (c) 2002-2020 T.Kondo All Right Reserved         */
/*                                                                 */
/*******************************************************************/
/*

  Schedule File Read and Check  
            2002-06-02 
            by T. Kondo

    How to run
       skdchk sked_file [-NOEARLY] [-LEGACY]
         where sked_file ---- schedule file name
               -NOEARLY   --- ignore "tape early start" parameter
               -LEGACY    --- use old method to analyze $CODE block of SKED file
                              to keep compatibility with the version before 2019-12-21
               
Ver 2.40 2020-05-13
      reflect revise of TRACK analysis in case of VEX (add data modulation analysis)
Ver 2.39 2019-12-25
      reflect debug in "set_up_codes" in libskdvex.c
Ver 2.38 2019-12-21
      renew method of analyzing $CODE block of SKED file. 
      use option '-LEGACY' to use the analyzing method adopted before 2019-12-21.
      revise display of frequency info to display for all stations in case of SKED file
Ver 2.37 2019-10-11
      reflect the revision of libskdvex.c
Ver 2.36 2019-06-16
      revised
Ver 2.35 2019-06-13
      add display frequency information
Ver 2.34 2019-05-22
      to reflect the revision of libskdvex
Ver 2.33 2018-10-11
      debug in case of # of scans < 5
Ver 2.32 2017-08-29
      reflect the change of maximum number of charcters in star name in libskdvex
Ver 2.31 2016-10-12
      reflect debug and revision in libskdvex.c
Ver 2.30 2016-09-16
      reflect large scale modification in libskdvex.h and libskdvex.c
Ver 2.25 2016-09-14
      minor debug
Ver 2.24 2009-11-19
      scanf("%s",&XX) was changed to scanf("%s",XX) when XX is 
      declared as like "char XX[20]" to avoid warning at the latest
      gcc (ver. 4.4)
      scanf(...) was changed to like i=scanf(...) to avoid warning
      at Ubuntu 10.10
      system(...) was changed to like i=system(...) to avoid warning
      at Ubuntu 10.10
Ver 2.23 2009-11-19  
      add warning messages in case of the lack of sizes in
      arrays for scans, stations, sources, and data ack terminals.
Ver 2.22 2009-11-17
      modify to reflect MAXSTAT in arrays
Ver 2.21 2008-02-19
      options '--help' and '--ver[sion]' were added
Ver 2.20 2007.03.28
      bug fix in calculating total seconds by station in case
      of subnet and early start is selected 
Ver 2.11 2007.03.16
      improve in display in case of 7 stations. use
      "scan" instead of "observation". improve in
      station position display.  add # of scans by station
      in final display
Ver 2.10 2006.12.08  
      add early parameter handling
Ver 2.01 2005.06.12
      update due to update in common header name
2005-01-12  support VEX file
2003-08-08  support subnet

../sked/jd0306.skd
../sked/gg057c.vex
*/
#define SKDCHK_VERSION "Ver 2.40    2020-05-13"
#include "libk5.h"

int main(int argc, char *argv[])
{
    //char* fname="ge154.skd";
    sked_data_t skd;
    //struct schedule skd;
    double totsec, a, b, c, d;
    //double totsec_by_sta[20];
    //int nscan_by_sta[20];  // 2007-03-16
    //double totsec_by_sta[MAXSTAT]; //2009-11-17
    //int nscan_by_sta[MAXSTAT];  // 2009-11-17
    double *totsec_by_sta; //2016-09-16
    int *nscan_by_sta;  // 2016-09-16
    int i, k, smax, imax, k1, k2,m;
    int gmax, n;   // 2019-06-12
    int vexmode;  // 0: normal schedule file    1: VEX file  2005-01-12
    int noearly;  // 1: no use of early parameter  0: use of early parameter
    int secadd;
    int kid;
    int idummy;
    int flag_legacy;  // 2019-12-21
    char stat_id[4];  // 2005-01-12
    char sideband[4], pol[4];  // 2019-06-13
    vex_t vex;    // 2005-01-12

    if (argc == 1 || (argc == 2 && strncmp(*(argv + 1), "--help", 6) == 0)) { // 2008-02-19
        printf("\n");
        printf("skdchk %s\n", SKDCHK_VERSION);
        printf("Usage: skdchk sked_file [-NOEARLY] [-LEGACY]\n");
        printf("\n");
        printf("         where  sked_file -- schedule file name\n");
        printf("                               (VEX file is supported)\n");
        printf("                -NOEARLY --- ignore EARLY start parameter\n");
        printf("                               in SKED or VEX\n");
        printf("                -LEGACY  --- use the old method to analyze $CODE block\n");
        printf("                             of SKED file to keep compatibility with the\n");
        printf("                             version before 2019-12-21\n");
        exit(0);
    }
    if (argc == 2 && strncmp(*(argv + 1), "--ver", 5) == 0) { // 2008-02-19
        printf("\n");
        printf("skdchk %s\n", SKDCHK_VERSION);
        printf("\n");
        exit(0);
    }

    printf("skdchk %s\n", SKDCHK_VERSION);
    strcpy(skd.skdname, argv[1]);
    printf("Schedule file is  %s\n", skd.skdname);
    noearly = 0;
    flag_legacy = 0;  // 2019-12-21
    if (argc >= 3) {
        if (strncmp(argv[2], "-NOEARLY", 8) == 0 ||
            strncmp(argv[2], "-noearly", 8) == 0) noearly = 1;   // set no early flag
        if (strncmp(argv[2], "-LEGACY", 7) == 0 ||
            strncmp(argv[2], "-legacy", 7) == 0) flag_legacy = 1;   // set legacy flag 2019-12-21
    }
    if (argc >= 4) {
        if (strncmp(argv[3], "-NOEARLY", 8) == 0 ||
            strncmp(argv[3], "-noearly", 8) == 0) noearly = 1;   // set no early flag
        if (strncmp(argv[3], "-LEGACY", 7) == 0 ||
            strncmp(argv[3], "-legacy", 7) == 0) flag_legacy = 1;   // set legacy flag 2019-12-21
    }
    if (flag_legacy == 1) { // 2019-12-21
        printf("  skdchk is ruuning in \"LEGACY\" mode\n");
        printf("  (compatible with the version before 2019-12-21)\n");
    }
    printf("\n");

    skd.flag_code_new_anal = flag_legacy;   // 2019-12-21

    skd.pickup_baseid[0] = '\0'; // 2016-10-12
    skd.pickup_source[0] = '\0'; // 2016-10-12

    /* schedule file format check original SKED or VEX or OVEX */
    vexmode = 0;
    check_sked_type(skd.skdname, skd.skedtype);

    if ((strncmp(skd.skedtype, "VEX", 3) == 0) ||
        (strncmp(skd.skedtype, "OVEX", 4) == 0)) {    /* VEX or OVEX format*/
        printf("File type is VEX\n");
        vexmode = 1;
        strcpy(vex.vex_file_name, skd.skdname);

        get_nelement_VEX(&vex);                       // 2016-09-16

        VEX_malloc(&vex);                             // 2016-09-16
        skd.nobs_max = vex.numobs_max;                  // 2016-09-16
        skd.nstar_max = vex.numstar_max;                // 2016-09-16
        skd.nstation_max = vex.nstation_max;            // 2016-09-16
        skd.ch_max = MAXCH;                             // 2016-09-16
        skd.code.ncode_max = MAXCODES;                                 // 2017-08-28

        SKED_malloc(&skd);                            // 2016-09-15

        vex.neop = 0;   /* initialize for no EOP data case*/
        vex_read(&vex);
        //SKED_mon(&vex);    //debug
        //FREQ_mon(&vex);    //debug
        //CLOCK_mon(&vex);  //debug
        //STATION_mon(&vex);  //debug
        //MODE_mon(&vex);  //debug
        //TRACKS_mon(&vex);  //debug
        stat_id[0] = '\0';
        vex2sked3(&vex, stat_id, &skd);

        VEX_free(&vex);                               // 2016-09-16

        /* slect station  */

        printf(" ****** STATION ID TABLE *********\n");
        for (i = 0; i < skd.nstation; i++) {
            printf("  %s --- %s\n", skd.statid[i], skd.statnam[i]);
        }
        printf(" *********************************\n");
        printf(" Enter Station ID ----> ");
        //scanf("%s",&stat_id);
        idummy = scanf("%s", stat_id);   // 2011-02-09
        printf(" Selected station_id is %s\n", stat_id);

        // setup sked data again using given stat_id
        //vex2sked3(&vex, stat_id, &skd);

        /* Pick up scans where given_station ID is included */
        pickup_skd(&skd, stat_id);

    }
    else {                                   /* SKED format */
    // check # ofscans, # of stations, # of stars in VEX file for future revision  2016-09-14

        get_nelement_SKED(&skd);                       // 2016-09-15
        SKED_malloc(&skd);                             // 2016-09-15
        //skd.flag_code_new_anal = 0;  // 2019-12-20
        skd.flag_code_new_anal = 1;  // 2019-12-20
        readskd(&skd);

    }

    //mon_st_freq(&skd);  // debug

    if (vexmode == 1) {
        // find index corresponding to stat_id 
         // get ID number for stat_id
        kid = 0;
        for (k = 0; k < skd.nstation; k++) {
            if (strncmp(skd.statid[k], stat_id, strlen(stat_id)) == 0) {
                kid = k;
                break;
            }
        }
    }

    printf("\n");
    printf("*** SCHEDULE FILE (%s) INFORMATION ***\n", skd.skdname);
    printf("Experiment code    : %s\n", skd.expcode);
    printf("Number of stations : %d\n", skd.nstation);


        for (i = 0; i < skd.nstation; i++) {
            //printf("  %3d %2s %10s %10f %10f %10f\n",i+1,skd.statid[i],skd.statnam[i],
            printf("  %3d %2s %10s %16f %16f %16f\n", i + 1, skd.statid[i], skd.statnam[i],
                skd.xyz[i][0], skd.xyz[i][1], skd.xyz[i][2]);
        }

        /* added on 2019-06-13 */
        if (vexmode == 1) {
            printf("Frequency Information for %s\n", skd.statnam[kid]);
        }
        else {
            printf("Frequency Information\n");
            kid = 0;
        }
        gmax = (skd.nfreq - 1) / 4 + 1;
        if (vexmode == 1) {
            //printf("   ------- Frequency Table (Freq MHz, USB/LSB Polarization) -------\n");
            printf("   ------- Frequency Table (FreqMHz, USB/LSB, Polarization, CH#, Thread#) -------\n");
            for (i = 0; i < gmax; i++) {
                printf("    Gr# %2d ", i + 1);
                for (k = 0; k < 4; k++) {
                    n = 4 * i + k;
                    if (n >= skd.nfreq) break;
                    //printf(" %9.2f %s",skd->freqtab[n],skd->sideband[n]);
                    sideband[0] = '\0';
                    if (skd.st_freq[kid].ul[n] == 1) strcpy(sideband, "USB");
                    if (skd.st_freq[kid].ul[n] == 0) strcpy(sideband, "LSB");
                    strcpy(pol, skd.st_freq[kid].pol[n]);
                    printf(" %9.2f %s %s", skd.st_freq[kid].freq[n], sideband, pol);
                    printf(" %2d", skd.st_freq[kid].ch[n]);
                    printf(" %1d", skd.st_freq[kid].thread[n]);
                }
                printf("\n");
            }
        }
        else {  // SKED mode
            /*
            printf("   ------- Frequency Table (FreqMHz, USB/LSB) -------\n");
            for (i = 0; i < gmax; i++) {
                printf("    Gr# %2d ", i + 1);
                for (k = 0; k < 4; k++) {
                    n = 4 * i + k;
                    if (n >= skd.nfreq) break;
                    //printf(" %9.2f %s",skd->freqtab[n],skd->sideband[n]);
                    sideband[0] = '\0';
                    if (skd.st_freq[kid].ul[n] == 1) strcpy(sideband, "USB");
                    if (skd.st_freq[kid].ul[n] == 0) strcpy(sideband, "LSB");
                    printf(" %9.2f %s", skd.st_freq[kid].freq[n], sideband);
                }
                printf("\n");
            }
            */
            //  above is revised as follows 2019-12-21
            for (m = 0; m < skd.code.ncode; m++) {
                printf(" Stations: ");
                for (i = 0; i < skd.code.ctab[m].nstation; i++)
                    printf(" %s", skd.code.ctab[m].statnam[i]); printf("\n");
                printf("   ------- Frequency Table (FreqMHz, USB/LSB) -------\n");
                for (i = 0; i < gmax; i++) {
                    printf("    Gr# %2d ", i + 1);
                    for (k = 0; k < 4; k++) {
                        n = 4 * i + k;
                        if (n >= skd.nfreq) break;
                        //printf(" %9.2f %s",skd->freqtab[n],skd->sideband[n]);
                        sideband[0] = '\0';
                        //if (skd.st_freq[kid].ul[n] == 1) strcpy(sideband, "USB");
                        //if (skd.st_freq[kid].ul[n] == 0) strcpy(sideband, "LSB");
                        if (skd.code.ctab[m].sideband[n][0] == 'U') strcpy(sideband, "USB");
                        if (skd.code.ctab[m].sideband[n][0] == 'L') strcpy(sideband, "LSB");
                        //printf(" %9.2f %s", skd.st_freq[kid].freq[n], sideband);
                        printf(" %9.2f %s", skd.code.ctab[m].freqtab[n], sideband);
                    }
                    printf("\n");
                }
                for (i = 0; i < gmax; i++) {
                    printf("   ------- PCAL Freq (kHz) Table ---------\n");
                    for (i = 0; i < gmax; i++) {
                        printf("    Gr# %2d ", i + 1);
                        for (k = 0; k < 4; k++) {
                            n = 4 * i + k;
                            if (n >= skd.nfreq) break;
                            //printf(" %8.1f",skd->pcalf[0][n]/1.0e3);
                            printf(" %8.1f", skd.st_freq[kid].pcalf[n][0] / 1.0e3);
                            //printf(" %8.1f", skd.code.ctab[m].pcalf[0][n] / 1.0e3);
                        }
                        printf("\n");
                    }
                }
                printf("   --------------------------------------------------\n");
                //printf("\n");

            }
        }
        //printf("   ----------------------------------------------------------------\n");
        if (vexmode == 1) {  // 2019-12-21  add this line
            printf("   ------- PCAL Freq (kHz) Table ---------\n");
            for (i = 0; i < gmax; i++) {
                printf("    Gr# %2d ", i + 1);
                for (k = 0; k < 4; k++) {
                    n = 4 * i + k;
                    if (n >= skd.nfreq) break;
                    //printf(" %8.1f",skd->pcalf[0][n]/1.0e3);
                    printf(" %8.1f", skd.st_freq[kid].pcalf[n][0] / 1.0e3);     // 2019-06-12
                }
                printf("\n");
            }
        }
        /* add end */


       printf("Number of stars : %d\n",skd.nstar);

       if(skd.nstar > 20) {
          imax=20;
          printf("   (only 20 stars are listed here)\n");
       } else {
          imax=skd.nstar;
       }

       for(i=0; i < imax; i++)
       {
           printf("  %3d %10s %10s %10f %10f %10f\n",i+1,skd.starnam1[i],skd.starnam2[i],
                         skd.ra[i],skd.dec[i],skd.epo[i]);
       }
       if(vexmode==1){
           printf("Number of scans for %10s: %d\n",skd.statnam[kid],skd.nobs);
       } else {
           printf("Number of scans : %d\n",skd.nobs);
       }

       if(skd.nobs > 11){   // add 2018-10-11
           printf("   First 5 scans are as follows:\n");
           for (i=0; i < 5; i++){
              printf("    %4d %10s %2d/%03d %02d:%02d:%02d  %3d\n",i+1,skd.obsstar[i],     // 2016-09-14
                       skd.start[i][0],skd.start[i][1],skd.start[i][2],
                       skd.start[i][3],skd.start[i][4],skd.dura[i]);
           }

           printf("   Last 5 scans are as follows:\n");
           for (i=skd.nobs-5; i < skd.nobs; i++){
              printf("    %4d %10s %2d/%03d %02d:%02d:%02d  %3d\n",i+1,skd.obsstar[i],     // 2016-09-14
                     skd.start[i][0],skd.start[i][1],skd.start[i][2],
                     skd.start[i][3],skd.start[i][4],skd.dura[i]);
           }
       } else {      // added on 2018-10-11
           printf("   Scan information is as follows:\n");
           for (i=0; i < skd.nobs; i++){
              printf("    %4d %10s %2d/%03d %02d:%02d:%02d  %3d\n",i+1,skd.obsstar[i],   
                       skd.start[i][0],skd.start[i][1],skd.start[i][2],
                       skd.start[i][3],skd.start[i][4],skd.dura[i]);
           }
       }

       totsec_by_sta=(double *)malloc(skd.nstation*sizeof(double));  // 2016-09-16
       nscan_by_sta =(int *)malloc(skd.nstation*sizeof(int));        // 2016-09-16

       /* Disk requirement check */
       totsec=0.0;
       for(k=0; k<skd.nstation; k++) {
           totsec_by_sta[k]=0.0;
           nscan_by_sta[k]=0;  // 2007-03-16
       }

       secadd=skd.earlysec;

       printf("\n     Early Start Parameter (sec) : %d",skd.earlysec);
       if(noearly==1) {
           printf("  but omitted by operator!");
           secadd=0;
       }
       printf("\n");

       for(i=0; i< skd.nobs; i++){
           if(vexmode==1){
               if(skd.datastop[i][kid] > 0)     // 2007-03-28
                  totsec += (double)(skd.datastop[i][kid]+secadd);
           } else {
               if(skd.dura[i] > 0)     // 2007-03-28
                  totsec += (double)(skd.dura[i]+secadd);
           }
           for(k=0; k<skd.nstation; k++){
               if(skd.datastop[i][k] > 0)     // 2007-03-28
                   totsec_by_sta[k]+= (double)(skd.datastop[i][k]+secadd);
               if(skd.datastop[i][k] > 0) nscan_by_sta[k]++;  // 2007-03-16
           }
       }
       a=totsec*32.0e6;  // total bit  in case of 32Mbps
       b=totsec*64.0e6;  // total bit  in case of 64Mbps
       c=totsec*128.0e6;  // total bit  in case of 128Mbps
       d=totsec*256.0e6;  // total bit  in case of 256Mbps
       a /= 8.0e9;   // in Gbytes
       b /= 8.0e9;   // in Gbytes
       c /= 8.0e9;   // in Gbytes
       d /= 8.0e9;   // in Gbytes
       printf("\n");
       if(vexmode==1){
           printf("    Disk requirements for %10s\n",skd.statnam[kid]);
       } else {
           printf("    Maximum Disk requirements\n");
       }
       printf("      Total observation time (sec)  = %d\n",(int)totsec);
       printf("       Disk requirements\n");
       printf("           32Mbps  : %10.3f GBytes\n",a);
       printf("           64Mbps  : %10.3f GBytes\n",b);
       printf("          128Mbps  : %10.3f GBytes\n",c);
       printf("          256Mbps  : %10.3f GBytes\n",d);

       if(vexmode==1) goto bypass;
       smax=7;
       k1=0; 
       while(1){
          k2=k1+smax;
          if(k2 > skd.nstation) k2=skd.nstation;
          printf("\n");
          printf("                   Disk Requirements by Station (GBytes)\n");
          for(k=0; k<78; k++) printf("="); printf("\n");
          printf("        ");
          for(k=k1; k<k2; k++) printf("%10s",skd.statnam[k]); printf("\n");
          for(k=0; k<78; k++) printf("-"); printf("\n");
          printf("    sec ");
          for(k=k1; k<k2; k++) printf("%10d",(int)totsec_by_sta[k]); printf("\n");
          printf(" #scans ");
          for(k=k1; k<k2; k++) printf("%10d",nscan_by_sta[k]); printf("\n");
          for(k=0; k<78; k++) printf("-"); printf("\n");
          printf(" 32Mbps ");
          for(k=k1; k<k2; k++) printf("%10.1f",totsec_by_sta[k]*4e-3); printf("\n");
          printf(" 64Mbps ");
          for(k=k1; k<k2; k++) printf("%10.1f",totsec_by_sta[k]*8e-3); printf("\n");
          printf("128Mbps ");
          for(k=k1; k<k2; k++) printf("%10.1f",totsec_by_sta[k]*16e-3); printf("\n");
          printf("256Mbps ");
          for(k=k1; k<k2; k++) printf("%10.1f",totsec_by_sta[k]*32e-3); printf("\n");
          for(k=0; k<78; k++) printf("="); printf("\n");
          k1+=smax;
          //if(k1 >= skd.nstation) break;
          if(k1 >= skd.nstation) break;  // 2008-03-16
       }

bypass:
       /* added on 2009-11-19 */
       if(vexmode==0){  // SKED file
           if(skd.nobs != skd.nobs_true){
               printf("Warning! Number of scans (%d) exceeds program limit (%d).\n",
                   skd.nobs_true,skd.nobs_max);
               printf("         So first %d scans were analyzed.\n",skd.nobs);
               //printf("         To solve this problem, change MAXOBS parameter in\n"); 
               //printf("         src/chead.h and compile again!\n");
           }
           if(skd.nstar != skd.nstar_true){
               printf("Warning! Number of sources (%d) exceeds program limit (%d).\n",
                   skd.nstar_true,skd.nstar_max);
               printf("         So first %d sources were analyzed.\n",skd.nstar);
               //printf("         To solve this problem, change MAXSTAR parameter in\n"); 
               //printf("         src/chead.h and compile again!\n");
           }
           if(skd.nstation != skd.nstation_true){
               printf("Warning! Number of stations (%d) exceeds program limit (%d).\n",
                   skd.nstation_true,skd.nstation_max);
               printf("         So first %d stations were analyzed.\n",skd.nstation);
               //printf("         To solve this problem, change MAXSTAT parameter in\n"); 
               //printf("         src/chead.h and compile again!\n");
           }
       }
       if(vexmode==1){  // VEX file
           if(vex.numobs != vex.numobs_true){
               printf("Warning! Number of scans (%d) exceeds program limit (%d).\n",
                   vex.numobs_true,vex.numobs_max);
               printf("         So first %d scans were analyzed.\n",vex.numobs);
               //printf("         To solve this problem, change MAXOBS parameter in\n"); 
               //printf("         src/chead.h and compile again!\n");
           }
           if(vex.numstar != vex.numstar_true){
               printf("Warning! Number of sources (%d) exceeds program limit (%d).\n",
                   vex.numstar_true,vex.numstar_max);
               printf("         So first %d sources were analyzed.\n",vex.numstar);
               //printf("         To solve this problem, change MAXSTAR parameter in\n"); 
               //printf("         src/chead.h and compile again!\n");
           }
           if(vex.nstation != vex.nstation_true){
               printf("Warning! Number of stations (%d) exceeds program limit (%d).\n",
                   vex.nstation_true,vex.nstation_max);
               printf("         So first %d stations were analyzed.\n",vex.nstation);
               //printf("         To solve this problem, change MAXSTAT parameter in\n"); 
               //printf("         src/chead.h and compile again!\n");
           }
           if(vex.numstat != vex.numstat_true){
               printf("Warning! Number of sites (%d) exceeds program limit (%d).\n",
                   vex.numstat_true,vex.numstat_max);
               printf("         So first %d stations were analyzed.\n",vex.numstat);
               //printf("         To solve this problem, change MAXSTAT parameter in\n"); 
               //printf("         src/chead.h and compile again!\n");
           }
           if(vex.ndas != vex.ndas_true){
               printf("Warning! Number of data ack terminals (%d) exceeds program limit (%d).\n",
                   vex.ndas_true,vex.ndas_max);
               printf("         So first %d stations were analyzed.\n",vex.ndas);
               //printf("         To solve this problem, change MAXDAS parameter in\n"); 
               //printf("         src/chead.h and compile again!\n");
           }
       }
       /* add end */

       SKED_free(&skd);  // 2016-09-15
       free(totsec_by_sta);
       free(nscan_by_sta);

       if(!system(NULL)) {  // command processor not implemented
          return 0;
       }
#ifndef WIN32
       /* Display Disk avilable size */
       printf("\n");
       printf("Disk mounted on your system is as follows:\n");
       printf("\n");
       //system("df");
       idummy=system("df");  // 2011-02-09
       printf("\n");
#endif

    return 0;
}
