#!/bin/sh
#

if [ -z $1 ]; then   #  display how to use
	echo '

	# K5/VSSP32 check procedure 2 by T.Kondo 2016-05-16
	#
	#  How to execute 
	#     
	#   ./vssp64test2.sh span [sfreq [adbit [ch [nkai [data_folder 
	#                [datachk_log_file [sampling_log_file
	#                [error_log_file ]]]]]
	#       where 
	#          span  --------  sampling period for a scan (sec)
	#          sfreq --------  sampling frequency 2, 4, 8, 16, 32, 64, 128 (MHz)
	#                            (default is 64)
	#          adbit --------  AD bits 1, 2, 4, 8  (default is 1)
	#          ch    --------  # of channels  1, 2, 4  (default is 1)
	#          nkai     ----- # of repeat   (default is 1)
	#          data_folder -- folder where sampled data file is created
	#                          (default is current directory)
	#          datachk_log_file -- file name for 'datachk' summary output
	#                         (default is "v32dtchkYYMMDDHHMMSS.HOST.log")
	#          sampling_log_file ---- log file name for 'sampling' program
	#                         (default is  "v32smplgYYMMDDHHMMSS.HOST.log")
	#          error_log_file  ---- error log file name for 'datachk' program
	#                               (default is  "v32errorYYMMDDHHMMSS.HOST.log")
	#              where YYMMDDHHMMSS is start time of this shell execution
	#                    HOST is a host PC name
	#
	#   Check items    
	#         sampling frequency :  given by operator (default is 64 MHz)
	#         AD bits            :  given by operator (default is 1)
	#         Channels           :  given by operator (default is 1)
	#         Scan length        :  given by operator 
	#         # of repeat        :  given by operator (default is 1)
	#
	#   Repeat 'nkai' times sampling with above fixed parameters
	#
	#   Execution example
	#      ./vssp32test2.sh 300 64 1 4 12
	#         ==>  this will last for about 1 hour.

	'
else

	# actual execution
	#pdir="./"        # program directory
	pdir="/home/kondo/ipvlbi/src/"        # program directory
	spl="sampling"
	chk="datachk"
#	ana="vssplogana"
	ana="vssplogana2"
	dco="setdcoffset"
	count=0
	tscan=0
	span=$1     # sampling span (sec)
	sf=$2       # sampling frequency
	ad=$3       # AD bits
	ch=$4       # channels
	nkai=$5     # repeat number
	d_dir=$6    # sampling data out directory
	d_log=$7    # log file name for 'datachk'
	s_log=$8    # log file name for 'sampling' and 'setdcoffset'
	e_log=$9    # errpr log file name for 'datachk'

	# get system time as YYMMDDHHMMSS
	ymd=`date +%y%m%d%H%M%S`
	# get a host PC name
	hn=`uname -n`

	if [ $1 = 0 ]; then  # $1 is 0
	   span=300
	fi
	if [ -z $2 ]; then  # $2 is null
	   sf=64
	fi
	if [ -z $3 ]; then  # $3 is null
	   ad=1
	fi
	if [ -z $4 ]; then  # $4 is null
	   ch=1
	fi
	if [ -z $5 ]; then  # $5 is null
	   nkai=1
	fi
	if [ -z $6 ]; then  # $6 is null
	   d_dir="./"
	fi
	if [ -z $7 ]; then  # $7 is null
	   d_log="v32dtchk"$ymd".$hn.log"
	fi
	if [ -z $8 ]; then  # $8 is null
	   s_log="v32smplg"$ymd".$hn.log"
	fi
	if [ -z $9 ]; then  # $9 is null
	   e_log="v32error"$ymd".$hn.log"
	fi
	echo 'span  = ' $span
	echo 'sfreq = ' $sf
	echo 'adbit = ' $ad
	echo 'ch    = ' $ch
	echo 'nkai  = ' $nkai
	echo 'data directory    is ' $d_dir
	echo 'datachk  log file is ' $d_log
	echo 'sampling and setdcoffset log file is ' $s_log
	echo 'datachk error log file is ' $e_log

	# delete sampling log file just in case
	# echo 'delete existed sampling log file (' $s_log ')'
	# rm $s_log

	# set board time using PC time to synchronize time in log file
#	$pdir/timesetpc 1 2

	# set DC affset adequate for testing
#	echo $pdir$dco TEST $s_log
#	$pdir$dco TEST $s_log

	# loop counter starts from 1

	count=`expr $count + 1`
	tscan=`expr $tscan + 1`
	
	# repeating kaisu
	while [ $count -le $nkai ];
	do 
       # data file name generation
       if [ `expr $count % 2` -eq 1 ]; then
           datf=$d_dir/test1.dat
       else
           datf=$d_dir/test4.dat
       fi
       echo 'rm '$datf
       rm $datf
	   echo '******** Loop# ' $count 'out of ' $nkai '********'
	   echo $sf'MHz '$ad'bit '$ch'ch '$span'sec testing...'
	   echo $pdir$spl $span $sf $ad $ch $datf $s_log
	   $pdir$spl $span $sf $ad $ch $datf $s_log
	   sleep 2s                                           # since 2016-09-01
	   if [ $tscan -eq 1 ]; then
	      # datacheck log file newly created
	      echo     $pdir$chk $datf 0 $d_log $e_log 1
	      $pdir$chk $datf 0 $d_log $e_log 1
	   else
	      # datacheck log file open as append mode
	      echo     $pdir$chk $datf 0 -$d_log -$e_log 1
	      $pdir$chk $datf 0 -$d_log -$e_log 1
	   fi
	   # log file analysis
	   echo $pdir$ana $s_log $d_log
	   $pdir$ana $s_log $d_log
	   tscan=`expr $tscan + 1`

	   count=`expr $count + 1`
	done

	# reset DC offset adequate for testing
#	echo $pdir$dco RESET $s_log
#	$pdir$dco RESET $s_log

	# end of actual execution
	echo ' '
	echo 'VSSP32 TEST completed.'
	echo '   sampling log file        : '$s_log
	echo '   datachk summary log file : '$d_log
	echo '   datachk error   log file : '$e_log
	echo '  You can execute '$ana' again to analyze above log files as follows.'
	echo '   '$pdir$ana $s_log $d_log
	echo '  See '$e_log' for kept errored data files.'
    echo ' '
fi
# end of all shell script
