/**************************************************************/
/*                                                            */
/*   File name :  libskdvex.c                                 */
/*                                                            */
/*      Version:   2020-10-10                                 */
/*                                                            */
/*   Copyright (c) 2005-2020 T.Kondo All Right Reserved       */
/*                                                            */
/**************************************************************/
/*

2020-10-10
  add "anal_track_then_conv_to_ch" function.
  revise "setup_ch_alloc_table_by_station", "vex_chinfo_get"
2020-09-30
  revise "setup_sked_to_st_freq" a bit
2020-06-26
  revise to use malloc for vparam_t type variable and cbuf, abuf according to a situation
2020-06-15
  add "TRACKS_check_adbit"
  revise "TRACKS_ana"
2020-05-13
  revise "TRACKS_ana", "TRACKS_mon" to include "data_modulation"
2020-04-13
  revise "TRACKS_ana", "TRACKS_mon" to include "bit/sampl"
  revise "FREQ_ana", "FREQ_mon" to include "chan_id0"
2020-04-12
  revise "SITE_ana", "sub_site_ana", "SITE_mon" to include 'mk4_id'
2020-01-11
  revise "vex_chinfo_get" to support the case that $PHASE_CAL_DETECT block is missed
  in a VEX file
2019-12-25
  bug-fix of "set_up_codes" in case of 1bit AD
2019-12-20
  add "set_up_codes" and "anal_skd_codes" functions
  revise "readskd_freq" function
2019-12-19
  add "mon_readskd_codes"
2019-10-11
 revise "vex_read" to avoid "$SCHEDULING_PARAMS;" keyword to misundrstand as "$SCHED;"
2019-06-16
 debug in "rraeskd"
2019-06-14
 add "setup_st_freq"
2019-06-13
 revise "vex2sked3", "vex2sked2", "vex_chinfo_get"
2019-06-12
  revise "SkdMonit", "readskd", "vex2sked2", "SKD_malloc", "SKD_free"
2019-06-11
  revise "SKED_malloc", "SKED_free", "readskd_freq", "setup_freq_table_for_sid_vex", 
  "vex_chinfo_get", "setup_ch_alloc_table_by_station", and "MODE_ana" functions.
2019-05-22
  add function "get_nIDs_in_VEX_MODE".
  revise function "get_nelement_VEX"
2018-10-12
  add function "setup_ch_alloc_table_by_station_for_sked"
2018-06-05
  dbug in serious bug in "EOP_ana"
2018-02-08
  debug in "get_nelement_VEX"
2018-02-02
  "SKED_malloc" and "SKED_free" were updated.
2017-09-04
   debug in "search_site", "search_site2", " CLOCK_get"
2017-09-03
   "setup_ch_alloc_table_by_station" was added
2017-09-01
  update "readskd", "SKED_malloc", and "SKED_free"
2017-08-29
  debug in "vex2sked3". vex->numstar is subtitued to skd->nstar
2017-08-28
   maximum number of characters of star name is changed from 11 to 15
2016-10-12
   debug in "get_VEX_nKEY". $SCHEDULING_PARAMS was confound with "$SCHED" key word.
   "vex2kked", "vex2sked2", "vex2sked3" were revised
2016-10-11
   add detail star information in "SkdMonit"
2016-09-30
   "VEX_malloc" was a bit revised
2016-09-29
   "readskd_codes" was revised
2016-09-16
   "readskd_sked", "skedlineana", "readskd_station", "sub_sked_ana", "SITE_ana",
   "STATION_ana", "SKED_ana", "SOURCE_ana", "sub_station_ana", "DAS_ana", "STATION_mon",
   "readskd", "pick_up_sked_codes_section" were modified.
   "get_nelement_VEX", "get_VEX_nKEY", "get_nelement_SKED", "get_SKED_nKEY",
   "SKED_malloc", "SKED_free", "VEX_malloc", and "VEX_free" were added.
2016-09-14
   "IF_ana", "BBC_ana", and "PCAL_ana" were revised to avoid error
2016-08-16
   "TRACKS_ana" was revised to get AD resolution information
2016-08-15
   "readskd_codes" and "parseSBinfo" functions were revised to AD resolution
   from a SKED file
2015-01-20
    minor update  (remove last '\0' in sprintf function)
2013-01-30
    functions "TrimTK", "pick_up_sked_codes_section", "readskd_codes" were
	added to reflect the multiple declaration of frequency tables in $CODES
	section in a SKED type schedule file.
	"readskd_freq" was also updated.
	"line_parse" and "parseSBinfo" were updated to remove extra 
	CR, LF, SPACE at trailer portion of strings.
2010-04-30
  "readskd" was updated not to confuse $STATWT to $STATIONS
2009-11-19
  "readskd_sked", "readskd_source", "readskd_station", "readskd",
  "SITE_ana", "STATION_ana", "SKED_ana", "SOUECE_ana", "DAS_ana", and
  "vex_read" were updated to know the true number of stations, 
  sources, scans, and data ack terminals in a schedule file even though
  the case of exceeding their allowded maximum defined in src/chead.h, 
  i.e.,  MAXSTAT, MAXSTAR, MAXOBS, and MAXDAS.
  Functions "sub_site_ana", "sub_station_ana", "sub_source_ana",
  "sub_sked_ana", and "sub_das_ana" were added.
2009-11-12
  Type 6 naming rule was added in "scan_file_naming"
2009-06-19
  "SITE_ana" was updated not to misunderstand a key word "site_position_epoch"
  to "site_position".
2007-09-26
  "readskd_freq" was modified to reflect sideband information 
  in a SKED file. A new function "parseSBinfo" was added to analyze 
  sideband info in a SKED file
2007-08-01
  allow "$EXTERN" in addition to "$NOZOMI" for external-delay model
  mode in "readskd_for_nozomi"
2006-02-17
  minor debug in "scan_file_naming". In naming type 5, station code
  was not reflected properly (lower case should be used).
2005-06-12
  libsked.h and libvex.h are merged into libskdvex.h.
  libskdvex.h is divided into libskdvex.h and libskdvex.c

*/
#include "libskdvex.h"


int line_parse(char *line, vparam_t *vparam)  //2013-01-29
{
/*
    Parse 1 line data into each item, where ' ' is separator.
    Total number of items is defined in
    type define statesment of vparam as

    typedef struct vparam
    {
        int  n;          //number of param
        char param[60][128];   // 60x128 characters
    } vparam_t;

    In this case 60 is a maximum number of items and
    128 characters are maximum char in an item.

    If runtime error occurrs, please try to increase the number of
    items in the vparam definition.

	2013-01-29  update to remove extra CR, LF, and SPACe at trailer.

*/
    //char abuf[500];
    char *abuf;   // 2020-06-26
    char *cset=" ";
    char *p=NULL;
    int k,len;
   
    len=strlen(line);
    if(len <= 0) return 0;

    abuf = malloc(_CBUF_SIZE); // 2020-06-26
    
	/* add on 2013-01-29 */
	while(len > 0){
	  if(line[len-1]=='\n' || line[len-1]=='\r' || line[len-1]==' '){
	 	  len--;
	  } else {
		  break;
	  }
	}
	line[len]='\0';
	/* add end */

    k=0;
    sprintf(abuf,"%s",strtok(line, cset));
    strcpy(vparam->param[k],abuf);  
    k++;
    while((p=strtok(NULL,cset))){
        sprintf(abuf,"%s",p);
        strcpy(vparam->param[k],abuf);  
        k++;
    }
    vparam->n=k;
    free(abuf); // 2020-06-26
    return k;
}

/* Monitor schedule file */
void SkdMonit(sked_data_t *skd)  //version 2019-06-12
{
    int i,n,k,gmax;
    int nst;  // 2019-06-12
    int year,ddd,hh,mm,ss,month,mday;
    int flag_star_ok, flag_baseid_ok;  // 2016-10-12
    char id1[3], id2[3];               // 2016-10-12
    char sideband[4], pol[4];  // 2019-06-12

    printf("SkdMonit: *********** Schedule file monitor **********\n");
    printf("SkdMonit: Schedule file = %s\n",skd->skdname);
    printf("SkdMonit: Expcode       = %s\n",skd->expcode);
    printf("SkdMonit: Total Scan #  = %d\n",skd->nobs);
    printf("SkdMonit: Total Star #  = %d\n",skd->nstar);
    printf("SkdMonit:\n");
    /* 1st and Last Observation Time Print */
    for (i=0; i<2; i++){
        if (i==0){
              k=0;
              printf("SkdMonit:  1st   Scan   = ");
        } else {
              k=skd->nobs-1; 
              printf("SkdMonit:  Last  Scan   = ");
        }
        year=skd->start[k][0]+2000;
        ddd=skd->start[k][1];
        hh=skd->start[k][2];
        mm=skd->start[k][3];
        ss=skd->start[k][4];
        ddd2md(year,ddd,&month,&mday);
        printf("%04d/%02d/%02d %02d:%02d:%02d  %s\n",
              year,month,mday,hh,mm,ss,skd->obsstar[k]);
    } 
    printf("SkdMonit:\n");
    printf("SkdMonit: ---------- Station ID Table ----------\n");
    for (i=0; i<skd->nstation ; i++){
          //printf("SkdMonit:  %s %s --- %s\n",skd->statid[i],skd->statid2[i],
          printf("SkdMonit:  %s --- %s\n",skd->statid[i],
              skd->statnam[i]);
    }
    if((strncmp(skd->skedtype,"VEX",3)==0)||
        (strncmp(skd->skedtype,"OVEX",4)==0)){
         printf("SkdMonit: (Note that 1 char station ID is not defined in a VEX file)\n");
         //printf("SkdMonit: (so 1 char station ID is generated by stcode_get function)\n");
    }
    printf("SkdMonit: --------------------------------------\n");

    /***  added on 2016-10-11 ***/
    printf("SkdMonit: ------- Star Table -------------------\n");
    printf("SkdMonit:          NAME1      NAME2  R.A.(deg)   DEC(deg)   EPOCH\n");
    for(i=0; i < skd->nstar; i++){
         printf("SkdMonit: %3d %10s %10s %10f %10f %10f\n",i+1,skd->starnam1[i],skd->starnam2[i],
                         skd->ra[i],skd->dec[i],skd->epo[i]);
    }
    printf("SkdMonit: --------------------------------------\n");
    /****** add end  *****/

/*
    printf("SkdMonit: ---------- Frequency Table -----------\n");
    printf("SkdMonit:   Gr# CH#  RF Freq(MHz)  PCAL Freq(Hz)\n");

    for(n=0; n<skd->nfreq; n++){           
        printf("SkdMonit:  %3d %3d   %10.2f     %8.1f\n",n/4+1,n+1,
                skd->freqtab[n],skd->pcalf[n]);
    }
    printf("SkdMonit: --------------------------------------\n");
*/
    gmax=(skd->nfreq-1)/4+1;
    nst = 0;   // station selection (1st statio)
    printf("SkdMonit: ------- Frequency Table (Freq MHZ, USB/LSB, Polarization)--------\n");
    for(i=0; i<gmax; i++){
        printf("SkdMonit: Gr# %2d ",i+1);
        for(k=0; k<4; k++){
            n=4*i+k;
            if(n >= skd->nfreq) break;
            //printf(" %9.2f %s",skd->freqtab[n],skd->sideband[n]);
            sideband[0] = '\0';                                              // 2019-06-12
            if (skd->st_freq[nst].ul[n] == 1) strcpy(sideband, "USB");       // 2019-06-12
            if (skd->st_freq[nst].ul[n] == 0) strcpy(sideband, "LSB");       // 2019-06-12
            strcpy(pol, skd->st_freq[nst].pol[n]);                           // 2019-06-12
            printf(" %9.2f %s %s",skd->st_freq[nst].freq[n],sideband,pol);   // 2019-06-12
        }
        printf("\n");
    }
    printf("SkdMonit: -----------------------------------------------------------------\n");
    printf("SkdMonit: ------- PCAL Freq (kHz) Table ---------\n");
    for(i=0; i<gmax; i++){
        printf("SkdMonit: Gr# %2d ",i+1);
        for(k=0; k<4; k++){
            n=4*i+k;
            if(n >= skd->nfreq) break;
            //printf(" %8.1f",skd->pcalf[0][n]/1.0e3);
            printf(" %8.1f",skd->st_freq[nst].pcalf[n][0]/1.0e3);     // 2019-06-12
        }
        printf("\n");
    }
    printf("SkdMonit: --------------------------------------\n");

    // added 2016-10-12
    if(skd->pickup_baseid[0] != '\0'){
        if(strlen(skd->pickup_baseid) == 4){
            id1[0]=skd->pickup_baseid[0]; id1[1]=skd->pickup_baseid[1]; id1[2]='\0';
            id2[0]=skd->pickup_baseid[2]; id2[1]=skd->pickup_baseid[3]; id2[2]='\0';
        } else {
            id1[0]=skd->pickup_baseid[0]; id1[1]='\0';
            id2[0]=skd->pickup_baseid[1]; id2[1]='\0';
        }
    }

    if(skd->pickup_source[0] != '\0' || skd->pickup_baseid[0] != '\0' ){
        printf("SkdMonit: --------- PICKUP SCAN TABLE ----------\n");
        printf("SkdMonit: SCAN#     SOURCE YYYY/DDD HH:MM:SS DURA  STATION_IDS\n"); 
        //printf("SkdMonit: -----   -------- -------- -------- ----  ---------------\n"); 
        for(i=0; i<skd->nobs; i++){
            // sorce name check 
            if(skd->pickup_source[0] == '\0'){
                flag_star_ok=1;
            } else {
                //if(strncmp(skd->pickup_source, skd->starnam1[skd->starid[i]], strlen(skd->pickup_source))==0 ||
                //   strncmp(skd->pickup_source, skd->starnam2[skd->starid[i]], strlen(skd->pickup_source))==0){
                if(strncmp_nocare(skd->pickup_source, skd->starnam1[skd->starid[i]], strlen(skd->pickup_source))==0 ||
                   strncmp_nocare(skd->pickup_source, skd->starnam2[skd->starid[i]], strlen(skd->pickup_source))==0){
                    flag_star_ok=1;
                } else {
                    flag_star_ok=0;
                }
            }
            // baseline ID check
            if(skd->pickup_baseid[0] == '\0'){
                flag_baseid_ok=1;
            } else {
                flag_baseid_ok=0;

                for(k=0; k<skd->nstation; k++){
                    if(skd->station_obs[i][k]==1){
                        if(strncmp(skd->statid[k],id1,strlen(id1))==0){
                            flag_baseid_ok++;
                            break;
                        }
                    }
                }
                for(k=0; k<skd->nstation; k++){
                    if(skd->station_obs[i][k]==1){
                        if(strncmp(skd->statid[k],id2,strlen(id2))==0){
                            flag_baseid_ok++;
                            break;
                        }
                    }
                }
                if(flag_baseid_ok==2){
                    flag_baseid_ok=1;
                } else {
                    flag_baseid_ok=0;
                }
            }

            if(flag_star_ok==1 && flag_baseid_ok==1){
                printf("SkdMonit: %5d %10s %4d/%03d %02d:%02d:%02d %4d ",
                    i+1, skd->obsstar[i],
                    2000+skd->start[i][0], skd->start[i][1],skd->start[i][2],skd->start[i][3],skd->start[i][4],
                    skd->dura[i]);
                // participated station IDs
                for(k=0; k<skd->nstation; k++){
                    if(skd->station_obs[i][k]==1) printf(" %s",skd->statid[k]);
                }
                printf("\n");
            }
        }
        printf("SkdMonit: --------------------------------------\n");
    }
}

void stcode_get(char *stcode_file, char *sid2, char *sid1)  // no more use
{     
/*   2003-07-13 minor update
       obtain 1char station code from 2 chr station code
    [in]
      stcode_file  -- station code file name ex. "stcode.txt"
      sid2         -- Two character station code
    [return]
      sid1         -- 1char station code accompanied by '\0' 
                   -- if No code found "-" returned
*/
    FILE *fin;
    //char line[60],s2[10],s1[10];
    char s2[10],s1[10];
    char *line; // 2020-06-26
    int k;

    line = malloc(_LINE_SIZE); // 2020-06-26
    strcpy(sid1,"-");

    if( (fin  = fopen(stcode_file, "r" )) == NULL ){
        printf("stcode_get: File (%s) not found or Open Error \n",stcode_file);
        printf("stcode_get:   Station code encode from 2 char to 1 char is not made for %s\n",sid2); //2003-07-13
        printf("stcode_get:   So 2 char code will be used for further processing\n"); //2003-07-13
        goto exit2return;
    } 

    //while(fgets(line, 60, fin ) != NULL) {
    while(fgets(line, _LINE_SIZE, fin ) != NULL) {  // 2020-06-26
        if(line[0] != '*'){
            k=sscanf(line, "%*s %s %s", s2,s1);
            //printf("stcode_get: %s %s\n",s2,s1);
            if(strncmp(sid2,s2,strlen(s2))==0){
                strcpy(sid1,s1);
                break;
            }
        }
    }
    k=fclose(fin);
    free(line);  // 2020-06-26
exit2return:;
}

void vex2sked(vex_t *vex, char* stcode_file, sked_data_t *skd)
{
    /*
      VEX structure to SKED structure data converter

      PCAL frequency of 10kHz is always supposed
        2003-03-21
        2016-10-12 revised

     [in]
       vex --  vex items
       stcode_file -- file name contain 2char station code to 1char station code
     [return]
       skd --  sked items

    */
    int i,n,k,m;
    int datastart, datastop, duramax;
    site_t site;
    //track_freq_t trfreq;

    strcpy(skd->skdname,vex->vex_file_name);
    strcpy(skd->expcode,vex->exper[0].exper_name);

    // station info set up

    skd->nstation=vex->nstation;
    for (n=0; n<vex->nstation ; n++){
        // find site info
        if(search_site(vex,vex->station[n].def,&site)==0){
            strcpy(skd->statid2[n], site.id);
            strcpy(skd->statid[n], site.id);
            stcode_get(stcode_file, site.id, skd->statid[n]); //2003-03-24
            //skd->statid[n][0]='\0';  //  for not to define 1 char station ID
            //skd->statid[n][1]='\0';
            strcpy(skd->statnam[n],site.name);
            for(i=0; i<3; i++){
                skd->xyz[n][i]=site.position[i];
            }
        } else {
            skd->statid2[n][0]='\0';
            skd->statid[n][0]='\0';
            skd->statnam[n][0]='\0';
            for(i=0; i<3; i++){
                skd->xyz[n][i]=0.0;
            }
        }
    }

    // scan info set up
    skd->nobs=vex->numobs;
    skd->nstar=vex->numstar;   // 2017-08-29
    for(n=0; n<vex->numobs; n++){
        for(i=0; i<5; i++){
            skd->start[n][i]=vex->scan[n].start[i];
        }
        skd->start[n][0]-=2000;   // definition in SKED file is 1 digit year
        strcpy(skd->obsstar[n],vex->scan[n].source);

        // find position in star table  2016-10-12
        skd->starid[n]=-1;
        for(i=0; i<skd->nstar; i++){
            if(strncmp(skd->obsstar[n],skd->starnam1[i],strlen(skd->obsstar[n]))==0 ||
                strncmp(skd->obsstar[n],skd->starnam2[i],strlen(skd->obsstar[n]))==0){
                    skd->starid[n]=i;
                    break;
            }
        }

        strcpy(skd->scanid[n],vex->scan[n].scanid); // 2005-01-13

//printf("OBS# %d  numstat %d  skdnstation  %d\n",n+1, vex->scan[n].numstat, skd->nstation);
        for(k=0; k<skd->nstation; k++){
            datastart=0; datastop=0; duramax=0;

            for(m=0; m< vex->scan[n].numstat; m++){
//printf("m= %d  %s  ",m,vex->scan[n].station[m].id);
                if(strncmp(vex->scan[n].station[m].id,skd->statid2[k],strlen(skd->statid2[k]))==0){
                    datastart=vex->scan[n].station[m].data_start;
                    datastop=vex->scan[n].station[m].data_stop;
//printf("datastart datastop %s %d %d\n",vex->scan[n].station[m].id,datastart,datastop);
                    break;
                }
            }
            skd->datastart[n][k]=datastart;
            skd->datastop[n][k] =datastop;
            if(datastop > duramax) duramax=datastop;
        }
        skd->dura[n]=duramax;  
    }

    //source info
    skd->nstar=vex->numstar;
    for(n=0; n<vex->numstar; n++){
        strcpy(skd->starnam1[n],vex->source[n].name);
        strcpy(skd->starnam2[n],vex->source[n].IAU_name);
        skd->dec[n]=vex->source[n].dec;
        skd->ra[n]=vex->source[n].ra;
        if(strncmp(vex->source[n].epoch,"J2000",5)==0){
           skd->epo[n]=2000.0;
        } else {
           skd->epo[n]=1950.0;
        }
    }

    //Frequency table temporaly set-up by using the 1st freq definition

    for(n=0; n<vex->freq_def[0].numch; n++){
        skd->freqtab[n]=vex->freq_def[0].chan_def[n].rf/1.0e6;  // MHz unit in SKED
        //skd->pcalf[0][n]=10000.0;
        skd->pcalf[n][0]=10000.0;  // 2019-12-21
    }
    skd->nfreq=vex->freq_def[0].numch;

}

void vex2sked2(vex_t *vex, sked_data_t *skd)
// 2004-06-17
// 2016-10-12  revised
// 2017-09-03 reivised
// 2019-06-12 reivised
{
    /*
      VEX structure to SKED structure data converter


     [in]
       vex --  vex items
     [return]
       skd --  sked items

    */
    int i,n,k,m,iok;
    int datastart, datastop, duramax;
    char mode[50];
    char cbuf[50];
    char wmode[20][50];
    int nmode;
    int modemax=20;
    site_t site;
    //freq_info_t freq_info;  // 2019-06-12
    //track_freq_t trfreq;

    strcpy(skd->skdname,vex->vex_file_name);
    strcpy(skd->expcode,vex->exper[0].exper_name);

    // station info set up

    skd->nstation=vex->nstation;
    for (n=0; n<vex->nstation ; n++){
        // find site info
        if(search_site(vex,vex->station[n].def,&site)==0){
            strcpy(skd->statid[n], site.id);
            strcpy(skd->statid2[n], site.id);
            strcpy(skd->statnam[n],site.name);
            for(i=0; i<3; i++){
                skd->xyz[n][i]=site.position[i];
            }
        } else {
            skd->statid[n][0]='\0';
            skd->statid2[n][0]='\0';
            skd->statnam[n][0]='\0';
            for(i=0; i<3; i++){
                skd->xyz[n][i]=0.0;
            }
        }
    }

    // check mode included in all scan
    nmode=0;
    if(vex->numobs <=0 ){  // in case of no scan
                           // 1st mode in $MODE definition is used
        printf("vex2sked2: Warning! No scan is included in VEX file.\n");
        strcpy(mode,vex->mode[0].def);
    } else {
        strcpy(mode,vex->scan[0].mode);
        strcpy(wmode[nmode],mode);
        nmode++;
        for(n=1; n<vex->numobs; n++){
            strcpy(cbuf,vex->scan[n].mode);
            iok=0;
            for(i=0; i<nmode; i++){
                if(strncmp(wmode[i],cbuf,strlen(cbuf))==0){
                    iok=-1;
                    break;
                }
            }
            if(iok ==0){
                strcpy(wmode[nmode],cbuf);
                nmode++;
            }
        }
        if(nmode > 1){ // 
            printf("vex2sked2: Warning! %d different MODEs are found in scans.\n",nmode);
            printf("vex2sked2:   1st mode (%s) will be used.\n",mode);
        }
    }


    //source info setup (moved from below)
    skd->nstar=vex->numstar;
    for(n=0; n<vex->numstar; n++){
        strcpy(skd->starnam1[n],vex->source[n].name);
        strcpy(skd->starnam2[n],vex->source[n].IAU_name);
        skd->dec[n]=vex->source[n].dec;
        skd->ra[n]=vex->source[n].ra;
        if(strncmp(vex->source[n].epoch,"J2000",5)==0){
           skd->epo[n]=2000.0;
        } else {
           skd->epo[n]=1950.0;
        }
    }


    // scan info set up
    skd->nobs=vex->numobs;
    skd->nstar=vex->numstar;   // 2017-08-29
    for(n=0; n<vex->numobs; n++){
        for(i=0; i<5; i++){
            skd->start[n][i]=vex->scan[n].start[i];
        }
        skd->start[n][0]-=2000;   // definition in SKED file is 1 digit year
        strcpy(skd->obsstar[n],vex->scan[n].source);

        // find position in star table  2016-10-12
        skd->starid[n]=-1;
        for(i=0; i<skd->nstar; i++){
            if(strncmp(skd->obsstar[n],skd->starnam1[i],strlen(skd->obsstar[n]))==0 ||
                strncmp(skd->obsstar[n],skd->starnam2[i],strlen(skd->obsstar[n]))==0){
                    skd->starid[n]=i;
                    break;
            }
        }


        strcpy(skd->scanid[n],vex->scan[n].scanid); // 2005-01-13
        duramax=0;        
        skd->dura[n]=duramax;  
        for(k=0; k<skd->nstation; k++){
            datastart=0; datastop=0; 
            skd->datastart[n][k]=datastart;
            skd->datastop[n][k] =datastop;
            skd->station_obs[n][k]=0;
            for(m=0; m< vex->scan[n].numstat; m++){
                if(strncmp(vex->scan[n].station[m].id,skd->statid[k],strlen(skd->statid[k]))==0){
                    datastart=vex->scan[n].station[m].data_start;
                    datastop=vex->scan[n].station[m].data_stop;
                    skd->datastart[n][k]=datastart;
                    skd->datastop[n][k] =datastop;
                    if(datastop > duramax) duramax=datastop;
                    if(datastop > 0) skd->station_obs[n][k]=1;
                    break;
                }
            }
        }
        skd->dura[n]=duramax;  
    }

    //Frequency table set-up using 1st site id in skd->statid table
    //  moved after X station is fixed, then use X station ID  2018-02-02
/*
    strcpy(sid,skd->statid[0]);

    for(n=0; n<skd->ch_max; n++) { 
        skd->freqtab[n]=0.0;
        //skd->sideband[n][0]='\0';
        strcpy(skd->sideband[n],"U");  // default
        for(i=0; i<10; i++) skd->pcalf[i][n]=0.0;
    }

    freq_info=vex_chinfo_get(vex,mode,sid,1);
    skd->nfreq=freq_info.chmax;
//printf("Chmax %d\n",skd->nfreq);
    for(n=0; n< skd->nfreq; n++){
        freq_info=vex_chinfo_get(vex,mode,sid,n+1);
        skd->freqtab[n]=freq_info.rf/1.0e6;  // MHz unit in SKED
        for(i=0; i<10; i++) skd->pcalf[i][n]=0.0;
        //printf("freq_info.npcal  %d\n",freq_info.npcal);
        for(i=0; i<freq_info.npcal; i++){
           skd->pcalf[i][n]=freq_info.pcalf[i]; 
           //printf("skd->pcalf  %d\n",skd->pcalf[i][n]);
        }
        strcpy(skd->sideband[n],freq_info.sideband);
    }

    setup_ch_alloc_table_by_station(vex, mode, skd);  // 2017-09-03
*/

    // add on 2019-06-12
    // set up st_freq data


    setup_ch_alloc_table_by_station(vex, vex->mode[0].def, skd);  // 1st mode is used for getting info
    skd->nfreq = skd->ch_max;

    setup_st_freq(skd, vex, vex->mode[0].def);
    /*
    for (n = 0; n < skd->nstation; n++) {
        freq_info = vex_chinfo_get(vex, vex->mode[0].def,  skd->statid[n], 1);
        skd->nfreq = freq_info.chmax;
        strcpy(skd->st_freq[n].sid, skd->statid[n]);
        skd->st_freq[n].nch = skd->nfreq;
        for (k = 0; k < skd->nfreq; k++) {
            freq_info = vex_chinfo_get(vex, vex->mode[0].def,  skd->statid[n], k+1);

            //skd->st_freq[n].freq[k] =  skd->freq_by_stat[n*skd->ch_max + k];
            skd->st_freq[n].freq[k] = freq_info.rf / 1.0e6;  // MHz unit in SKED

            for (i = 0; i < 10; i++) skd->st_freq[n].pcalf[k][i] = 0.0;
            for (i = 0; i < 10; i++) {
                //skd->st_freq[n].pcalf[k][i] = skd->pcalf[i][k];
                skd->st_freq[n].pcalf[k][i] = freq_info.pcalf[i];
            }
            //skd->st_freq[n].ul[k] = skd->ul_by_stat[n*skd->ch_max + k];
            skd->st_freq[n].ul[k] = 1;
            if(freq_info.sideband[0]=='L' || freq_info.sideband[0]=='l') skd->st_freq[n].ul[k] = 0;
            //strcpy(skd->st_freq[n].pol[k], skd->pol_by_stat[n*skd->ch_max + k]);
            strcpy(skd->st_freq[n].pol[k], freq_info.pol);
            //skd->st_freq[n].vbw[k] = skd->vbw_by_stat[n*skd->ch_max + k];  
            skd->st_freq[n].vbw[k] = freq_info.vbw;  
            //skd->st_freq[n].thread[k] = skd->thread_by_stat[n*skd->ch_max + k];   
            skd->st_freq[n].thread[k] = freq_info.thread;   
            skd->st_freq[n].ch[k] = freq_info.ch; // 2019-06-13
        }

    }
    */


}

void vex2sked3(vex_t *vex, char *stat_id, sked_data_t *skd)
// 2009-11-19
// 2016-10-12 revised
// 2017-08-29 debug  (skd->nstar is not fixed)
// 2017-09-03 reivised
// 2019-06-13 revised
{
    /*
      VEX structure to SKED structure data converter

      Ver up from vex2sked2
      Frequency table is set up for given station ID if it is not empty


     [in]
       vex --  vex items
       stat_id -- 2 char station ID  or NULL code
     [return]
       skd --  sked items

    */
    int i,n,k,m,iok,iok2;
    int datastart, datastop, duramax;
    char mode[50], sid[4], sid2[4];
    char cbuf[50];
    char wmode[20][50];
    int nmode;
    int modemax=20;
	int pickup_das_at_station; // index # in DAS table at station 2009-11-19
    site_t site;
    freq_info_t freq_info;
    //track_freq_t trfreq;

    strcpy(skd->skdname,vex->vex_file_name);
    strcpy(skd->expcode,vex->exper[0].exper_name);

	pickup_das_at_station=0; // 2009-11-19 if you want to pick others
	                         // consider this as a variable

    // station info set up

    skd->nstation=vex->nstation;
    for (n=0; n<vex->nstation ; n++){
        // find site info
        if(search_site2(vex,vex->station[n].def,&site)==0){
            strcpy(skd->statid[n], site.id);
            strcpy(skd->statid2[n], site.id);  //2005-02-03
            strcpy(skd->statnam[n],site.name);
            for(i=0; i<3; i++){
                skd->xyz[n][i]=site.position[i];
            }
        } else {
            skd->statid[n][0]='\0';
            skd->statid2[n][0]='\0';     //2005-02-03
            skd->statnam[n][0]='\0';
            for(i=0; i<3; i++){
                skd->xyz[n][i]=0.0;
            }
        }
    }

    // set station ID to pick up 
    // If given station ID is NULL or not match with IDs in a station ID table
    // 1st station in a station ID table will be used.

    sid[0]='\0';
    if(stat_id[0] != '\0'){ 
        // check station ID table as if ID is existed
        for(n=0; skd->nstation; n++){
            if(strncmp(skd->statid[n],stat_id,strlen(stat_id))==0){
                strcpy(sid,skd->statid[n]);  // ID is surely in a table 
                break;
            }
        }
    }
    if(sid[0]=='\0') strcpy(sid,skd->statid[0]);

    // Set tape early start parameter for given station ID 
    // (added on 2005-07-25)
    skd->earlysec=0;
    if(stat_id[0] != '\0'){ 
        for (n=0; n<vex->nstation ; n++){
            // find site info
            if(search_site2(vex,vex->station[n].def,&site)==0){
                if(strncmp(site.id,stat_id,strlen(stat_id))==0){
                    // DAS name is station[n].das
                    for(k=0; k<vex->ndas; k++){
                        //if(strncmp(vex->station[n].das,vex->das[k].def,strlen(vex->das[k].def))==0){
                        if(strncmp(vex->station[n].das[pickup_das_at_station],vex->das[k].def,strlen(vex->das[k].def))==0){ //2009-11-19
                           skd->earlysec=vex->das[k].earlysec;
                        }
                    }
                }
            }
        }
    }

    // check mode included in all scan for station ID = sid
    nmode=0;
    mode[0]='\0';
    //printf(" numobs %d\n",vex->numobs);
    if(vex->numobs <=0 ){  // in case of no scan
                           // 1st mode in $MODE definition is used
        printf("vex2sked3: Warning! No scan is included in VEX file.\n");
        strcpy(mode,vex->mode[0].def);
    } else {
        for(n=0; n<vex->numobs; n++){
            strcpy(cbuf,vex->scan[n].mode);
            iok=-1;
            // check this scan includes station ID sid and obs period not 0
            for(i=0; i<vex->scan[n].numstat; i++){
                strcpy(sid2,vex->scan[n].station[i].id);
                if(strncmp(sid2, sid, strlen(sid))==0){
                    //  station id is OK
                    if(vex->scan[n].station[i].data_stop > 0){
                       // scan length is greater than 0
                       iok=0;
                       break;
                    }
                }
            }
            if(iok==0){
                if(nmode ==0 ){
                   strcpy(mode,cbuf);
                   strcpy(wmode[nmode],mode);
                   nmode++;
                } else {
                   iok2=0;
                   for(i=0; i<nmode; i++){
                     if(strncmp(wmode[i],cbuf,strlen(cbuf))==0){
                        iok2=-1;
                        break;
                     }
                   }
                   if(iok2 ==0){
                       strcpy(wmode[nmode],cbuf);
                       nmode++;
                   } 

               } 
            }
        }
        if(nmode > 1){ // 
            printf("vex2sked3: Warning! %d different MODEs are found in scans.\n",nmode);
            printf("vex2sked3:   1st mode (%s) will be used.\n",mode);
        }
    }


    // scan info set up
    skd->nobs=vex->numobs;
    skd->nstar=vex->numstar;   // 2017-08-29
    for(n=0; n<vex->numobs; n++){
        for(i=0; i<5; i++){
            skd->start[n][i]=vex->scan[n].start[i];
        }
        skd->start[n][0]-=2000;   // definition in SKED file is 1 digit year
        strcpy(skd->obsstar[n],vex->scan[n].source);

        // find position in star table  2016-10-12

        skd->starid[n]=-1;
        for(i=0; i<skd->nstar; i++){
            if(strncmp(skd->obsstar[n],skd->starnam1[i],strlen(skd->obsstar[n]))==0 ||
                strncmp(skd->obsstar[n],skd->starnam2[i],strlen(skd->obsstar[n]))==0){
                    skd->starid[n]=i;
                    break;
            }
        }

        strcpy(skd->scanid[n],vex->scan[n].scanid); // 2005-01-13
        duramax=0;        
        skd->dura[n]=duramax;  
        for(k=0; k<skd->nstation; k++){
            datastart=0; datastop=0; 
            skd->datastart[n][k]=datastart;
            skd->datastop[n][k] =datastop;
            skd->station_obs[n][k]=0;
            for(m=0; m< vex->scan[n].numstat; m++){
                if(strncmp(vex->scan[n].station[m].id,skd->statid[k],strlen(skd->statid[k]))==0){
                    datastart=vex->scan[n].station[m].data_start;
                    datastop=vex->scan[n].station[m].data_stop;
                    skd->datastart[n][k]=datastart;
                    skd->datastop[n][k] =datastop;
                    if(datastop > duramax) duramax=datastop;
                    if(datastop > 0) skd->station_obs[n][k]=1;
                    break;
                }
            }
        }
        skd->dura[n]=duramax;  
    }

    //source info
    skd->nstar=vex->numstar;
    for(n=0; n<vex->numstar; n++){
        strcpy(skd->starnam1[n],vex->source[n].name);
        strcpy(skd->starnam2[n],vex->source[n].IAU_name);
        skd->dec[n]=vex->source[n].dec;
        skd->ra[n]=vex->source[n].ra;
        if(strncmp(vex->source[n].epoch,"J2000",5)==0){
           skd->epo[n]=2000.0;
        } else {
           skd->epo[n]=1950.0;
        }
    }


    for(n=0; n<16; n++) {   
        skd->freqtab[n]=0.0;
        //skd->sideband[n][0]='\0';
        strcpy(skd->sideband[n],"U");  
        //for(i=0; i<10; i++) skd->pcalf[i][n]=0.0;
        for(i=0; i<10; i++) skd->pcalf[n][i]=0.0; // 2019-12-21
    }

    // check mode 
/*    deleted on 2019-06-13
    freq_info=vex_chinfo_get(vex,mode,sid,1);
    skd->nfreq=freq_info.chmax;
    //printf("Chmax %d\n",skd->nfreq);
    for(n=0; n< skd->nfreq; n++){
        freq_info=vex_chinfo_get(vex,mode,sid,n+1);
        skd->freqtab[n]=freq_info.rf/1.0e6;  // MHz unit in SKED
        for(i=0; i<10; i++) skd->pcalf[i][n]=0.0;
        //printf("freq_info.npcal  %d\n",freq_info.npcal);
        for(i=0; i<freq_info.npcal; i++){
           skd->pcalf[i][n]=freq_info.pcalf[i]; 
           //printf("skd->pcalf  %d\n",skd->pcalf[i][n]);
        }
        strcpy(skd->sideband[n],freq_info.sideband);
    }


    setup_ch_alloc_table_by_station(vex, mode, skd);  // 2017-09-03
*/

    /* added on 2019-06-13 */
    // set up st_freq data


    setup_ch_alloc_table_by_station(vex, vex->mode[0].def, skd);  // 1st mode is used for getting info
    skd->nfreq = skd->ch_max;
    for (n = 0; n < skd->nstation; n++) {
        freq_info = vex_chinfo_get(vex, vex->mode[0].def, skd->statid[n], 1);
        skd->nfreq = freq_info.chmax;
        strcpy(skd->st_freq[n].sid, skd->statid[n]);
        skd->st_freq[n].nch = skd->nfreq;
        for (k = 0; k < skd->nfreq; k++) {
            freq_info = vex_chinfo_get(vex, vex->mode[0].def, skd->statid[n], k + 1);

            //skd->st_freq[n].freq[k] =  skd->freq_by_stat[n*skd->ch_max + k];
            skd->st_freq[n].freq[k] = freq_info.rf / 1.0e6;  // MHz unit in SKED

            for (i = 0; i < 10; i++) skd->st_freq[n].pcalf[k][i] = 0.0;
            for (i = 0; i < 10; i++) {
                //skd->st_freq[n].pcalf[k][i] = skd->pcalf[i][k];
                skd->st_freq[n].pcalf[k][i] = freq_info.pcalf[i];
            }
            //skd->st_freq[n].ul[k] = skd->ul_by_stat[n*skd->ch_max + k];
            skd->st_freq[n].ul[k] = 1;
            if (freq_info.sideband[0] == 'L' || freq_info.sideband[0] == 'l') skd->st_freq[n].ul[k] = 0;
            //strcpy(skd->st_freq[n].pol[k], skd->pol_by_stat[n*skd->ch_max + k]);
            strcpy(skd->st_freq[n].pol[k], freq_info.pol);
            //skd->st_freq[n].vbw[k] = skd->vbw_by_stat[n*skd->ch_max + k];  
            skd->st_freq[n].vbw[k] = freq_info.vbw;
            //skd->st_freq[n].thread[k] = skd->thread_by_stat[n*skd->ch_max + k];   
            skd->st_freq[n].thread[k] = freq_info.thread;

            skd->st_freq[n].ch[k] = freq_info.ch; // 2019-06-13
            
            //skd->st_freq[n].ch[k] = skd->ch_by_stat[n*skd->ch_max + k];  // 2020-10-10
        }

    }


    /* add end */
    // debug
    /*
    for(n=0; n< skd->nstation; n++){
        printf("Debug %10s  ",skd->statnam[n]);
        for(k=0; k <skd->nfreq; k++){
            printf(" %3d",skd->ch_by_stat[n*skd->ch_max+k]);
        }
        printf("\n");
    }
    */

}


void setup_ch_alloc_table_by_station_old(vex_t *vex, char *mode, sked_data_t *skd)
// 2017-09-03
// 2019-06-11
{
    /*
      set up channle allocation table in SKED type data

     [in]
       vex --  vex items
       mode ---  MODE
       skd->statid[]
     [return]
       skd->ch_by_stat[] --  channel allocation table
       skd->ul_by_stat[] --  USB/LSB (1/0) table
       skd->freq_by_stat[] --  Frequency table
       skd->pol_by_stat[] --  Polarization table  (2019-06-11)
*/

    int n,k,i,iok;
    char sid[4], freq_code[50], track_code[50], ch_id[10];
    char cbuf[60];
    //int track, ch, fanout;
    int ch;
    int nmode;
    int stab[64],mtab[64];
    int ul;    // 2018-02-02
    double freq; // 2018-02-02
    int flag2bit;
    int wmin,chmax;
    int ok_both;    // 2018-02-02

    // initialize ch_by_stat witn default
    for(k=0; k<skd->nstation; k++){
        for(n=0; n<skd->ch_max; n++){
           skd->ch_by_stat[k*skd->ch_max+n]=n+1;
           skd->ul_by_stat[k*skd->ch_max+n]=1;
           skd->freq_by_stat[k*skd->ch_max+n]=0.0;
           skd->pol_by_stat[k*skd->ch_max+n][0]='\0';  // 2019-06-11
        }
    }

    nmode=0;

    if(mode[0]!='\0'){  // find mode
        for (k=0; k<vex->nmode; k++){
            if(strncmp(vex->mode[k].def, mode, strlen(mode))==0) {
                nmode=k;
                break;
            }
        }
    }

    for(k=0; k<skd->nstation; k++){
        iok=0;
        ok_both=0;
        strcpy(sid,skd->statid[k]);   // get station ID
        // find entry point in MODE data   
        for(n=0; n<vex->mode[nmode].nmode; n++){
            //if(strncmp(vex->mode[nmode].mode_freq[n].id, sid,strlen(sid))==0){  //  found frequency code
            //    strcpy(freq_code, vex->mode[nmode].mode_freq[n].mode);   // get FREQ code
            //    strcpy(track_code, vex->mode[nmode].mode_track[n].mode);   // get TRACK code
            //    iok=1;
            //    break;
            //}
            if(strncmp( sid,vex->mode[nmode].mode_freq[n].id,strlen(vex->mode[nmode].mode_freq[n].id))==0){ //  found frequency code  // 2018-02-02
                strcpy(freq_code, vex->mode[nmode].mode_freq[n].mode);   // get FREQ code                 // 2018-02-02
                ok_both++;                                                                                // 2018-02-02
            }                                                                                             // 2018-02-02
            if(strncmp(sid,vex->mode[nmode].mode_track[n].id,strlen(vex->mode[nmode].mode_track[n].id))==0){  //  found track code     // 2018-02-02
                strcpy(track_code, vex->mode[nmode].mode_track[n].mode);   // get TRACK code              // 2018-02-02
                ok_both++;                                                                                // 2018-02-02
            }                                                                                             // 2018-02-02
        }
        //if(iok ==0) continue;
        if(ok_both != 2) continue;  // 2018-02-02


        // pick up frequency info

        for (n=0; n<vex->nfreq_def; n++){
            if(strncmp(freq_code, vex->freq_def[n].def, strlen(vex->freq_def[n].def))==0 &&
                strlen(freq_code)== strlen(vex->freq_def[n].def)){                            // 2018-02-02
                    for(i=0; i<vex->freq_def[n].numch; i++){
                         ul=-1;
                         freq=0.0;
                         strcpy(ch_id,vex->freq_def[n].chan_def[i].chan_id);
                         cbuf[0]=ch_id[3]; cbuf[1]=ch_id[4]; cbuf[2]='\0';       // pick up channel #
                         ch=atoi(cbuf);
                         freq=vex->freq_def[n].chan_def[i].rf;
                         ul=1;
                         if(vex->freq_def[n].chan_def[i].sideband[0]=='L') ul=0;  // LSB
                         skd->ul_by_stat[k*skd->ch_max+ch-1]=ul;
                         skd->freq_by_stat[k*skd->ch_max+ch-1]=freq;
                    }
            }
        }

        // pick up track 
        for(n=0; n<64; n++){
            stab[n]=-1; mtab[n]=-1;
        }
        flag2bit=0;
        wmin=99;
        chmax=0;
        for (n=0; n<vex->ntrack_def; n++){
            if(strncmp(track_code, vex->track_def[n].def, strlen(vex->track_def[n].def))==0 &&
                strlen(track_code)== strlen(vex->track_def[n].def)){
                if(strncmp("Mark4",vex->track_def[n].format,strlen(vex->track_def[n].format))==0){  // Mark 4 issupposed to be converted to Mark5B outside
                                                         // so pick up channel should be 1,2,3,.....
                                                         // so keep default setting
                    goto pass1;
                }

                // copy track table
                for(i=0; i<vex->track_def[n].nfanout_def; i++){
                    strcpy(ch_id,vex->track_def[n].fanout_def[i].chan_id);  // copy CH id
                    cbuf[0]=ch_id[3]; cbuf[1]=ch_id[4]; cbuf[2]='\0';       // pick up channel #
                    //KOKO wo Kansuni  to support &CH01 or & CHA01

                    ch=atoi(cbuf);
                    if(ch > chmax) chmax=ch;
                    if(strncmp(vex->track_def[n].fanout_def[i].stream_info,"mag",3)==0){  // 'mag'
                        mtab[ch-1]=vex->track_def[n].fanout_def[i].track[0];        // set track# of 'mag'
                        flag2bit=1;
                    } else { 
                        stab[ch-1]=vex->track_def[n].fanout_def[i].track[0];        // set track# of 'sign'
                    }
                    if(vex->track_def[n].fanout_def[i].track[0] < wmin) wmin=vex->track_def[n].fanout_def[i].track[0];
                }

                // debug
                /*
                printf("%s stab:",sid);                           
                for(i=0; i<chmax; i++) printf(" %d",stab[i]);
                printf("\n");
                printf("%s mtab:",sid);
                for(i=0; i<chmax; i++) printf(" %d",mtab[i]);
                printf("\n");
                */

                // convert track table to pick up channel position
                if(vex->track_def[n].fanout_def[0].fanout == 2){  // in case of fanout=2)
                   // nothing to do right now (use default)
                   if(flag2bit==1){     // 2 bit AD

                   } else {

                   }
                } else {     // fanout=1
                   if(flag2bit==1){    // 2bit AD
                       for(i=0; i<chmax; i++){
                            skd->ch_by_stat[k*skd->ch_max+i]=(stab[i]-wmin)/2 + 1;
                       }
                    } else {    // 1bit AD
                        for(i=0; i<chmax; i++){
                            skd->ch_by_stat[k*skd->ch_max+i]=(stab[i]-wmin) + 1;
                        }
                    }
               }
                continue;
            }


        }
pass1:;
            //debug
        /*
            printf("%s\n",sid);
            printf("FREQ ",sid);
            for(i=0; i<chmax; i++) printf(" %f",skd->freq_by_stat[k*skd->ch_max+i]);
            printf("\n");
            printf("U/L ",sid);
            for(i=0; i<chmax; i++) printf(" %d",skd->ul_by_stat[k*skd->ch_max+i]);
            printf("\n");
            printf("CHinWord ",sid);
            for(i=0; i<chmax; i++) printf(" %d",skd->ch_by_stat[k*skd->ch_max+i]);
            printf("\n");
        */

    }
}

void setup_ch_alloc_table_by_station(vex_t *vex, char *mode, sked_data_t *skd)
// 2017-09-03
// 2019-06-11
// 2020-10-10 revised
{
    /*
      set up channle allocation table in SKED type data

     [in]
       vex --  vex items
       mode ---  MODE
       skd->statid[]
     [return]
       skd->ch_by_stat[] --  channel allocation table
       skd->ul_by_stat[] --  USB/LSB (1/0) table
       skd->freq_by_stat[] --  Frequency table
       skd->pol_by_stat[] --  Polarization table  (2019-06-11)
*/

    int n, k, kk, i, ii, iok;
    char sid[4], freq_code[50], track_code[50], ch_id[10];
    char cbuf[60];
    //int track, ch, fanout;
    int ch;
    int nmode;
    int stab[64], mtab[64];
    int ul;    // 2018-02-02
    double freq; // 2018-02-02
    double vbw;  // 2019-06-12
    int flag2bit;
    int wmin, chmax;
    int ok_both;    // 2018-02-02
    sid_mode_t sid_mode; // 2019-06-11
    char bbcid[30];   // 2019-06-11
    char ifid[30];    // 2019-06-11
    int thread;       // 2019-06-11
    int *ch_table;  // 2020-10-10

    // initialize ch_by_stat witn default
    ch_table = (int *)malloc(skd->ch_max * sizeof(int));
    for (k = 0; k < skd->nstation; k++) {
        for (n = 0; n < skd->ch_max; n++) {
            skd->ch_by_stat[k*skd->ch_max + n] = n + 1;
            skd->ul_by_stat[k*skd->ch_max + n] = 1;
            skd->freq_by_stat[k*skd->ch_max + n] = 0.0;
            skd->pol_by_stat[k*skd->ch_max + n][0] = '\0';  // 2019-06-11
            skd->thread_by_stat[k*skd->ch_max + n] = 0;  // 2019-06-11
        }
    }

    nmode = 0;

    if (mode[0] != '\0') {  // find mode
        for (k = 0; k < vex->nmode; k++) {
            if (strncmp(vex->mode[k].def, mode, strlen(mode)) == 0) {
                nmode = k;
                break;
            }
        }
    }

    // debug
    //for (n = 0; n < vex->mode[nmode].nmode; n++) {
    //    printf("BBC   %d %s %s\n",n, vex->mode[nmode].mode_bbc[n].id, vex->mode[nmode].mode_bbc[n].mode);
    //    printf("FREQ  %d %s %s\n",n, vex->mode[nmode].mode_freq[n].id, vex->mode[nmode].mode_freq[n].mode);
    //    printf("IF    %d %s %s\n",n, vex->mode[nmode].mode_if[n].id, vex->mode[nmode].mode_if[n].mode);
    //    printf("TRACK %d %s %s\n",n, vex->mode[nmode].mode_track[n].id, vex->mode[nmode].mode_track[n].mode);
    //}

    for (k = 0; k < skd->nstation; k++) {
        iok = 0;
        ok_both = 0;
        strcpy(sid, skd->statid[k]);   // get station ID
        strcpy(sid_mode.sid, sid);
        strcpy(sid_mode.staion, skd->statnam[k]);

        // bbc mode finding for sid
        iok = -1;
        for (i = 0; i < vex->mode[nmode].nmode; i++) {
            if (strncmp(vex->mode[nmode].mode_bbc[i].id, sid, strlen(sid)) == 0) {  // MODE found
                strcpy(sid_mode.bbc_mode, vex->mode[nmode].mode_bbc[i].mode);
                iok = 0;
                break;
            }
        }
        if (iok < 0) continue;
        // freq mode finding for sid
        iok = -1;
        for (i = 0; i < vex->mode[nmode].nmode; i++) {
            if (strncmp(vex->mode[nmode].mode_freq[i].id, sid, strlen(sid)) == 0) {  // MODE found
                strcpy(sid_mode.freq_mode, vex->mode[nmode].mode_freq[i].mode);
                iok = 0;
                break;
            }
        }
        if (iok < 0) continue;
        // if mode finding for sid
        iok = -1;
        for (i = 0; i < vex->mode[nmode].nmode; i++) {
            if (strncmp(vex->mode[nmode].mode_if[i].id, sid, strlen(sid)) == 0) {  // MODE found
                strcpy(sid_mode.if_mode, vex->mode[nmode].mode_if[i].mode);
                iok = 0;
                break;
            }
        }
        if (iok < 0) continue;
        // track mode finding for sid
        iok = -1;
        for (i = 0; i < vex->mode[nmode].nmode; i++) {
            if (strncmp(vex->mode[nmode].mode_track[i].id, sid, strlen(sid)) == 0) {  // MODE found
                strcpy(sid_mode.tracks_mode, vex->mode[nmode].mode_track[i].mode);
                iok = 0;
                break;
            }
        }
        if (iok < 0) continue;


        iok = 0;
/*
        // find entry point in MODE data   
        for (n = 0; n < vex->mode[nmode].nmode; n++) {
            //if(strncmp(vex->mode[nmode].mode_freq[n].id, sid,strlen(sid))==0){  //  found frequency code
            //    strcpy(freq_code, vex->mode[nmode].mode_freq[n].mode);   // get FREQ code
            //    strcpy(track_code, vex->mode[nmode].mode_track[n].mode);   // get TRACK code
            //    iok=1;
            //    break;
            //}
            if (strncmp(sid, vex->mode[nmode].mode_freq[n].id, strlen(vex->mode[nmode].mode_freq[n].id)) == 0) { //  found frequency code  // 2018-02-02
                strcpy(freq_code, vex->mode[nmode].mode_freq[n].mode);   // get FREQ code                 // 2018-02-02
                ok_both++;                                                                                // 2018-02-02
            }                                                                                             // 2018-02-02
            if (strncmp(sid, vex->mode[nmode].mode_track[n].id, strlen(vex->mode[nmode].mode_track[n].id)) == 0) {  //  found track code     // 2018-02-02
                strcpy(track_code, vex->mode[nmode].mode_track[n].mode);   // get TRACK code              // 2018-02-02
                ok_both++;                                                                                // 2018-02-02
            }                                                                                             // 2018-02-02
        }
        //if(iok ==0) continue;
        if (ok_both != 2) continue;  // 2018-02-02
*/
        strcpy(freq_code, sid_mode.freq_mode);
        strcpy(track_code, sid_mode.tracks_mode);
        // pick up frequency info

        for (n = 0; n < vex->nfreq_def; n++) {
            if (strncmp(freq_code, vex->freq_def[n].def, strlen(vex->freq_def[n].def)) == 0 &&
                strlen(freq_code) == strlen(vex->freq_def[n].def)) {                            // 2018-02-02
                for (i = 0; i < vex->freq_def[n].numch; i++) {
                    ul = -1;
                    freq = 0.0;
                    thread = 0;
                    strcpy(ch_id, vex->freq_def[n].chan_def[i].chan_id);
                    //  ch_id   either "&CH01" or "&CHA01" type  <= 2019-06-11
                    if (strlen(ch_id) == 5) {  // "&CHXX" type  
                        cbuf[0] = ch_id[3]; cbuf[1] = ch_id[4]; cbuf[2] = '\0';       // pick up channel #
                        ch = atoi(cbuf);
                    } 
                    else if (strlen(ch_id) == 6) {  // "&CHAXX" type
                        cbuf[0] = ch_id[4]; cbuf[1] = ch_id[5]; cbuf[2] = '\0';       // pick up channel #
                        ch = atoi(cbuf);
                        if (ch_id[3] == 'A' || ch_id[3] == 'a') thread = 0;
                        if (ch_id[3] == 'B' || ch_id[3] == 'b') thread = 1;
                        if (ch_id[3] == 'C' || ch_id[3] == 'c') thread = 2;
                        if (ch_id[3] == 'D' || ch_id[3] == 'd') thread = 3;
                        //if (ch_id[3] == 'E' || ch_id[3] == 'e') thread = 4;
                        //if (ch_id[3] == 'F' || ch_id[3] == 'f') thread = 5;
                        //if (ch_id[3] == 'G' || ch_id[3] == 'g') thread = 6;
                        //if (ch_id[3] == 'H' || ch_id[3] == 'h') thread = 7;
                        ch += 16 * thread;       // for extend to 64ch otherwise ch repates every 16ch

                    }
                    freq = vex->freq_def[n].chan_def[i].rf;
                    vbw = vex->freq_def[n].chan_def[i].vbw;  // 2019-06-12
                    ul = 1;
                    if (vex->freq_def[n].chan_def[i].sideband[0] == 'L') ul = 0;  // LSB

                    skd->ul_by_stat[k*skd->ch_max + ch - 1] = ul;
                    skd->freq_by_stat[k*skd->ch_max + ch - 1] = freq;
                    skd->thread_by_stat[k*skd->ch_max + ch - 1] = thread;  // 2019-06-11
                    skd->vbw_by_stat[k*skd->ch_max + ch - 1] = vbw;  // 2019-06-12

                    skd->pol_by_stat[k*skd->ch_max + ch - 1][0] = '\0';
                    // get polarization information
                      // at 1st get BBC ID
                    strcpy(bbcid, vex->freq_def[n].chan_def[i].bbc_id);  // 2019-06-11
                      // then get  IF ID
                    ifid[0] = '\0';
                    for (ii = 0; ii < vex->nbbc; ii++) {
                        for (kk = 0; kk < vex->bbc[ii].nbbc; kk++) {
                            if (strncmp(bbcid, vex->bbc[ii].bbc_def[kk].bbc_id, strlen(bbcid)) == 0) {
                                strcpy(ifid, vex->bbc[ii].bbc_def[kk].if_id);
                            }
                        }
                    }
                       // then get polarization information
                    for (ii = 0; ii < vex->nifb; ii++) {
                        if (strncmp(vex->ifb[ii].def, sid_mode.if_mode, strlen(sid_mode.if_mode)) == 0) {  // MODE found
                            for (kk = 0; kk < vex->ifb[ii].nif; kk++) {
                                if (strncmp(vex->ifb[ii].if_def[kk].if_id, ifid, strlen(ifid)) == 0) {  // 2019-06-11
                                    strcpy(skd->pol_by_stat[k*skd->ch_max + ch - 1], vex->ifb[ii].if_def[kk].pol);
                                }

                            }
                        }
                    }
                }
            }
        }


        anal_track_then_conv_to_ch(track_code, vex, ch_table);     // 2020-10-10
        for (i = 0; i < skd->ch_max; i++) {                        // 2020-10-10
            if(ch_table[i] > 0)                                    // 2020-10-10
                skd->ch_by_stat[k*skd->ch_max + i] = ch_table[i];  // 2020-10-10
        }                                                          // 2020-10-10

        /*
        // pick up track 
        for (n = 0; n < 64; n++) {
            stab[n] = -1; mtab[n] = -1;
        }
        flag2bit = 0;
        wmin = 99;
        chmax = 0;
        for (n = 0; n < vex->ntrack_def; n++) {
            if (strncmp(track_code, vex->track_def[n].def, strlen(vex->track_def[n].def)) == 0 &&
                strlen(track_code) == strlen(vex->track_def[n].def)) {
                if (strncmp("Mark4", vex->track_def[n].format, strlen(vex->track_def[n].format)) == 0) {  // Mark 4 issupposed to be converted to Mark5B outside
                                                         // so pick up channel should be 1,2,3,.....
                                                         // so keep default setting
                    goto pass1;
                }

                // copy track table
                for (i = 0; i < vex->track_def[n].nfanout_def; i++) {
                    strcpy(ch_id, vex->track_def[n].fanout_def[i].chan_id);  // copy CH id
                    if (strlen(ch_id) == 5) {    // &CHXX
                        cbuf[0] = ch_id[3]; cbuf[1] = ch_id[4]; cbuf[2] = '\0';       // pick up channel #
                    }
                    else if (strlen(ch_id) == 6) { // &CHAXX
                        cbuf[0] = ch_id[4]; cbuf[1] = ch_id[5]; cbuf[2] = '\0';       // pick up channel #

                    }
                    ch = atoi(cbuf);
                    if (ch > chmax) chmax = ch;
                    if (strncmp(vex->track_def[n].fanout_def[i].stream_info, "mag", 3) == 0) {  // 'mag'
                        mtab[ch - 1] = vex->track_def[n].fanout_def[i].track[0];        // set track# of 'mag'
                        flag2bit = 1;
                    }
                    else {
                        stab[ch - 1] = vex->track_def[n].fanout_def[i].track[0];        // set track# of 'sign'
                    }
                    if (vex->track_def[n].fanout_def[i].track[0] < wmin) wmin = vex->track_def[n].fanout_def[i].track[0];
                }

                // debug
                // **
                printf("%s stab:",sid);
                for(i=0; i<chmax; i++) printf(" %d",stab[i]);
                printf("\n");
                printf("%s mtab:",sid);
                for(i=0; i<chmax; i++) printf(" %d",mtab[i]);
                printf("\n");
                // **

                // convert track table to pick up channel position
                if (vex->track_def[n].fanout_def[0].fanout == 2) {  // in case of fanout=2)
                   // nothing to do right now (use default)
                    if (flag2bit == 1) {     // 2 bit AD

                    }
                    else {

                    }
                }
                else {     // fanout=1
                    if (flag2bit == 1) {    // 2bit AD
                        for (i = 0; i < chmax; i++) {
                            skd->ch_by_stat[k*skd->ch_max + i] = (stab[i] - wmin) / 2 + 1;
                        }
                    }
                    else {    // 1bit AD
                        for (i = 0; i < chmax; i++) {
                            skd->ch_by_stat[k*skd->ch_max + i] = (stab[i] - wmin) + 1;
                        }
                    }
                }
                continue;
            }


        }
    pass1:;
        */

        //debug
        /*
        printf("%s\n",sid);
        printf("FREQ ");
        for(i=0; i<chmax; i++) printf(" %f",skd->freq_by_stat[k*skd->ch_max+i]);
        printf("\n");
        printf("U/L ");
        for(i=0; i<chmax; i++) printf(" %d",skd->ul_by_stat[k*skd->ch_max+i]);
        printf("\n");
        printf("CHinWord ");
        for(i=0; i<chmax; i++) printf(" %d",skd->ch_by_stat[k*skd->ch_max+i]);
        printf("\n");
        */

    }
    free(ch_table);  // 2020-10-10
}


/*
void skedlineana(char *line, skedline_t *skl)  // 2003-08-08
{
    int len,i,k,j;
    char cbuf[60], s1[21], fmt0[400], fmt[400];
    // get source info
    k=sscanf(line,"%s",skl->source);
    // get start time and max scan length
    k=sscanf(line, "%*s %*s %*s %*s %s %d", cbuf,&skl->dura); 
    strncpy(s1,cbuf,2);    s1[2]='\0';    //YY
    skl->start[0]=atoi(s1);   
    strncpy(s1,cbuf+2,3);  s1[3]='\0';    //DDD
    skl->start[1]=atoi(s1);   
    strncpy(s1,cbuf+5,2);  s1[2]='\0';    //HH
    skl->start[2]=atoi(s1);   
    strncpy(s1,cbuf+7,2);  s1[2]='\0';    //MM
    skl->start[3]=atoi(s1);   
    strncpy(s1,cbuf+9,2);  s1[2]='\0';    //SS
    skl->start[4]=atoi(s1);
    // participated station id get
    k=sscanf(line, "%*s %*s %*s %*s %*s %*s %*s %*s %*s %s", cbuf); 
    len=strlen(cbuf);
    skl->nstat=len/2;
    for(i=0; i<skl->nstat; i++){
       strncpy(skl->id_stat[i],cbuf+2*i,1);      //1character id is supposed
       skl->id_stat[i][1]='\0';
    }
    // scan length by station get
    strcpy(fmt,"%*s %*s %*s %*s %*s %*s %*s %*s %*s %*s");
    for(i=0; i<skl->nstat; i++){  // to skip footage field  
        strcat(fmt," %*s");
    }
    strcat(fmt," %*s");    // to skip prflags field
    strcpy(fmt0,fmt);
    for(i=0; i<skl->nstat; i++){ 
        strcpy(fmt,fmt0);
        for(j=0; j<i; j++) strcat(fmt," %*s");
        strcat(fmt," %s");
        k=sscanf(line, fmt, s1);
        skl->dura_stat[i]=atoi(s1);
    }
}
*/

void skedlineana(char *line, skedline_t *skl)  // debug on 2004-11-14
{
    char abuf[80];
    char *cset=" ";
    char *p=NULL;
    int i,k,j,len;
    char cbuf[60], pid[60], s1[21];

    cbuf[0]='\0';
    pid[0]='\0';
    skl->source[0]='\0';
    skl->dura=0;
    for(i=0; i<5; i++) skl->start[i]=0;
    skl->nstat=0;
    //for(i=0; i<MAXSTAT; i++) skl->dura_stat[i]=0;
    //for(i=0; i<MAXSTAT; i++) skl->id_stat[i][0]='\0';
    for(i=0; i<skl->nstat_max; i++) skl->dura_stat[i]=0;           // 2016-09-16
    for(i=0; i<skl->nstat_max; i++) skl->id_stat[i][0]='\0';       // 2016-09-16
   
    k=0;
    sprintf(abuf,"%s",strtok(line, cset));
    strcpy(skl->source,abuf);  // source info
    k++;
    while((p=strtok(NULL,cset))){
        sprintf(abuf,"%s",p);
        if(k==4) strcpy(cbuf,abuf);
        if(k==5) skl->dura=atoi(abuf);
        if(k==9) strcpy(pid,abuf);
        k++;
        if(k==10) break;
    }
    if(k > 4) {
       strncpy(s1,cbuf,2);    s1[2]='\0';    //YY
       skl->start[0]=atoi(s1);   
       strncpy(s1,cbuf+2,3);  s1[3]='\0';    //DDD
       skl->start[1]=atoi(s1);   
       strncpy(s1,cbuf+5,2);  s1[2]='\0';    //HH
       skl->start[2]=atoi(s1);   
       strncpy(s1,cbuf+7,2);  s1[2]='\0';    //MM
       skl->start[3]=atoi(s1);   
       strncpy(s1,cbuf+9,2);  s1[2]='\0';    //SS
       skl->start[4]=atoi(s1);
    }
    if(k > 9) {
       // participated station id get
       len=strlen(pid);
       skl->nstat=len/2;
       for(i=0; i<skl->nstat; i++){
          strncpy(skl->id_stat[i],pid+2*i,1);      //1character id is supposed
          skl->id_stat[i][1]='\0';
       } 
    }
    // scan length by station initialize by maximum scan length 
    //     just in case for length_by_station parameters are omitted 
    for(i=0; i<skl->nstat; i++){
        skl->dura_stat[i]=skl->dura;
    }

    // scan length by station get according to actual parameters if existed
    j=0;
    while((p=strtok(NULL,cset))){
        sprintf(abuf,"%s",p);
        //printf("*%d %s\n",k,abuf);
        if(k > 10+ skl->nstat){
            skl->dura_stat[j]=atoi(abuf);
            j++;
            if(j >= skl->nstat) break;     // added on 2004-11-14
        }
        k++;
    }
    //printf("k j nstat %d %d %d\n",k,j,skl->nstat);
}


int readskd_source(FILE *fin, char *line, sked_data_t *skd)
{
    int k;
    int i1,i2,i3,i4,i5;
    double d1,d2,d3,f3;
    char s1[21],s2[21],s3[21],s4[21];
    //char starnam[11],altnam[11];
    char starnam[MAX_SRC_NAME],altnam[MAX_SRC_NAME];    // 2017-08-28
	int reach_flag; //2009-11-19
    skd->nstar=0;
    skd->nstar_true=0; //2009-11-19
	reach_flag=0; //2009-11-19
             //printf("readskd: $SOURCES Detected! \n");
 loop:
    //if( fgets( line, 500, fin ) == NULL) return -1;
    if( fgets( line, _LINE_SIZE, fin ) == NULL) return -1;  // 2020-06-26
    if(line[0]=='*') goto loop;  // this is comment line (nothing to do)  
    if(line[0]=='$') return 0;  // this is command line  
    k=sscanf(line, "%s %s %d %d %s %s %d %s %s",    
                   starnam,altnam,&i1,&i2,s1,s3,&i4,s2,s4);     
    d1=atof(s1);
    d2=atof(s2);
    i3=atoi(s3);
    f3=atof(s4);
    //if(skd->nstar < MAXSTAR) { //2009-11-19
    if(skd->nstar < skd->nstar_max) { //2016-09-15
		//printf("%d %f %f\n",skd->nstar+1,d1,d2);
		strcpy(skd->starnam1[skd->nstar],starnam);
		strcpy(skd->starnam2[skd->nstar],altnam);
		d3=(double)i1*15.0+(double)i2/4.0 + d1/240.0 ;
		skd->ra[skd->nstar]=d3;
		i5=abs(i3);
		d3=(double)i5+(double)i4/60.0 + d2/3600.0 ;
		if (strncmp(s3,"-",1) == 0) d3 = -d3 ; 
		skd->dec[skd->nstar]=d3;
		skd->epo[skd->nstar]=f3;
		skd->nstar++;
	} else {
		if(reach_flag==0){
            //printf("readskd_source: Number of Stars reaches limit (%d)\n",MAXSTAR);
            printf("readskd_source: Number of Stars reaches limit (%d)\n",skd->nstar_max);  // 2016-09-15
			reach_flag=1;
		}
	}
    skd->nstar_true++; //2009-11-19
    goto loop;               
}

int readskd_station(FILE *fin, char *line, sked_data_t *skd)
{
    int i,k;
    int idct;
    int flag;
	int reach_flag; //2009-11-19
    double d1,d2,d3;
    char s1[21],s2[21],s3[21],cbuf[20],cbuf2[10];
    //char wstatnam[MAXSTAT][11];  // station name work
    //char wstatid[MAXSTAT][2];    // 1 char station id
    //char (*wstatnam)[11];  // station name work   2016-09-16
    char (*wstatnam)[MAX_STAT_NAME];  // station name work   2017-08-28
    char (*wstatid)[2];    // 1 char station id   2016-09-16
    idct=0;
    skd->nstation=0;
    skd->nstation_true=0; //2009-11-19
    flag=0;
	reach_flag=0; // 2009-11-19

    //wstatnam=malloc(skd->nstation_max*11);   // 2016-09-16
    wstatnam=malloc(skd->nstation_max*MAX_STAT_NAME);   // 2017-08-28
    wstatid =malloc(skd->nstation_max*2);    // 2016-09-16

 loop:
    //if( fgets( line, 500, fin ) == NULL) {
    if( fgets( line, _LINE_SIZE, fin ) == NULL) {  // 2020-06-26
        flag=-1;
        goto exitend;
    }
    if(line[0]=='*') goto loop;  // this is comment line (nothing to do)  
    if(line[0]=='$') goto exitend;  // this is command line  
    if(line[0]=='A') {    // 1 charcter Station ID set
        k=sscanf(line, "%*s %s %s", s1,s2); 
        //printf("readskd_station: ID NAME length %d %d\n",strlen(s1),strlen(s2));
		//if(idct < MAXSTAT){ // 2009-11-19
        if(idct < skd->nstation_max){ // 2016-09-16
           strcpy(wstatid[idct],s1);
           strcpy(wstatnam[idct],s2);
           idct++;
		}
    }
    if(line[0]=='P'){    // Station XYZ
        k=sscanf(line, "%*s %s %s %s %s %s", cbuf2,cbuf,s1,s2,s3); 
        d1=atof(s1);
        d2=atof(s2);
        d3=atof(s3);
		//if(skd->nstation < MAXSTAT){ // 2009-11-19
        if(skd->nstation < skd->nstation_max){ // 2016-09-15
			strcpy(skd->statid2[skd->nstation],cbuf2);  // 2 char station id
			strcpy(skd->statnam[skd->nstation],cbuf);   // station name
			skd->xyz[skd->nstation][0]=d1;
			skd->xyz[skd->nstation][1]=d2;
			skd->xyz[skd->nstation][2]=d3;
			skd->nstation++;
		} else {
			if(reach_flag==0){
               //printf("readskd_station: Number of stations reaches limit (%d)\n",MAXSTAT);
                printf("readskd_station: Number of stations reaches limit (%d)\n",skd->nstation_max);  // 2016-09-15
			   reach_flag=1;
			}
        }
        skd->nstation_true++; //2009-11-19
    }
    goto loop;               
exitend:

    /* Station ID table set up */
    // modified on 2004-11-14 to allow no station XYZ data
    if(skd->nstation > 0){  // added on 2004-11-14
        for (i=0; i < skd->nstation; i++){
           strcpy(cbuf,skd->statnam[i]);
           for (k=0; k < idct; k++){
               strcpy(s1,wstatnam[k]);
               if (strncmp(cbuf,s1,strlen(s1))==0){
                  //strncpy(skd->statid[i],' ',1);
                  strcpy(skd->statid[i],wstatid[k]);
                  break; 
               } 
           } 
        }
    } else {  // no "P" field data in sked
        skd->nstation=idct;
        for (i=0; i < skd->nstation; i++){
            strcpy(skd->statnam[i],wstatnam[i]);
            strcpy(skd->statid[i],wstatid[i]);
            //set position data zero because of no information
            skd->xyz[i][0]=0.0;
            skd->xyz[i][1]=0.0;
            skd->xyz[i][2]=0.0;
        } 
    }
    
    free(wstatnam);   // 2016-09-16
    free(wstatid);    // 2016-09-16

    return flag;
}

//void parseSBinfo(char *s, int *tus, int *tls)
void parseSBinfo(char *s, int *tus, int *tls, int *tum, int *tlm)  // 2016-08-15
{
 /*  
       2007-09-26
       2016-08-15 tum, tlm are added in calling sequence

   parse sideband information
        in a frequency field in SKED file


    $CODES
    F GEOSX4F  4F TSUKUB32 WETTZELL 
    C 4F X 8212.99  10000.0   1 Mk341:2  8.000 1(-1,3)  <= parse last field of this line
  
    [in]
       s  ---  P(tus,tls,tum,tlm) field in frequency info line
    [return]
       tus --  track # of USB sign bit  (-99 means no data)
       tls --  track # of LSB sign bit  (-99 means no data)
       tum --  track # of USB magnitude bit  (-99 means no data)
       tlm --  track # of LSB magnitude bit  (-99 means no data)


 */
        
    char cus[5],cls[5];
    char cum[5],clm[5];  // 2018-08-15
    int us,ls,len,n,k;
    int um, lm;          // 2016-08-15
    us=-99;  ls=-99;
    um=-99;  lm=-99;     // 2016-08-15
    cus[0]='\0';  cls[0]='\0';
    cum[0]='\0';  clm[0]='\0';  // 2016-08-15
   
    len=strlen(s);
    if(len <= 0) goto bypass;

	/* add on 2013-01-29 */
    // remove trailing character of ' ','\r','\n'
	while(len > 0){
	  if(s[len-1]=='\n' || s[len-1]=='\r' || s[len-1]==' '){
	 	  len--;
	  } else {
		  break;
	  }
	}
	s[len]='\0';
	/* add end */


    // remove leading characters till '(' inclusive
    k=0;
    while( (char)(*(s+k)) !='(' && k < len){
        k++;
    }
    k++;
    // copy USB track # data for sign bit
    n=0;
    while( ( (char)(*(s+k))!=',' && (char)(*(s+k))!=')' ) && k < len){
        cus[n]=s[k];
        n++;
        k++;
    }
    cus[n]='\0';
    k++;
    // copy LSB track # data for sign bit
    n=0;
    while( ( (char)(*(s+k))!=',' && (char)(*(s+k))!=')' ) && k < len){
        cls[n]=s[k];
        n++;
        k++;
    }
    cls[n]='\0';
    //
    if(cus[0] != '\0') us=atoi(cus);
    if(cls[0] != '\0') ls=atoi(cls);

    /* added on 2016-08-15 */
    k++;
    // copy USB track # data for magnitude bit
    n=0;
    while( ( (char)(*(s+k))!=',' && (char)(*(s+k))!=')' ) && k < len){
        cum[n]=s[k];
        n++;
        k++;
    }
    cum[n]='\0';
    k++;
    // copy LSB track # data for magnitude bit
    n=0;
    while( ( (char)(*(s+k))!=',' && (char)(*(s+k))!=')' ) && k < len){
        clm[n]=s[k];
        n++;
        k++;
    }
    clm[n]='\0';
    //
    if(cum[0] != '\0') um=atoi(cum);
    if(clm[0] != '\0') lm=atoi(clm);
    /* add end */

bypass:
    *tus=us; *tls=ls;
    *tum=um; *tlm=lm;    // 2016-08-16
}


int readskd_freq(FILE *fin, char *line, sked_data_t *skd) // 2013-01-30
{
    /*
        Ver. 2013-01-30  support multiple frequency tables
             2019-12-20  revised


        multiple table are stored in newly defined member in sked_data_t skd,
        that is, codes_t code, i.e., skd->code.
        skd->freqtab, skd->pcalf, skd->sideband are setup by the
        1st frequency table information to keep backward compatibility
    */

    int i, k;
    int ierr;


    if (skd->flag_code_new_anal == 1) { // adopt new analysis $CODES 2019-12-20
        ierr = anal_skd_codes(fin, line, &skd->code);   // 2019-12-20
        //mon_readskd_codes(&skd->code); // debug
    }
    else {
        ierr = readskd_codes(fin, line, &skd->code);
    }

	/* setup legacy data by the 1st freq table to keep backward compatibility */

    skd->nfreq=skd->code.ctab[0].nfreq;
	for(k=0; k<skd->nfreq; k++){
		skd->freqtab[k]=skd->code.ctab[0].freqtab[k];
		for(i=0; i<10; i++){
		    //skd->pcalf[i][k]=skd->code.ctab[0].pcalf[i][k];
		    skd->pcalf[k][i]=skd->code.ctab[0].pcalf[k][i];
		}
		strcpy(skd->sideband[k],skd->code.ctab[0].sideband[k]);
        strcpy(skd->pol[k], " ");       // 2019-06-11   polarization is not written in SKED file
	}

    // setup skd->st_freq data
    setup_sked_to_st_freq(skd);

    //mon_st_freq(skd);  // debug


	return ierr;
}

int readskd_sked(FILE *fin, char *line, sked_data_t *skd)
{
    int i,n;
	int reach_flag; //2009-11-19
    int nerr;  // 2016-09-16
    skedline_t skl;
    skd->nobs=0;
    skd->nobs_true=0; //2009-11-19
	reach_flag=0; // 2009-11-19

    skl.nstat_max=skd->nstation_max;   // 2016-09-16
    skl.id_stat=malloc(skl.nstat_max*3);    // 2016-09-16
    skl.dura_stat=(int *)malloc(skl.nstat_max*sizeof(int));  // 2016-09-16

    nerr=0;  // 2016-09-16

 loop:
    //if( fgets( line, 1000, fin ) == NULL) return -1;
    if( fgets( line, _LINE_SIZE, fin ) == NULL) { // 2020-06-26
        nerr=-1;
        goto exit2return; 
    }
    if(line[0]=='*') goto loop;  // this is comment line (nothing to do)  
    //if(line[0]=='$') return 0;  // this is command line  
    if(line[0]=='$') goto exit2return;  // this is command line    // 2016-09-16

    //if(skd->nobs < MAXOBS) { //2009-11-19
    if(skd->nobs < skd->nobs_max) { //2016-09-15

		//for(i=0; i<MAXSTAT; i++){
        for(i=0; i<skd->nstation_max; i++){  // 2016-09-15
			skd->station_obs[skd->nobs][i]=0;
			skd->datastart[skd->nobs][i]=0;
			skd->datastop[skd->nobs][i]=0;
		}
		//printf("Debug here %s\n",line);
		skedlineana(line, &skl);
		for(i=0; i<5; i++) skd->start[skd->nobs][i]=skl.start[i];
		strcpy(skd->obsstar[skd->nobs],skl.source);
		skd->dura[skd->nobs]=skl.dura;
		if(skd->nstation > 0){ //  then already station info set
			for(n=0; n<skl.nstat; n++){
				for(i=0; i<skd->nstation; i++){
					if(skl.id_stat[n][0]==skd->statid[i][0]){  // found id (1char)
						skd->station_obs[skd->nobs][i]=1;
						skd->datastop[skd->nobs][i]=skl.dura_stat[n];
						break;
					}
				}
			}
		}
		skd->nobs++;
	} else {
		if(reach_flag==0){
           //printf("readskd_sked: Number of scans reaches limit (%d)\n",MAXOBS);
           printf("readskd_sked: Number of scans reaches limit (%d)\n",skd->nobs_max);  // 2016-09-15
		   reach_flag=1;
		}
	}

    skd->nobs_true++; // 2009-11-19
    goto loop; 

exit2return:               // 2016-09-16
    free(skl.id_stat);     // 2016-09-16
    free(skl.dura_stat);   // 2016-09-16
    return nerr;           // 2016-09-16

}

int readskd_param(FILE *fin, char *line, sked_data_t *skd) //added on 2005-07-25
{
    int i,k;
    vparam_t vp;
    
    skd->earlysec=0;
loop:
    i=0;
    //if(fgets(line, 1000, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if(fgets(line, _LINE_SIZE, fin ) == NULL) return -1; //goto exitend;  2020-06-26
    if(line[0]=='*') goto loop;  // this is comment line (nothing to do)  
    if(line[0]=='$') return 0;  // this is command line  
    k=line_parse(line, &vp);
    if(k <=0 ) goto loop;
    if(strncmp(vp.param[0],"$",1)==0) return 0; //
    for(i=0; i<k-1; i++){ 
        if(strncmp(vp.param[i],"EARLY",5)==0) { //  pick up only early parameter
           skd->earlysec=atoi(vp.param[i+1]);
        }
    }
    goto loop;               
}

void readskd(sked_data_t *skd)  //2019-06-12
{
    //sked_data_t skd;
    //char line[1000];
    char *line;  // 2020-06-26
    char cbuf[21];
    char s1[21];
    //char* fname="D:/IPVLBI/data/tds16a.dat"; 
    int i, k;
    int len;
    int flag_sid; // flag to show already STATION field is read or not
    FILE *stream;

    //printf("%s \n",skd->skdname);  // debug

    if ((stream = fopen(skd->skdname, "r")) == NULL) {
        printf("readskd: File (%s) not found or Open Error! \n", skd->skdname);
        exit(1);
        //goto exitend2;
    }
    //printf("File %s opened\n",skd->skdname);

    line = malloc(_LINE_SIZE);  // 2020-06-26

    skd->nstar = 0;
    skd->nstation = 0;
    skd->nfreq = 0;
    skd->nobs = 0;
    flag_sid = 0;
    skd->earlysec = 0; //2005-07-25

    // initialize code data  2013-01-30
    skd->code.ncode = 0;                    //2013-01-30
    //for(i=0; i<MAXCODES; i++){            //2013-01-30
    for (i = 0; i < skd->code.ncode_max; i++) {            //2016-09-16
        skd->code.ctab[i].nfreq = 0;        //2013-01-30
        skd->code.ctab[i].nstation = 0;     //2013-01-30
    }                                     //2013-01-30
    skd->code.fsample = 0.0;                //2013-01-30
    skd->code.usb = skd->usb;               //2013-01-30


    skd->nstar_true = 0;    //2009-11-19
    skd->nstation_true = 0; //2009-11-19
    skd->nobs_true = 0;     //2009-11-19

    /* first step */
    // station info get first to reflect sub-net in each scan info

loop:
    //if (fgets(line, 1000, stream) == NULL) goto nextstep;
    if (fgets(line, _LINE_SIZE, stream) == NULL) goto nextstep;  // 2020-06-26
    /* Schedule File Analysis */
    //if(strncmp(line,"$STATIONS",5)==0){        // $STATIONS block   
    if (strncmp(line, "$STATIONS", 8) == 0) {        // $STATIONS block   2010-04-30
       //printf("readskd: $STATIONS Detected! \n");
        i = readskd_station(stream, line, skd);
        goto nextstep;
    }
    goto loop;

nextstep:
    rewind(stream);

loop1:
    //if (fgets(line, 1000, stream) == NULL) goto exitend;
    if (fgets(line, _LINE_SIZE, stream) == NULL) goto exitend;  // 2020-06-26
    /* Schedule File Analysis */
    if (line[0] == '*') goto loop1;  // this is comment line (nothing to do)  

    if (line[0] == '$') {    // this is command line  
    loop2:
        len = strlen(line);
        if (strncmp(line, "$EXPER", 5) == 0) {        // $EXPER block   
            k = sscanf(line, "%s %s", cbuf, skd->expcode);
            //printf("readskd: $EXPER Detected! \n");
            //printf("readskd: EXPCODE = %s\n",skd->expcode);
            goto loop1;
        }
        if (strncmp(line, "$PARAM", 5) == 0) {       // $PARAM block  
            /* pickup only "EARLY" parameter */
            if (readskd_param(stream, line, skd) < 0) goto exitend;
            goto loop2;
        }
        if (strncmp(line, "$SOURCES", 5) == 0) {       // $SOURCES block   
           //printf("readskd: $SOURCES Detected! \n");
            if (readskd_source(stream, line, skd) < 0) goto exitend;
            goto loop2;
        }
        //       if(strncmp(line,"$STATIONS",5)==0){        // $STATIONS block   
        //          //printf("readskd: $STATIONS Detected! \n");
        //          if(readskd_station(stream, line, skd) < 0) goto exitend;
        //          goto loop2;
        //       }
        if (strncmp(line, "$CODES", 5) == 0) {       // $CODES block (FREQ info)   
           //printf("readskd: $CODES Detected! \n");
            if (readskd_freq(stream, line, skd) < 0) goto exitend;
            //mon_readskd_codes(&skd->code);   // debug 2019-12-19
            //mon_st_freq(skd);  // debug 2019-12-21

            goto loop2;
        }
        if (strncmp(line, "$SKED", 5) == 0) {       // $SKED block 
           //printf("readskd: $SKED Detected! \n");
            if (readskd_sked(stream, line, skd) < 0) goto exitend;
            goto loop2;
        }
        if (strncmp(line, "$HEAD", 5) == 0) {       // head stack information  
            //printf("$HEAD Detected! \n");
            goto loop1;
        }
        if (strncmp(line, "$FLUX", 5) == 0) {       // FLUX info   
            //printf("$FLUX Detected! \n");
            goto loop1;
        }
        if (strncmp(line, "$PROC", 5) == 0) {       // station procedure info   
            //printf("$PROC Detected! \n");
            goto loop1;
        }
    }
    goto loop1;
exitend:
    i = fclose(stream);
    free(line);  // 2020-06-26
    //printf("%d bytes read\n",pos); 

    //printf("readskd: Number of Sources = %d\n",skd->nstar);
    //printf("readskd: Number of Stations = %d\n",skd->nstation);
    //printf("readskd: Number of Observations = %d\n",skd->nobs);

    /* Obs Star Index Table Setup */
    if (skd->nstar > 0) {
        for (i = 0; i < skd->nobs; i++) {
            strcpy(cbuf, skd->obsstar[i]);
            for (k = 0; k < skd->nstar; k++) {
                strcpy(s1, skd->starnam1[k]);
                len = strlen(s1);
                //printf("len = %d\n",len);
                if (strncmp(cbuf, s1, len) == 0) {
                    skd->starid[i] = k;
                    goto exithere;
                }
                strcpy(s1, skd->starnam2[k]);
                len = strlen(s1);
                if (strncmp(cbuf, s1, len) == 0) {
                    skd->starid[i] = k;
                    goto exithere;
                }
            }
            skd->starid[i] = -1;
        exithere:;
        }
    }

    // add on 2017-09-01
    // set default CH allocation by station    (2017-09-01)
    for (i = 0; i < skd->nstation_max; i++) {
        for (k = 0; k < skd->ch_max; k++) {
            skd->ch_by_stat[i*skd->ch_max + k] = k + 1;
        }
    }

    // add on 2019-06-12
    // set up st_freq data (obsoleted no more use because done in read_freq function
    // obsoleted on 2019-12-21
    /*
    for (n = 0; n < skd->nstation_max; n++) {
        for (k = 0; k < skd->nfreq; k++) {
            skd->st_freq[n].freq[k]=skd->freqtab[k];
            for (i = 0; i < 10; i++) {
                skd->st_freq[n].pcalf[k][i]=skd->pcalf[i][k];
            }
            skd->st_freq[n].ul[k] = 1;
            if(skd->sideband[k][0]=='L' || skd->sideband[k][0]=='l' ) skd->st_freq[n].ul[k] = 0;
            skd->st_freq[n].pol[k][0]='\0';       //  polarization is not written in SKED file
            skd->st_freq[n].vbw[k] = skd->code.ctab[0].vbw[k];
            skd->st_freq[n].thread[k] = 0;   // default 
            skd->st_freq[n].ch[k] = k+1;  
        }

    }
    */
         
/*
       printf("readskd: Number of Observations = %d\n",skd->nobs);
       
       for (i=0; i < 10; i++)
       {
          printf("%d %s %d %d %d %d %d %d   %d %s\n",i+1,skd->obsstar[i],skd->starid[i],
                 skd->start[i][0],skd->start[i][1],skd->start[i][2],
                 skd->start[i][3],skd->start[i][4],skd->dura[i],skd->obsid[i]);
       }
       for (i=skd->nobs-10; i < skd->nobs; i++)
       {
          printf("%d %s %d %d %d %d %d %d   %d %s\n",i+1,skd->obsstar[i],skd->starid[i],
                 skd->start[i][0],skd->start[i][1],skd->start[i][2],
                 skd->start[i][3],skd->start[i][4],skd->dura[i],skd->obsid[i]);
       }
 */      
 //exitend2:

}

void readskd_for_nozomi(char *skdname, char *baseid, char *apefile, int *iok)  
//2003-04-12   to obtain NOZOMI apriori file name
//    iok =-1  not found in schedule file
{
    //sked_data_t skd;
    //char line[500];
    char *line;   // 2020-06-26
    char s1[10];
    char s2[80];
    //char* fname="D:/IPVLBI/data/tds16a.dat"; 
    int i,k;
    int len;
    //char timelab[30];
    //int sfreq;
    FILE *stream;

    *iok=-1;
    apefile[0]='\0';

    if( (stream  = fopen(skdname, "r" )) == NULL ){
       printf("readskd_for_nozomi: File (%s) not found or Open Error! \n",skdname);
       exit(1);
       //goto exitend2;
    }

    line = malloc(_LINE_SIZE);  // 2020-06-26

     loop1:

     //if( fgets( line, 500, stream ) == NULL){
     if( fgets( line, _LINE_SIZE, stream ) == NULL){   // 2020-06-26
       //printf("File EOF or Read Error!\n");
        goto exitend;
     }
     /* Schedule File Analysis */    
     if(line[0]=='*') goto loop1;  // this is comment line (nothing to do)  
       
     if(line[0]=='$') {    // this is command line  
          len = strlen(line);
          //if(strncmp(line,"$NOZOMI",7)==0) {       // $NOZOMI block   
          if((strncmp(line,"$NOZOMI",7)==0)||(strncmp(line,"$EXTERN",7)==0)) { // 2007-08-01   
              //printf("readskd_for_nozomi: $NOZOMI Detected! \n");
  loop_nozomi:
              //if( fgets( line, 500, stream ) == NULL) goto exitend;
              if( fgets( line, _LINE_SIZE, stream ) == NULL) goto exitend; // 2020-06-26
              if(line[0]=='*')  goto loop_nozomi;  // this is comment line (nothing to do)  
              //if(line[0]=='$')  goto loop2;  // this wil be next comment line  
              if(line[0]=='$') goto exitend;  // this wil be next comment line  
              k=sscanf(line, "%s %s", s1,s2); 
              if(strncmp(baseid,s1,strlen(baseid))==0) {  // baseline info found
                  strcpy(apefile,s2);
                  *iok=0;
                  goto exitend;
              }
              goto loop_nozomi;               
           }
     }
     goto loop1;
 exitend:
     i=fclose( stream ) ;
     free(line);   // 2020-06-26
}

/* check schedule file type */
void check_sked_type(char* skdname, char* skedtype)  // 2003-03-21
{
    /*
      check schedule file type by reading 1st line
    */
    int i;
    //char cbuf[60];
    char *line;  // 2020-06-26
    char atype[][6]={"SKED","VEX","OVEX","OTHER"};

    FILE *fin;
    if((fin  = fopen(skdname, "r" )) == NULL ){
       printf("check_sked_type: File (%s) not found or Open Error! \n",skdname);
       exit(1);
    }

    line = malloc(_LINE_SIZE); // 2020-06-26
    // 1st line read
    //if( fgets(cbuf, 60, fin) == NULL){
    if( fgets(line, _LINE_SIZE, fin) == NULL){  // 2020-06-26
        i=fclose(fin);
        goto exitend;
    }
    i=fclose(fin);
    // first line analysis */  
    strcpy(skedtype,atype[3]); // other type
    if(strncmp(line,"$EXPER",6)==0) strcpy(skedtype,atype[0]);  // original schedule file (SKED) type
    if(strncmp(line,"VEX",3)==0)  strcpy(skedtype,atype[1]);     // VEX
    if(strncmp(line,"$OVEX",5)==0)  strcpy(skedtype,atype[2]);   // OVEX     
exitend:;
    free(line); // 2020-06-26
}


void pickup_skd(sked_data_t *skd, char *stat_id)
// 2005-06-16
{
    /*
      Pickup scans which includes stat_id

     [in]
       skd --  sked items
       stat_id -- 2 char sattion ID  or NULL code
     [return]
       skd --  sked items  (pickuped scans)

    */
    int i,n,k,m,kid;
    int numobs;
    int *worktabl;   // work table to contain obs index#

    //worktabl = (int *)malloc(MAXOBS*sizeof(int));
    worktabl = (int *)malloc(skd->nobs_max*sizeof(int));  // 2016-09-15

    // get ID number for stat_id
    kid=0;
    for(n=0; n<skd->nstation; n++){
        if(strncmp(skd->statid[n],stat_id,strlen(stat_id))==0){
            kid=n;
            break;
        }
    }

    numobs=0;
    for(n=0; n<skd->nobs; n++){
       //check station id
        if(skd->station_obs[n][kid]==1){ // stat_id is included in this scan
            worktabl[numobs]=n;
            numobs++;
        }
    }

    // pick up data
    for(n=0; n<numobs; n++){
        m=worktabl[n];
        strcpy(skd->obsstar[n],skd->obsstar[m]);
        strcpy(skd->obsid[n],skd->obsid[m]);
        strcpy(skd->scanid[n],skd->scanid[m]);
        skd->starid[n]=skd->starid[m];  
        skd->dura[n]=skd->dura[m];  
        for(i=0; i<5; i++){
            skd->start[n][i]=skd->start[m][i];
        }
        for(k=0; k<skd->nstation; k++){
            skd->datastart[n][k]=skd->datastart[m][k];
            skd->datastop[n][k] =skd->datastop[m][k];
            skd->station_obs[n][k]=skd->station_obs[m][k];
        }
    }
    skd->nobs=numobs;

    free(worktabl);
}


void setscanid_skd(sked_data_t *skd)
// 2005-06-17
{
    /*
      Set scan id  'ddd-hhmm'
      if multi scans are in same minutes, then
      first one is ddd-hhmma, then next one ddd-hhmmb, .....

     [in]
       skd --  sked items
     [return]
       skd --  sked items  

    */
    char cwork[21];
    char a[2];
    int n;
    int mnext,mold;

    mold=-1;
    //a[0]=0x61;   // 'a'
    a[0]='a';  
    a[1]='\0';
    for(n=0; n<skd->nobs; n++){
        if(skd->start[n][3] != mold){ // new minutes start
            a[0]='a';
        } 
        if(n < skd->nobs-1){
            mnext=skd->start[n+1][3];// minutes of next scan
        } else {
            mnext=-1;
        }
        sprintf(cwork,"%03d-%02d%02d",
            skd->start[n][1],skd->start[n][2],skd->start[n][3]);
        if(skd->start[n][3]==mnext || skd->start[n][3]==mold){  // add 'a','b','c'....
            strcat(cwork,a);
            a[0]++;
        } else {
            a[0]='a';
        }
        strcpy(skd->scanid[n],cwork);
        mold=skd->start[n][3];
    }

}

void scan_file_naming(obs_file_naming_t *ofn, char *fname)
{
/*  Definition of out file name     (revised on 2009-11-12)
          
            Type I or -I

              XDDDNNNN.[#ch.]dat   
               where X    -- satation id (1 char) 
                     DDD  -- total day at 1st scan (3 digits)
                     NNNN -- obs number  (4digits)
                     #ch  -- # of channels included (1 or 4)(for Type '-I')

            Type II or -II

              sidDDDHHMMSSG.[#ch.]dat
                where sid   -- station id  (1 char or 2 char)
                      DDD   -- total day at current scan (3digits)
                      HH    -- hour at the start of scan (2digits)
                      MM    -- minute at the start of scan (2digits)
                      SS    -- second at the start of scan (2 digits)
                      G     -- frequency group id (a|b|c|d)
                      #ch   -- # of channels included (1 or 4)(for Type '-II')

            Type III

              expid_sidG_scanid_YYYYDDDHHMMSS.k5
                where expid  -- experiment code
                      sid    -- station ID (2 lower-case characters)
                      G      -- PC id  1|2|3|4  or null
                      scanid -- scan id
                      YYYY   -- year (4digits)
                      DDD    -- total day at current scan (3digits)
                      HH     -- hour at the start of scan (2digits)
                      MM     -- minute at the start of scan (2digits)
                      SS     -- second at the start of scan (2 digits)

            Type V   (for JIVE)
                  
             expid_scan#.stcode.k5a(-d)
			 
            Type VI   (new)
			 
			 expid_sid_scanid.k5a(-d) 

*/
    

       if(ofn->naming_type == 1){  // naming type I  
           sprintf(fname,"%1s%03d%04d.dat",ofn->statid,ofn->tf.ddd,ofn->obsnum);  
       } else if(ofn->naming_type == -1){  // naming type '-I'  
           sprintf(fname,"%1s%03d%04d.%d.dat",ofn->statid,ofn->tf.ddd,ofn->obsnum,ofn->numch);  
       } else if (ofn->naming_type ==2){               // naming type II
           sprintf(fname,"%s%03d%02d%02d%02d%s.dat",
               ofn->statid,ofn->tn.ddd,ofn->tn.hh,ofn->tn.mm,ofn->tn.ss,ofn->freqg);
       } else if (ofn->naming_type ==-2){               // naming type '-II'
           sprintf(fname,"%s%03d%02d%02d%02d%s.%d.dat",
               ofn->statid,ofn->tn.ddd,ofn->tn.hh,ofn->tn.mm,ofn->tn.ss,ofn->freqg,ofn->numch);
       } else if (ofn->naming_type ==3){               // naming type 'III'
           sprintf(fname,"%s_%s%s_%s_%04d%03d%02d%02d%02d.k5",
               ofn->expcode,ofn->statid2,ofn->pcid,ofn->scanid,
               ofn->tn.year,ofn->tn.ddd,ofn->tn.hh,ofn->tn.mm,ofn->tn.ss);
	   } else if (ofn->naming_type ==6){  // naming type 6   //2009-11-12
           sprintf(fname,"%s_%s_%s.k5%s",
               ofn->expcode,ofn->statid2,ofn->scanid,ofn->freqg); 
       } else {  // naming type V  
           sprintf(fname,"%s_%d.%s.k5%s",
               // ofn->expcode,ofn->obsnum,ofn->statid,ofn->freqg);
               ofn->expcode,ofn->obsnum,ofn->statid2,ofn->freqg);  //2006-02-17
       }

}
    
/* merge libvex.c */

/*******************************************************************/
/*     library for VEX file analysis                               */
/*      by T.Kondo for Mark5 to K5 conversion and others           */
/*                                                                 */
/*        Ver. 1.25  2005-01-13                                    */
/*                                                                 */
/*      Copyright (c) 2003-2005 T.Kondo/NICT All Right Reserved    */
/*******************************************************************/

/*

2005-01-13
   char site[20] was added as a member of sidtable type variable

2004-12-25
    "search_track_freq" function was modified to include bbc info
    to support K5 to Mark5 conversion

2004-09-13
    int headstack was added as a member of track_freq_sub structual valiabl.
    head stack info is added in "search_track_freq" function

2004-06-29
    sub_pass info was added in a structure variable track_freq_sub.
    "search_track_freq" function was modified to include sub_pass info
    in track_freq structure variable.

2004-06-17 
    "search_site" : station name was added in No Clock info message.
    "PCAL_ana" :  debug in no pcal case

2003-10-28 
    Algorithm calculating PCAL frequency was updated in 
    "vex_freq_get" function.
    PCAL_ana was modified not to included tone# 0 in tone number table

2003-10-12 
    # of tracks was increased to support 64 track mode
        in  tracks structure and track_freq structure
    TRACKS_ana -- allow omittion of "sub-pass ID" in fanout_def parameters

2003-07-17
    BBC_ana -- $BBC block analysis was added
    BBC_mon -- $BBC block monitor was added
    IF_ana  -- $IF block analysis was added
    IF_mon  -- $IF block monitor was added
    PCAL_ana  -- $PCAL block analysis was added
    PCAL_mon  -- $PCAL block monitor was added

   in "typedef struct track_freq_sub"
     double pcalf[10];     // PCAL frequency in video (up to 10th tone) 2003-07-16
     int npcal;            //       valid number of data in pcalf
     were added as members

2003-07-13  
    eop_get, eop_get2, sa_clock_get, clock_get are added

2003-07-12  debug in "search_track_freq"
2003-07-04  "typedef struct vex"
              the number of $FREQ block is incresed from 5 to 20
              as follows
              //freq_t freq_def[5];        // $FREQ block info
              freq_t freq_def[20];        // $FREQ block info   2003-07-04
*/



double clock_get(siteclock_t clock, int *prt)  //2003-07-13
{
    // get clock parameter at PRT
    //   IN
    //      clock --  site clock parameter in VEX file
    //      PRT[5]   --  PRT
    double coffset,t1,t2;
    coffset=0.0;
    t1=(double)clock.clock_epoch[1]*86400.0   // DDD
        +(double)clock.clock_epoch[2]*3600.0  // HH
        +(double)clock.clock_epoch[3]*60.0    // MM
        +(double)clock.clock_epoch[4]*1.0;     // SS
    t2=(double)prt[1]*86400.0   // DDD
        +(double)prt[2]*3600.0  // HH
        +(double)prt[3]*60.0    // MM
        +(double)prt[4]*1.0;     // SS
    coffset=(t2-t1)*clock.clock_rate + clock.clock_offset;
    return coffset;
}

void sa_clock_get(vex_t *vex, char *stat_id1, char *stat_id2, int *prt, 
                  double *coffset, double *crate, double *xcoff)  //2005-05-19
{
    // get clock parameter at PRT
    //   IN
    //      clock --  site clock parameter in VEX file
    //      PRT[5]   --  PRT
    int i,k1,k2;
    char cw1[10],cw2[10];
    double o12, r12,o1,o2,r1,r2;
    strcpy(cw1,"--");
    strcpy(cw2,"--");
    k1=-1; k2=-1;
    o1=0.0; o2=0.0;
    r1=0.0; r2=0.0;
    o12=0.0; r12=0.0;
    //find station key for stat_id1
    for(i=0; i<vex->nstation; i++){
       if(strncmp(stat_id1,vex->station[i].def,strlen(stat_id1))==0){
            strcpy(cw1,vex->station[i].clock);
            break;
       }
    }
    for(i=0; i<vex->nclock; i++){
       if(strncmp(cw1,vex->siteclock[i].def,strlen(cw1))==0){
           k1=i;
           break;
       }
    }
    if(k1 >= 0) {
        o1=clock_get(vex->siteclock[k1], prt);
        r1=vex->siteclock[k1].clock_rate;
    }
    //find station key for stat_id2
    for(i=0; i<vex->nstation; i++){
       if(strncmp(stat_id2,vex->station[i].def,strlen(stat_id2))==0){
            strcpy(cw2,vex->station[i].clock);
            break;
       }
    }
    for(i=0; i<vex->nclock; i++){
       if(strncmp(cw2,vex->siteclock[i].def,strlen(cw2))==0){
           k2=i;
           break;
       }
    }
    if(k1 >= 0) {
        o2=clock_get(vex->siteclock[k2], prt);
        r2=vex->siteclock[k2].clock_rate;
    }
    o12=o2-o1;
    r12=r2-r1;
    //o12=o1-o2;   // debug
    //r12=r1-r2;   // debug
    *coffset=o12;
    *crate=r12;
    *xcoff=o1;
}


void eop_get2(eop_t eop, int *prt, double *ut1, double *xwob, double *ywob)  //2003-07-13
{
    // get EOP parameter at PRT
    //   IN
    //      clock --  site clock parameter in VEX file
    //      PRT[5]   --  PRT
    int i;
    double t0,tspan,t1;
    double tdat[20];
    t0=(double)eop.epoch[1]*86400.0   // DDD
        +(double)eop.epoch[2]*3600.0  // HH
        +(double)eop.epoch[3]*60.0    // MM
        +(double)eop.epoch[4]*1.0;     // SS
    tspan=(double)eop.interval_hr*3600.0;
    tdat[0]=t0;
    for(i=1; i<eop.points; i++){
        tdat[i]=tdat[i-1]+tspan;
    }
    // found prt location
    t1=(double)prt[1]*86400.0   // DDD
        +(double)prt[2]*3600.0  // HH
        +(double)prt[3]*60.0    // MM
        +(double)prt[4]*1.0;     // SS
    if(t1 < tdat[0]){
        *ut1=eop.ut1[0];
        *xwob=eop.xwob[0];
        *ywob=eop.ywob[0];
        goto exit2return;
    }
    if(t1 >= tdat[eop.points-1]){
        *ut1=eop.ut1[eop.points-1];
        *xwob=eop.xwob[eop.points-1];
        *ywob=eop.ywob[eop.points-1];
        goto exit2return;
    }
    for(i=0; i<eop.points-1; i++){
        if((t1 >= tdat[i]) && (t1 < tdat[i+1])){
            *ut1=eop.ut1[i]+(eop.ut1[i+1]-eop.ut1[i])*(t1-tdat[i])/tspan;
            *xwob=eop.xwob[i]+(eop.xwob[i+1]-eop.xwob[i])*(t1-tdat[i])/tspan;
            *ywob=eop.ywob[i]+(eop.ywob[i+1]-eop.ywob[i])*(t1-tdat[i])/tspan;
            break;
        }
    }
exit2return:;
}

void eop_get(vex_t *vex, int *prt, 
                  double *ut1, double *xwob, double *ywob)  //2017-09-04
{
    // get EOP parameter at PRT
    //   IN
    //      clock --  site clock parameter in VEX file
    //      PRT[5]   --  PRT

    int k;
    //find EOP data set  (obsoleted on 2017-09-04
    //                    because vex->global.eop is not defined in some VEX file)
    //k=-1;
    //for(i=0; i<vex->neop; i++){
    //   if(strncmp(vex->global.eop,vex->eop[i].def,strlen(vex->global.eop))==0){
    //       k=i;
    //       break;
    //   }
    //}

    if(vex->neop <=0) goto exit2return;  //2018-02-02
    k=0;   // 2017-09-04  use first data
    //if(k<0) goto exit2return;


    eop_get2(vex->eop[k], prt, &(*ut1), &(*xwob), &(*ywob));
exit2return:;
}


void parsetime1(char *ctime, int *itime)  //2003-03-21
{
//   convert YYYYyDDDdHHhMMmSSs to  yyyy,ddd,hh,mm,ss
    int i,len;
    char abuf[10];
    for(i=0; i<5; i++) itime[i]=0;

    len=strlen(ctime);
    if(len <4 ) goto exitend;
    strncpy(abuf,ctime,4); abuf[4]='\0';
    itime[0]=atoi(abuf);
    if(len <8 ) goto exitend;
    strncpy(abuf,ctime+5,3); abuf[3]='\0';
    //printf("debug abuf %s\n",abuf);
    itime[1]=atoi(abuf);
    if(len <11 ) goto exitend;
    strncpy(abuf,ctime+9,2); abuf[2]='\0';
    itime[2]=atoi(abuf);
    if(len <14 ) goto exitend;
    strncpy(abuf,ctime+12,2);abuf[2]='\0';
    itime[3]=atoi(abuf);
    if(len <17 ) goto exitend;
    strncpy(abuf,ctime+15,2);abuf[2]='\0';
    itime[4]=atoi(abuf);
exitend:;
}

void asc_ra2deg(char *ra, double *radeg) //2003-03-21
{
//   convert HHyMMmSS.SSSSSs to degree
    char abuf[30];
    int i,i1,i2,len;
    int iw[2];
    double dsec,dmin;

    len=strlen(ra);
    *radeg=0.0;
    // found 'h'
    i1=0;
    i2=0;
    for(i=0; i<len; i++){
        if(ra[i]=='h'){
            i2=i;
            break;
        }
    }
    if(i2==0) goto exitend;
    strncpy(abuf,ra,i2); abuf[i2]='\0';
    iw[0]=atoi(abuf);
    // found 'm'
    i1=i2+1;
    for(i=i1; i<len; i++){
        if(ra[i]=='m'){
            i2=i;
            break;
        }
    }
    if(i2==i1) goto exitend;
    i2=i2-i1;
    //strncpy(abuf,ra+i1,i2); abuf[i2]='\0';
    // to avoid warning message in linux compilar above 1 line is revised as follows (2018-02-09)
    if(i2 < 30){
        strncpy(abuf,ra+i1,i2); abuf[i2]='\0';
    } else {
        goto exitend;
    }

    iw[1]=atoi(abuf);
    // found 's'
    i1=i1+i2+1;
    for(i=i1; i<len; i++){
        if(ra[i]=='s'){
            i2=i;
            break;
        }
    }
    if(i2==i1) goto exitend;
    i2=i2-i1;
    strncpy(abuf,ra+i1,i2); abuf[i2]='\0';
    dsec=atof(abuf);

    dmin=(double)iw[1]+dsec/60.0;
    *radeg=15.0*(double)iw[0]+dmin/4.0;
exitend:;
}


void asc_dec2deg(char *dec, double *decdeg) //2003-03-21
{
//   convert +/-DDHHdM'SS.SSSSS" to degree
    char abuf[30];
    int i,i1,i2,len;
    int iw[2];
    double dsec,dmin,ddeg;

    //printf("%s\n",dec);
    len=strlen(dec);
    *decdeg=0.0;
    // found 'd'
    i1=0;
    i2=0;
    for(i=0; i<len; i++){
        if(dec[i]=='d'){
            i2=i;
            break;
        }
    }
    if(i2==0) goto exitend;
    strncpy(abuf,dec,i2); abuf[i2]='\0';
    iw[0]=atoi(abuf);
    // found '''
    i1=i2+1;
    for(i=i1; i<len; i++){
        if(dec[i]=='\''){
            i2=i;
            break;
        }
    }
    if(i2==i1) goto exitend;
    i2=i2-i1;
    //strncpy(abuf,dec+i1,i2); abuf[i2]='\0';
    // to avoid warning message in linux compilar above 1 line is revised as follows (2018-02-09)
    if(i2 < 30){
       strncpy(abuf,dec+i1,i2); abuf[i2]='\0';
    } else {
       goto exitend;
    }

    //printf("debug %d %d %s\n",i1,i2,abuf);
    iw[1]=atoi(abuf);
    // found '"'
    i1=i1+i2+1;
    for(i=i1; i<len; i++){
        if(dec[i]=='\"'){
            i2=i;
            break;
        }
    }
    if(i2==i1) goto exitend;
    i2=i2-i1;
    strncpy(abuf,dec+i1,i2); abuf[i2]='\0';
    dsec=atof(abuf);

    dmin=(double)iw[1]+dsec/60.0;

    if(iw[0]<0) {
        ddeg=-(double)iw[0];
    } else {
        ddeg=(double)iw[0];
    }
    //printf("DEBUG %d %d %f\n",iw[0],iw[1],dsec);
    *decdeg=ddeg+dmin/60.0;
    if(iw[0]<0) *decdeg=-*decdeg;
exitend:;
}


int vex_parse(char *cbuf, vparam_t *vparam)  //2003-03-21
{
/*
    Parse VEX parameters by '=',':',';', and ' '.
    '=',':',';' are returned as a parsed item for further analysis
    Total number of items including '=',':',';' is defined in
    type define statesment of vparam as

    typedef struct vparam
    {
        int  n;          //number of param
        char param[60][128];   // 60x128 characters
    } vparam_t;

    In this case 60 is a maximum number of items and
    128 characters are maximum char in an item.

    If runtime error occurrs, please try to increase the number of
    items in the vparam definition.

       2020-04-12  revised a bit


*/

    // vex_parse =0;  single statement in a cbuf
    //           =1;  more statement left in a cbuf, start point in cbuf 
    //                is updated 
    //char abuf[500];
    char *abuf;
    char *cset=" ";
    char *p=NULL;
    int i,ipos,k,kpos,flag,mflag,len;
    int iret;  // 2020-06-26

    abuf = malloc(_CBUF_SIZE);  // 2020-06-26
    len=strlen(cbuf);
    //printf("len = %d\n",len);

    iret = 0;

    //remove extra space in front

    if(cbuf[0]==' '){
        kpos=0;
        for(k=0; k<len; k++){
            if(cbuf[k]!=' '){
                kpos=k;
                break;
            }
        }
        i=0;
        for(k=kpos; k<len; k++){
            cbuf[i]=cbuf[k];
            i++;
        }
        cbuf[i]='\0';
    }
    // len again

    len=strlen(cbuf);

    // initialize vparam
    //for (i = 0; i < 60; i++) vparam->param[i][0] = '\0';  // 2020-04-12
    for (i = 0; i < MAX_VPARAM; i++) vparam->param[i][0] = '\0';  // 2020-06-26


    // parse start

    if(cbuf[0] == '*') { // comment
        vparam->n=0; 
        //return 0;
        goto exit2return;  // 2020-06-26
    }
    i=0;
    ipos=0;
    flag=0;
    for(k=0; k<len; k++){
        if(cbuf[k] == ' ' || cbuf[k] == '=' || cbuf[k] == ':'){
            if(flag==1) {  // one parameter end
                vparam->param[ipos][i]='\0';
                flag=0;
                i=0;
                ipos++;
            }
            if(cbuf[k] != ' '){
                vparam->param[ipos][i]=cbuf[k];
                vparam->param[ipos][i+1]='\0';
                flag=0;
                i=0;
                ipos++;
            }
            goto nextk;
        }
        if(cbuf[k] == ';'){
            kpos=k;
            goto exithere;
        }
        if(cbuf[k] != ' '){
            flag=1;
            vparam->param[ipos][i]=cbuf[k];
            i++;
        }
nextk:;
    }
    kpos=k;
exithere:
    vparam->param[ipos][i]='\0';
    ipos++;
    vparam->n=ipos;
    i=0;
    mflag=0;
    if(kpos < len-1){
        for(k=kpos+1; k<len; k++) {
            if(cbuf[k] != '\0'){
                abuf[i]=cbuf[k];
                i++;
            }
            if(cbuf[k] == ';') mflag=1;
        }
    }
    if(mflag==1){
        abuf[i]='\0';
        strcpy(cbuf,abuf);
        //return 1;
        iret = 1;  // 2020-06-26
        //goto exit2return;  // 2020-06-26
    }
exit2return:;     // 2020-06-26
    free(abuf);   // 2020-06-26
    //return 0;
    return iret;  // 2020-06-26
}


/* GLOBAL block analysis */
int GLOBAL_ana(FILE *fin, char *line, vex_t *vex) //2017-09-04
{
    int k;
    //vparam_t vp;
    vparam_t *vp;   // 2020-06-26
    //char line[500];
    // return -1 for exitend   0 for goto loop1


    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    vex->global.exper[0]='\0';  // 2017-09-04
    vex->global.eop[0]='\0';    // 2017-09-04

loop:    
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 1 line read  2020-06-26
        free(vp);  // 2020-06-26
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
loop00:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp);  // 2020-06-26
    //if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {  // 2020-06-26
        free(vp);
        return 0;
    }
    if(vp->n > 0 && strncmp(vp->param[1],"$EXPER",6)==0){
         strcpy(vex->global.exper,vp->param[3]); // save def param
         if(k > 0) goto loop00;
         goto loop;
    }
    if(vp->n > 0 && strncmp(vp->param[1],"$EOP",4)==0){
         strcpy(vex->global.eop,vp->param[3]); // save def param
         if(k > 0) goto loop00;
         goto loop;
    }
    if(k > 0) goto loop00;
    goto loop;
}

/* EXPER block analysis */
int EXPER_ana(FILE *fin, char *line, vex_t *vex)  //2003-03-21
{
    int k;
    //vparam_t vp;
    vparam_t *vp;   // 2020-06-26
    //char line[500];
    // return -1 for exitend   0 for goto loop1

    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    vex->nexper=0;
loop:
    
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, 500, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; //goto exitend;  // 1 line read
    }
loop1:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp);  // 2020-06-26
    //if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {
        free(vp);
        return 0; //goto loop1;
    }
    if(vp->n > 0 && strncmp(vp->param[0],"def",3)==0){
         strcpy(vex->exper[vex->nexper].def,vp->param[1]); // save def param
         if(k > 0) goto loop01;
loop00:
         //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
         if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
             free(vp);
             return -1; //goto exitend;  // 1 line read
         }
loop01:
         //k=vex_parse(line, &vp);
         k=vex_parse(line, vp);  // 2020-06-26
         if(vp->n>0 && strncmp(vp->param[0],"enddef",6)==0){
             vex->nexper++;
             goto loop;
         }
         if(vp->n>0 && strncmp(vp->param[0],"exper_name",10)==0){ // 
             strcpy(vex->exper[vex->nexper].exper_name,vp->param[2]);
             if(k > 0) goto loop01;
             goto loop00;
         }
         goto loop00;
    } 
    if(k > 0) goto loop1;
    goto loop;
}


/* SCHEDULE block analysis */
int SKED_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int k,n;
	scan_t scan;     //2009-11-19
	int reach_flag;  //2009-11-19
    int nerr;  // 2016-09-16
    //vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->numobs=0;
    vex->numobs_true=0; //2009-11-19
	reach_flag=0;

    scan.nstation_max=vex->nstation_max;  // 2016-09-16
    scan.station=(station_in_scan_t *)malloc(scan.nstation_max*sizeof(station_in_scan_t));  // 2016-09-16
    nerr=0;

loopscan:
    
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(scan.station);
        return -1; //goto exitend;  // 1 line read
    }
	/* renewed on 2009-11-19 */
	k=sub_sked_ana(fin, line, &scan);
	//if(k == -1) return -1; // EOF encountered
    if(k == -1){
        nerr=-1;
        goto exit2return;
    }
	//if(k == 1) return 0; // "$" encountered
	if(k == 1) goto exit2return;
	if(k == 0){
		//if(vex->numobs < MAXOBS){
        if(vex->numobs < vex->numobs_max){   // 2016-09-16
			vex->scan[vex->numobs].number=vex->numobs+1;
			vex->scan[vex->numobs].numstat=scan.numstat;
			strcpy(vex->scan[vex->numobs].scanid,scan.scanid); // save scan param
			for(n=0; n<5; n++) vex->scan[vex->numobs].start[n]=scan.start[n];
			strcpy(vex->scan[vex->numobs].mode,scan.mode);
			strcpy(vex->scan[vex->numobs].source,scan.source);
			for(n=0; n< scan.numstat; n++){
				strcpy(vex->scan[vex->numobs].station[n].id,scan.station[n].id);
				vex->scan[vex->numobs].station[n].data_start=scan.station[n].data_start;
				vex->scan[vex->numobs].station[n].data_stop=scan.station[n].data_stop;
			}
            vex->numobs++;
		} else {
			if(reach_flag==0){
               //printf("SKED_ana: Number of scans reaches limit (%d)\n",MAXOBS);
               printf("SKED_ana: Number of scans reaches limit (%d)\n",vex->numobs_max);  // 2016-09-16
			   reach_flag=1;
			}
		}
        vex->numobs_true++;
	}
    goto loopscan;
exit2return:
    free(scan.station);
    return nerr;

/*
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if(vp.n > 0 && strncmp(vp.param[0],"scan",4)==0){
         strcpy(vex->scan[vex->numobs].scanid,vp.param[1]); // save scan param
         if(k > 0) goto loopscan01;
loopscan00:
         if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loopscan01:
         k=vex_parse(line, &vp);
         if(vp.n>0 && strncmp(vp.param[0],"endscan",7)==0){
             vex->scan[vex->numobs].number=vex->numobs+1;
             vex->scan[vex->numobs].numstat=i;
             vex->numobs++;
			 // added on 2009-11-19
             vex->numobs_true++;
             if(vex->numobs >= MAXOBS) {
                printf("SKED_ana: Number of observations exceeds limit (%d)\n",MAXOBS);
                vex->numobs -= 1;    // to avoid array overflow
             }
			 // added end
             goto loopscan;
         }
         if(vp.n>0 && strncmp(vp.param[0],"start",5)==0){ // 
             parsetime1(vp.param[2],vex->scan[vex->numobs].start);
             if(k > 0) goto loopscan01;
             goto loopscan00;
         }
         if(vp.n>0 && strncmp(vp.param[0],"mode",4)==0){ // 
             strcpy(vex->scan[vex->numobs].mode,vp.param[2]);
             if(k > 0) goto loopscan01;
             goto loopscan00;
         }
         if(vp.n>0 && strncmp(vp.param[0],"source",6)==0){ // 
             strcpy(vex->scan[vex->numobs].source,vp.param[2]);
             if(k > 0) goto loopscan01;
             goto loopscan00;
         }
         if(vp.n>0 && strncmp(vp.param[0],"station",7)==0){ // 
             strcpy(vex->scan[vex->numobs].station[i].id,vp.param[2]);
             //printf("DDD %s %s\n",vex->scan[vex->numobs].station[i].id,vp.param[4]);
             vex->scan[vex->numobs].station[i].data_start=atoi(vp.param[4]);
             vex->scan[vex->numobs].station[i].data_stop=atoi(vp.param[7]);
             i++;
             if(k > 0) goto loopscan01;
             goto loopscan00;
         }
         goto loopscan00;
    } 
    if(k > 0) goto loop1;
 */
}

int sub_sked_ana(FILE* fin, char *line, scan_t *scan)  //2009-11-19
{
    /*
         return   0: one "def .... enddef" block is sucessfully analyzed
                 -1: eof encountered before reach enddef
                  1: encounter "$" which means the start of next block

    */

    int i, k, n;
    //vparam_t vp;
    vparam_t *vp;   // 2020-06-26

    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26

    scan->scanid[0] = '\0';
    scan->mode[0] = '\0';
    scan->source[0] = '\0';
    scan->number = 0;
    scan->numstat = 0;
    for (n = 0; n < 5; n++) scan->start[n] = 0;
    //for(n=0; n<MAXSTAT; n++){
    for (n = 0; n < scan->nstation_max; n++) {    // 2016-09-16
        scan->station[n].id[0] = '\0';
        scan->station[n].data_start = 0;
        scan->station[n].data_stop = 0;
    }

    i = 0;

loop1:
    //k = vex_parse(line, &vp);
    k = vex_parse(line, vp);  // 2020-06-26

    //if (vp.n > 0 && strncmp(vp.param[0], "$", 1) == 0) return 1;
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {
        free(vp);
        return 1;
    }
    if (vp->n > 0 && strncmp(vp->param[0], "scan", 4) == 0) {
        strcpy(scan->scanid, vp->param[1]); // save scan param
        if (k > 0) goto loopscan01;
    loopscan00:
        //if (fgets(line, 500, fin) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
    loopscan01:
        //k = vex_parse(line, &vp);
        k = vex_parse(line, vp);  // 2020-06-26
        if (vp->n > 0 && strncmp(vp->param[0], "endscan", 7) == 0) {
            scan->numstat = i;
            free(vp);  // 2020-06-26
            return 0;
        }
        if (vp->n > 0 && strncmp(vp->param[0], "start", 5) == 0) { // 
            parsetime1(vp->param[2], scan->start);
            if (k > 0) goto loopscan01;
            goto loopscan00;
        }
        if (vp->n > 0 && strncmp(vp->param[0], "mode", 4) == 0) { // 
            strcpy(scan->mode, vp->param[2]);
            if (k > 0) goto loopscan01;
            goto loopscan00;
        }
        if (vp->n > 0 && strncmp(vp->param[0], "source", 6) == 0) { // 
            strcpy(scan->source, vp->param[2]);
            if (k > 0) goto loopscan01;
            goto loopscan00;
        }
        if (vp->n > 0 && strncmp(vp->param[0], "station", 7) == 0) { // 
            strcpy(scan->station[i].id, vp->param[2]);
            //printf("DDD %s %s\n",scan->station[i].id,vp.param[4]);
            scan->station[i].data_start = atoi(vp->param[4]);
            scan->station[i].data_stop = atoi(vp->param[7]);
            i++;
            if (k > 0) goto loopscan01;
            goto loopscan00;
        }
        goto loopscan00;
    }
    if (k > 0) goto loop1;
    //if (fgets(line, 500, fin) == NULL) return -1;
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1;
    }
    goto loop1;
}

/* TRACKS block analysis */
int TRACKS_ana(FILE* fin, char *line, vex_t *vex) //2016-08-16
{
    // 2020-04-13  add bit/sampl
    // 2020-05-13  add "data_modulation" analysis
    // 2020-06-15  revised


    int i, k, m, ioffset;
    int adbit_ok;  // 2016-08-16
    //vparam_t vp;
    vparam_t *vp;   // 2020-06-26
    //char line[500];
    // return -1 for exitend   0 for goto loop1

    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26

    vex->ntrack_def = 0;

looptrack:
    i = 0;
    //if (fgets(line, 500, fin) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
loop1:
    //k = vex_parse(line, &vp);
    k = vex_parse(line, vp);
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {
        // check adbit     2020-06-15
        TRACKS_check_adbit(vex);  //2020-06-15
        free(vp);  // 2020-06-26
        return 0; //goto loop1;
    }
    if (vp->n > 0 && strncmp(vp->param[0], "def", 3) == 0) {
        strcpy(vex->track_def[vex->ntrack_def].def, vp->param[1]); // save def param
        vex->track_def[vex->ntrack_def].format[0] = '\0';
        vex->track_def[vex->ntrack_def].bitssampl = 1;       // 2016-08-16
        vex->track_def[vex->ntrack_def].data_modulation = 0;   // 2020-05-13

        adbit_ok = 0;                                        // 2016-08-16
        if (k > 0) goto looptrack01;
    looptrack00:
        //if (fgets(line, 500, fin) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
    looptrack01:
        //k = vex_parse(line, &vp);
        k = vex_parse(line, vp);  // 2020-06-26
        if (vp->n > 0 && strncmp(vp->param[0], "enddef", 6) == 0) {
            vex->ntrack_def++;
            goto looptrack;
        }
        if (vp->n > 0 && strncmp(vp->param[0], "track_frame_format", 17) == 0) { // 
            strcpy(vex->track_def[vex->ntrack_def].format, vp->param[2]);
            if (k > 0) goto looptrack01;
            goto looptrack00;
        }
        if (vp->n > 0 && strncmp(vp->param[0], "data_modulation", 15) == 0) { //   2020-05-13
             if (strncmp(vp->param[2], "on", 2) == 0) {
                vex->track_def[vex->ntrack_def].data_modulation = 1;
             }
             if(k > 0) goto looptrack01;
             goto looptrack00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"bits/sample",11)==0){ //   2020-04-13
             vex->track_def[vex->ntrack_def].bitssampl=atoi(vp->param[2]);
             adbit_ok = 1;  // adbit info is taken from "bit/sample" expression
             if(k > 0) goto looptrack01;
             goto looptrack00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"fanout_def",10)==0){ // 
            //printf("TRACK: %s\n",line);
            m=0;

            ioffset=0;
            if(strncmp(vp->param[2],":",1)==0) {  // no sub_pass character
                strcpy(vex->track_def[vex->ntrack_def].fanout_def[i].sub_pass,"\0");
                ioffset=-1;
            } else {
                strcpy(vex->track_def[vex->ntrack_def].fanout_def[i].sub_pass,vp->param[2]);
            }
            strcpy(vex->track_def[vex->ntrack_def].fanout_def[i].chan_id,vp->param[4+ioffset]);
            strcpy(vex->track_def[vex->ntrack_def].fanout_def[i].stream_info,vp->param[6+ioffset]);

            if(strncmp(vp->param[6+ioffset],"mag",3)==0){  // 2016-08-16
                if(adbit_ok==0){
                    vex->track_def[vex->ntrack_def].bitssampl=2;
                    adbit_ok=1;
                }
            }

            vex->track_def[vex->ntrack_def].fanout_def[i].headstack=atoi(vp->param[8+ioffset]);
            while(2*m+10+ioffset < vp->n){
               vex->track_def[vex->ntrack_def].fanout_def[i].track[m]=atoi(vp->param[10+ioffset+2*m]);
               m++;
            } 
            vex->track_def[vex->ntrack_def].fanout_def[i].fanout=m;
            i++;
            if (i > MAXCH) {   // 2019-05-23
                printf(" TRACKS_ana: Warning!!  # of fanout_def fields exceeds limit (%d)\n", MAXCH);
                i--;
            }

            vex->track_def[vex->ntrack_def].nfanout_def=i;

            if(k > 0) goto looptrack01;
            goto looptrack00;
        }
        goto looptrack00;
    }
    if(k > 0) goto loop1;
    goto looptrack;
}

/* SITE block analysis */
int SITE_ana(FILE* fin, char *line, vex_t *vex) //2009-06-19
{
    //  2020-04-12  add analysis of "mk4_site_id"

    int k;
	int reach_flag; //2009-11-19
	site_t site; // 2009-11-19
    //vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->numstat=0;
    vex->numstat_true=0; //2009-11-19
	reach_flag=0; // 2009-11-19
loop:
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if(fgets(line, _LINE_SIZE, fin ) == NULL) return -1; //goto exitend;  // 1 line read 2020-06-26
	/* renewed on 2009-11-19 */
	k=sub_site_ana(fin, line, &site);
	if(k == -1) return -1; // EOF encountered
	if(k == 1) return 0; // "$" encountered
	if(k == 0){
		//if(vex->numstat < MAXSTAT){ // 2009-11-19
        if(vex->numstat < vex->nstation_max){ // 2016-09-16
			strcpy(vex->site[vex->numstat].def,site.def); 
			strcpy(vex->site[vex->numstat].name,site.name);
			strcpy(vex->site[vex->numstat].id,site.id);
			strcpy(vex->site[vex->numstat].mk4_id,site.mk4_id);  // 2020-04-12
			vex->site[vex->numstat].position[0]=site.position[0];
			vex->site[vex->numstat].position[1]=site.position[1];
			vex->site[vex->numstat].position[2]=site.position[2];
            vex->numstat++;
		} else {
			if(reach_flag==0){
               //printf("SITE_ana: Number of sites reaches limit (%d)\n",MAXSTAT);
               printf("SITE_ana: Number of sites reaches limit (%d)\n",vex->nstation_max); // 2016-09-16
			   reach_flag=1;
			}
		}
        vex->numstat_true++;
	}
    goto loop;

/*
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; 
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){


        strcpy(vex->site[vex->numstat].def,vp.param[1]); // save def param
        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
             vex->numstat++;
			 // added on 2009-11-19
             vex->numstat_true++;
             if(vex->numstat >= MAXSTAT) {
                printf("SITE_ana: Number of sites exceeds limit (%d)\n",MAXSTAT);
                vex->numstat -= 1;    // to avoid array overflow
             }
			 // added end 
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[0],"site_name",9)==0){ // 
             strcpy(vex->site[vex->numstat].name,vp.param[2]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"site_ID",7)==0){ // 
             strcpy(vex->site[vex->numstat].id,vp.param[2]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        //if(vp.n>0 && strncmp(vp.param[0],"site_position",13)==0){ // 
        if(vp.n>0 && (strncmp(vp.param[0],"site_position",13)==0 && strlen(vp.param[0])==13)){ // 2009-06-19
             vex->site[vex->numstat].position[0]=atof(vp.param[2]);
             vex->site[vex->numstat].position[1]=atof(vp.param[5]);
             vex->site[vex->numstat].position[2]=atof(vp.param[8]);

             if(k > 0) goto loop01;
             goto loop00;
        }
        goto loop00;


    }
    if(k > 0) goto loop1;
    goto loop;
	*/
}

int sub_site_ana(FILE* fin, char *line, site_t *site)  //2009-11-19
{
/* 
     return   0: one "def .... enddef" block is sucessfully analyzed
	         -1: eof encountered before reach enddef
			  1: encounter "$" which means the start of next block

         2020-04-12 add analysis of mk4_site_id
	          
*/

	int k;
    //vparam_t vp;
    vparam_t *vp;  // 2020-06-26

    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    site->def[0]='\0';
	site->name[0]='\0';
	site->id[0]='\0';
    site->mk4_id[0] = '\0';  // 2020-04-12
	site->position[0]=0.0;
	site->position[1]=0.0;
	site->position[2]=0.0;

loop1:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp);  // 2020-06-26
    //if(vp.n > 0 && strncmp(vp.param[0],"$",1)==0) return 1; 
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {  // 2020-06-26
        free(vp);
        return 1;
    }
    if(vp->n > 0 && strncmp(vp->param[0],"def",3)==0){
        strcpy(site->def,vp->param[1]); // save def param
        if(k > 0) goto loop01;
loop00:
        //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
    loop01:
        //k=vex_parse(line, &vp);
        k=vex_parse(line, vp);
        //if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0) return 0;
        if (vp->n > 0 && strncmp(vp->param[0], "enddef", 6) == 0) {  // 2020-06-26
            free(vp);
            return 0;
        }
        if(vp->n>0 && strncmp(vp->param[0],"site_name",9)==0){
             strcpy(site->name,vp->param[2]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"site_ID",7)==0){ // 
			 strcpy(site->id,vp->param[2]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp->n>0 && (strncmp(vp->param[0],"site_position",13)==0 && strlen(vp->param[0])==13)){
             site->position[0]=atof(vp->param[2]);
             site->position[1]=atof(vp->param[5]);
             site->position[2]=atof(vp->param[8]);

             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"mk4_site_ID",11)==0){ // 2020-04-12
			 strcpy(site->mk4_id,vp->param[2]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        goto loop00;


    }
    if(k > 0) goto loop1;
    //if(fgets(line, 500, fin ) == NULL) return -1;
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
    goto loop1;
}

/* STATION block analysis */
int STATION_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int k,n;
	int reach_flag;  //2009-11-19
    int nerr;  // 2016-09-16
    station_def_t station; //2009-11-19

	//vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->nstation=0;
    vex->nstation_true=0; //2009-11-19
	reach_flag=0;

    station.ndas_max=vex->ndas_max;          // 2016-09-16
    station.das=malloc(station.ndas_max*20); // 2016-09-16
    nerr=0;


loop:
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        nerr = -1;
        goto exit2return;
    }
    /* renewed on 2009-11-19 */
	k=sub_station_ana(fin, line, &station);
    //if(k == -1)return -1; // EOF encountered
    if(k == -1){
        nerr=-1;
        goto exit2return;
    }
	//if(k == 1) return 0; // "$" encountered
	if(k == 1) goto exit2return; // "$" encountered
	if(k == 0){
		//if(vex->nstation < MAXSTAT){
        if(vex->nstation < vex->nstation_max){   // 2016-09-16
			strcpy(vex->station[vex->nstation].def,station.def);
			//for(n=0; n<MAXDAS; n++){
            for(n=0; n<vex->ndas_max; n++){   // 2016-09-16
                vex->station[vex->nstation].das[n][0]='\0'; // just in case
				if(station.das[n][0]!='\0')
                     strcpy(vex->station[vex->nstation].das[n],station.das[n]);
			}
			strcpy(vex->station[vex->nstation].clock,station.clock);
			strcpy(vex->station[vex->nstation].site,station.site);
			strcpy(vex->station[vex->nstation].antenna,station.antenna);
            vex->nstation++;
		} else {
			if(reach_flag==0){
               //printf("STATION_ana: Number of stations reaches limit (%d)\n",MAXSTAT);
               printf("STATION_ana: Number of stations reaches limit (%d)\n",vex->nstation_max);  // 2016-09-16
			   reach_flag=1;
			}
		}
        vex->nstation_true++;
	}
    goto loop;
exit2return:
    free(station.das);
    return nerr;


/*
loop1:
    k=vex_parse(line, &vp);
    //printf("STATION_ana: %s\n",line);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->station[vex->nstation].def,vp.param[1]); // save def param
        vex->station[vex->nstation].antenna[0]='\0';
        vex->station[vex->nstation].clock[0]='\0';
        vex->station[vex->nstation].das[0]='\0';
        vex->station[vex->nstation].site[0]='\0';
        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
             vex->nstation++;
			 // added on 2009-11-19
             vex->nstation_true++;
             if(vex->nstation >= MAXSTAT) {
                printf("STATION_ana: Number of stations exceeds limit (%d)\n",MAXSTAT);
                vex->nstation -= 1;    // to avoid array overflow
             }
			 // added end
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[1],"$DAS",4)==0){ // 
             strcpy(vex->station[vex->nstation].das,vp.param[3]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[1],"$CLOCK",6)==0){ // 
             strcpy(vex->station[vex->nstation].clock,vp.param[3]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[1],"$SITE",5)==0){ // 
             strcpy(vex->station[vex->nstation].site,vp.param[3]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[1],"$ANTENNA",8)==0){ // 
             strcpy(vex->station[vex->nstation].antenna,vp.param[3]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
*/
}


int sub_station_ana(FILE* fin, char *line, station_def_t *station)  //2009-11-19
{
/* 
     return   0: one "def .... enddef" block is sucessfully analyzed
	         -1: eof encountered before reach enddef
			  1: encounter "$" which means the start of next block
	          
*/

	int k,n;
    //vparam_t vp;
    vparam_t *vp;   // 2020-06-26

    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    station->def[0]='\0';
	//for(n=0; n<MAXDAS; n++) station->das[n][0]='\0';
    for(n=0; n<station->ndas_max; n++) station->das[n][0]='\0';  // 2016-09-16
	station->clock[0]='\0';
	station->antenna[0]='\0';
	station->site[0]='\0';

loop1:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp);  // 2020-06-26
    //if(vp.n > 0 && strncmp(vp.param[0],"$",1)==0) return 1; 
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {  // 2020-06-26
        free(vp);
        return 1;
    }
    if(vp->n > 0 && strncmp(vp->param[0],"def",3)==0){
        strcpy(station->def,vp->param[1]); // save def param
        if(k > 0) goto loop01;
loop00:
        //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
    loop01:
        //k=vex_parse(line, &vp);
        k=vex_parse(line, vp);  // 2020-06-26
        //if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0) return 0;
        if (vp->n > 0 && strncmp(vp->param[0], "enddef", 6) == 0) { // 2020-06-26
            free(vp);
            return 0;
        }
        if(vp->n>0 && strncmp(vp->param[1],"$DAS",4)==0){
			//for(n=0; n<MAXDAS; n++){
            for(n=0; n<station->ndas_max; n++){  // 2016-09-16
				if(station->das[n][0]=='\0'){
                    strcpy(station->das[n],vp->param[3]);
					break;
				}
			}
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[1],"$CLOCK",6)==0){
             strcpy(station->clock,vp->param[3]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[1],"$SITE",5)==0){ 
             strcpy(station->site,vp->param[3]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[1],"$ANTENNA",8)==0){ // 
             strcpy(station->antenna,vp->param[3]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        goto loop00;


    }
    if(k > 0) goto loop1;
    //if(fgets(line, 500, fin ) == NULL) return -1;
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
    goto loop1;
}

/* SOURCE block analysis */
int SOURCE_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int k;
	int reach_flag; //2009-11-19
	source_t source; //2009-11-19
    //vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->numstar=0;
    vex->numstar_true=0;  //2009-11-19
	reach_flag=0;
loop:
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
    /* renewed on 2009-11-19 */
	k=sub_source_ana(fin, line, &source);
	if(k == -1) return -1; // EOF encountered
	if(k == 1) return 0; // "$" encountered
	if(k == 0){
		//if(vex->numstar < MAXSTAR){
        if(vex->numstar < vex->numstar_max){    // 2016-09-16
			strcpy(vex->source[vex->numstar].def,source.def); 
			strcpy(vex->source[vex->numstar].name,source.name);
			strcpy(vex->source[vex->numstar].IAU_name,source.IAU_name);
			strcpy(vex->source[vex->numstar].epoch,source.epoch);
			vex->source[vex->numstar].ra=source.ra;
			vex->source[vex->numstar].dec=source.dec;
            vex->numstar++;
		} else {
			if(reach_flag==0){
               //printf("SOURCE_ana: Number of sources reaches limit (%d)\n",MAXSTAR);
               printf("SOURCE_ana: Number of sources reaches limit (%d)\n",vex->numstar_max);  // 2016-09-16
			   reach_flag=1;
			}
		}
        vex->numstar_true++;
	}
    goto loop;

/*
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->source[vex->numstar].def,vp.param[1]); // save def param
        vex->source[vex->numstar].name[0]='\0';
        vex->source[vex->numstar].IAU_name[0]='\0';
        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
             vex->numstar++;
			 // added on 2009-11-19 
             vex->numstar_true++;
             if(vex->numstar >= MAXSTAR) {
                printf("SOURCE_ana: Number of sources exceeds limit (%d)\n",MAXSTAR);
                vex->numstar -= 1;    // to avoid array overflow
             }
			 // added end
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[0],"source_name",11)==0){ // 
             strcpy(vex->source[vex->numstar].name,vp.param[2]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"IAU_name",8)==0){ // 
             strcpy(vex->source[vex->numstar].IAU_name,vp.param[2]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"ref_coord_frame",15)==0){ //
             strcpy(vex->source[vex->numstar].epoch,vp.param[2]);
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"ra",2)==0){ // 
            asc_ra2deg(vp.param[2],&vex->source[vex->numstar].ra);
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp.n>0 && strncmp(vp.param[0],"dec",3)==0){ // 
            asc_dec2deg(vp.param[2],&vex->source[vex->numstar].dec);
            if(k > 0) goto loop01;
            goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
*/
}

int sub_source_ana(FILE* fin, char *line, source_t *source)  //2009-11-19
{
/* 
     return   0: one "def .... enddef" block is sucessfully analyzed
	         -1: eof encountered before reach enddef
			  1: encounter "$" which means the start of next block
	          
*/

	int k;
    //vparam_t vp;
    vparam_t *vp;  // 2020-06-26

    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    source->def[0]='\0';
	source->epoch[0]='\0';
	source->type[0]='\0';
	source->name[0]='\0';
	source->IAU_name[0]='\0';
	source->ra=0.0;
	source->dec=0.0;

loop1:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp);  // 2020-06-26
    //if(vp.n > 0 && strncmp(vp.param[0],"$",1)==0) return 1; 
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {
        free(vp);
        return 1;
    }
    if(vp->n > 0 && strncmp(vp->param[0],"def",3)==0){
        strcpy(source->def,vp->param[1]); // save def param
        if(k > 0) goto loop01;
loop00:
        //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
    loop01:
        //k=vex_parse(line, &vp);
        k=vex_parse(line, vp);   // 2020-06-26
        //if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0) return 0;
        if (vp->n > 0 && strncmp(vp->param[0], "enddef", 6) == 0) {  // 2020-06-26
            free(vp);
            return 0;
        }
        if(vp->n>0 && strncmp(vp->param[0],"source_name",11)==0){ // 
             strcpy(source->name,vp->param[2]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"IAU_name",8)==0){ // 
             strcpy(source->IAU_name,vp->param[2]);
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"ref_coord_frame",15)==0){ //
             strcpy(source->epoch,vp->param[2]);
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"ra",2)==0){ // 
            asc_ra2deg(vp->param[2],&source->ra);
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"dec",3)==0){ // 
            asc_dec2deg(vp->param[2],&source->dec);
            if(k > 0) goto loop01;
            goto loop00;
        }
        goto loop00;

    }
    if(k > 0) goto loop1;
    //if(fgets(line, 500, fin ) == NULL) return -1;
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
    goto loop1;
}

/* CLOCK block analysis */
int CLOCK_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int k,i;
    //vparam_t vp;
    vparam_t *vp;  // 2020-06-26
    char cbuf[50];
    //char line[500];
    // return -1 for exitend   0 for goto loop1

    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    vex->nclock=0;
loop:
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
    //printf("CLOCK: %s\n",line);
loop1:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp);   // 2020-06-26
    //if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {  // 2020-06-26
        free(vp);
        return 0; //goto loop1;
    }
    if(vp->n > 0 && strncmp(vp->param[0],"def",3)==0){
        //printf("CLOCK2:%s\n",vp.param[0]);
        strcpy(vex->siteclock[vex->nclock].def,vp->param[1]); // save def param
        // default set
        for(i=0; i<5; i++){
           vex->siteclock[vex->nclock].validity_epoch[i]=0;
           vex->siteclock[vex->nclock].clock_epoch[i]=0;
        }
        vex->siteclock[vex->nclock].clock_offset=0.0;
        vex->siteclock[vex->nclock].clock_rate=0.0;
        if(k > 0) goto loop01;
loop00:
        //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
    loop01:
        //k=vex_parse(line, &vp);
        k=vex_parse(line, vp);  // 2020-06-26
        if(vp->n>0 && strncmp(vp->param[0],"enddef",6)==0){
             vex->nclock++;
             //printf("CLOCK3: %d\n",vex->nclock);
             goto loop;
        }
        if(vp->n>0 && strncmp(vp->param[0],"clock_early",11)==0){ // 
             strcpy(cbuf,vp->param[2]);
             parsetime1(cbuf,vex->siteclock[vex->nclock].validity_epoch);
             strcpy(cbuf,vp->param[4]);
             vex->siteclock[vex->nclock].clock_offset=atof(cbuf);
             if(strncmp(vp->param[5],"msec",4)==0) 
                  vex->siteclock[vex->nclock].clock_offset*=1.0e-3;
             if(strncmp(vp->param[5],"usec",4)==0) 
                  vex->siteclock[vex->nclock].clock_offset*=1.0e-6;
             if(strncmp(vp->param[5],"nsec",4)==0) 
                  vex->siteclock[vex->nclock].clock_offset*=1.0e-9;
             if(strncmp(vp->param[5],"psec",4)==0) 
                  vex->siteclock[vex->nclock].clock_offset*=1.0e-12;
             if(vp->n>7){
                strcpy(cbuf,vp->param[7]);
                parsetime1(cbuf,vex->siteclock[vex->nclock].clock_epoch);
                strcpy(cbuf,vp->param[9]);
                vex->siteclock[vex->nclock].clock_rate=atof(cbuf);
                vex->siteclock[vex->nclock].clock_rate*=1.0e-6; //2003-07-13
             }
             if(k > 0) goto loop01;
             goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* EOP block analysis */
int EOP_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    // 2018-02-01 revised
    int k,m;
    //vparam_t vp;
    vparam_t *vp;
    char cbuf[50];
    //char line[500];
    // return -1 for exitend   0 for goto loop1

    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    vex->neop=0;
    vex->eop[vex->neop].points=0;  // 2018-02-01
loop:
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
loop1:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp);  // 2020-06-26
    //if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {  // 2020-06-26
        free(vp);
        return 0; //goto loop1;
    }
    if(vp->n > 0 && strncmp(vp->param[0],"def",3)==0){
        strcpy(vex->eop[vex->neop].def,vp->param[1]); // save def param
        if(k > 0) goto loop01;
loop00:
        //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
    loop01:
        //k=vex_parse(line, &vp);
        k=vex_parse(line, vp);  // 2020-06-26
        if(vp->n>0 && strncmp(vp->param[0],"enddef",6)==0){
             vex->neop++;
             if(vex->neop >= vex->neop_max) return 0;     // 2018-06-05
             vex->eop[vex->neop].points=0;  // 2018-02-01
             goto loop;
        }
        if(vp->n>0 && strncmp(vp->param[0],"TAI-UTC",7)==0){ // 
            // nothing to do 
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"eop_ref_epoch",13)==0){ // 
            strcpy(cbuf,vp->param[2]);
            parsetime1(cbuf,vex->eop[vex->neop].epoch);
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"num_eop_points",14)==0){ // 
            strcpy(cbuf,vp->param[2]);
            vex->eop[vex->neop].points=atoi(cbuf);
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"eop_interval",12)==0){ // 
            strcpy(cbuf,vp->param[2]);
            vex->eop[vex->neop].interval_hr=atoi(cbuf);
            // if u want to check unit use following method
            //if(strncmp(vp.param[3],"hr",2)==0) 
            //    vex->eop[vex->neop].interval_hr*=1;
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"ut1-utc",7)==0){ // 
            m=0;
            strcpy(cbuf,vp->param[2]);
            while(1){
               strcpy(cbuf,vp->param[m*3+2]);
               vex->eop[vex->neop].ut1[m]=atof(cbuf);
               if(strncmp(vp->param[m*3+3],"msec",4)==0) 
                    vex->eop[vex->neop].ut1[m]*=1.0e-3;
               m++;
               //if(m>=vp.n) break;
               if(m*3+2 >=vp->n) break;
            }
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"x_wobble",8)==0){ // 
            m=0;
            strcpy(cbuf,vp->param[2]);
            while(1){
               strcpy(cbuf,vp->param[m*3+2]);
               vex->eop[vex->neop].xwob[m]=atof(cbuf);
               //if(strncmp(vp.param[m*3+3],"asec",4)==0) // asec unit 
               //     vex->eop[vex->neop].xwob[m]*=1.0;
               m++;
               //if(m>=vp.n) break;
               if(m*3+2 >=vp->n) break;
            }
            if(k > 0) goto loop01;
            goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"y_wobble",8)==0){ // 
            m=0;
            strcpy(cbuf,vp->param[2]);
            while(1){
               strcpy(cbuf,vp->param[m*3+2]);
               vex->eop[vex->neop].ywob[m]=atof(cbuf);
               //if(strncmp(vp.param[m*3+3],"asec",4)==0) // asec unit 
               //     vex->eop[vex->neop].ywob[m]*=1.0;
               m++;
               //if(m>=vp.n) break;
               if(m*3+2 >=vp->n) break;
            }
            if(k > 0) goto loop01;
            goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* ROLL block analysis */
int ROLL_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    int k;
    //vparam_t vp;
    vparam_t *vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1

    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    vex->nroll=0;
loop:
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
loop1:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp);  // 2020-06-26
    //if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //goto loop1;
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {  // 2020-06-26
        free(vp);
        return 0; //goto loop1;
    }
    if(vp->n > 0 && strncmp(vp->param[0],"def",3)==0){
        strcpy(vex->roll[vex->nroll].def,vp->param[1]); // save def param
        if(k > 0) goto loop01;
loop00:
        //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
    loop01:
        //k=vex_parse(line, &vp);
        k=vex_parse(line, vp);  // 2020-06-26
        if(vp->n>0 && strncmp(vp->param[0],"enddef",6)==0){
             vex->nroll++;
             goto loop;
        }
        if(vp->n>0 && strncmp(vp->param[0],"roll",4)==0){ // 
            //printf("ROLL_ana: %s\n",vp.param[2]);
            strcpy(vex->roll[vex->nroll].rolldef,vp->param[2]);
            if(k > 0) goto loop01;
            goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* FREQ block analysis */
int FREQ_ana(FILE* fin, char *line, vex_t *vex) //2003-03-21
{
    // 2020-04-13 revised

    int i,k,m,m1,mpos,ioffset;
    //vparam_t vp;
    vparam_t *vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1

    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    vex->nfreq_def=0;

    ioffset=1;
    if(strncmp(vex->flag,"OVEX",4)==0) ioffset=2;
loop:
    i=0;
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
loop1:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp);  // 2020-06-26
    //if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {  // 2020-06-26
        free(vp);
        return 0; //
    }
    if(vp->n > 0 && strncmp(vp->param[0],"def",3)==0){
        strcpy(vex->freq_def[vex->nfreq_def].def,vp->param[1]); // save def param
        vex->freq_def[vex->nfreq_def].samplerate=0.0;
        if(k > 0) goto loop01;
loop00:
        //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
    loop01:
        //k=vex_parse(line, &vp);
        k=vex_parse(line, vp);
        if(vp->n>0 && strncmp(vp->param[0],"enddef",6)==0){
            vex->freq_def[vex->nfreq_def].numch=i;
             vex->nfreq_def++;
             goto loop;
        }
        if(vp->n>0 && strncmp(vp->param[0],"sample_rate",11)==0){ // 
             vex->freq_def[vex->nfreq_def].samplerate=atof(vp->param[2]);
             if(strncmp(vp->param[3],"Ms/sec",6)==0) {
                  vex->freq_def[vex->nfreq_def].samplerate*=1.0e6;
             }
             if(strncmp(vp->param[3],"ks/sec",6)==0) {
                  vex->freq_def[vex->nfreq_def].samplerate*=1.0e3;
             }
             if(k > 0) goto loop01;
             goto loop00;
        }
        if(vp->n>0 && strncmp(vp->param[0],"chan_def",8)==0){ // 
             strcpy(vex->freq_def[vex->nfreq_def].chan_def[i].chan_id0, vp->param[2]); // 2020-04-13
             vex->freq_def[vex->nfreq_def].chan_def[i].rf=atof(vp->param[3]);

             m1=0;
             for(m=0; m<vp->n; m++){
                 if(strncmp(vp->param[m],":",1)==0) {
                     m1++;
                     if(m1==ioffset){
                         mpos=m+1;
                         break;
                     }
                 }
             }
             vex->freq_def[vex->nfreq_def].chan_def[i].rf=atof(vp->param[mpos]);
             if(strncmp(vp->param[mpos+1],"GHz",3)==0) {
                  vex->freq_def[vex->nfreq_def].chan_def[i].rf*=1.0e9;
             }
             if(strncmp(vp->param[mpos+1],"MHz",3)==0) {
                  vex->freq_def[vex->nfreq_def].chan_def[i].rf*=1.0e6;
             }
             if(strncmp(vp->param[mpos+1],"kHz",3)==0) {
                  vex->freq_def[vex->nfreq_def].chan_def[i].rf*=1.0e3;
             }
             strcpy(vex->freq_def[vex->nfreq_def].chan_def[i].sideband,vp->param[mpos+3]);
             vex->freq_def[vex->nfreq_def].chan_def[i].vbw=atof(vp->param[mpos+5]);
             if(strncmp(vp->param[mpos+6],"GHz",3)==0) {
                  vex->freq_def[vex->nfreq_def].chan_def[i].vbw*=1.0e9;
             }
             if(strncmp(vp->param[mpos+6],"MHz",3)==0) {
                  vex->freq_def[vex->nfreq_def].chan_def[i].vbw*=1.0e6;
             }
             if(strncmp(vp->param[mpos+6],"kHz",3)==0) {
                  vex->freq_def[vex->nfreq_def].chan_def[i].vbw*=1.0e3;
             }
             strcpy(vex->freq_def[vex->nfreq_def].chan_def[i].chan_id,vp->param[mpos+8]);
             strcpy(vex->freq_def[vex->nfreq_def].chan_def[i].bbc_id,vp->param[mpos+10]);
             strcpy(vex->freq_def[vex->nfreq_def].chan_def[i].pcal_id,vp->param[mpos+12]);
             i++;
             if (i > MAXCH) {   // 2019-05-23
                 printf(" FREQ_ana: Warning!!  # of chan_def fields exceeds limit (%d)\n", MAXCH);
                 i--;
             }

        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* MODE block analysis */
int MODE_ana(FILE* fin, char *line, vex_t *vex)  //2003-07-17
{
    // 2019-06-11  revised
    int i,k,nfreq,nroll,ntrack,npcal,m;
    int nif,nbbc;
    int nmax;  // 2019-06-11
    //vparam_t vp;
    vparam_t *vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    vex->nmode=0;

loop:
    nfreq=0; nroll=0; ntrack=0; npcal=0; nif=0; nbbc=0;
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
loop1:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp);   // 2020-06-26
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {
        free(vp); // 2020-06-26
        return 0;
    }
    if(vp->n > 0 && strncmp(vp->param[0],"def",3)==0){
        strcpy(vex->mode[vex->nmode].def,vp->param[1]); // save def param
        if(k > 0) goto loop01;
loop00:
        //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
    loop01:
        //k=vex_parse(line, &vp);
        k=vex_parse(line, vp);  // 2020-06-26
        if(vp->n>0 && strncmp(vp->param[0],"enddef",6)==0){
            // check table discrepancy
            //if (nfreq != ntrack) printf("MODE_ana: Warning!! \n");  // delete 2019-06-11
            // find maximum  2019-06-11
            nmax = nfreq;
            if (ntrack > nmax) nmax = ntrack;
            if (nroll  > nmax) nmax = nroll;
            if (npcal  > nmax) nmax = npcal;
            if (nif    > nmax) nmax = nif;
            if (nbbc   > nmax) nmax = nbbc;

            //vex->mode[vex->nmode].nmode=nfreq;
            vex->mode[vex->nmode].nmode=nmax;  // 2019-06-11
            // clean up unused area (2019-06-11)
            for (i = nfreq; i < nmax; i++) {
                vex->mode[vex->nmode].mode_freq[i].mode[0] = '\0';
                vex->mode[vex->nmode].mode_freq[i].id[0] = '\0';
            }
            for (i = ntrack; i < nmax; i++) {
                vex->mode[vex->nmode].mode_track[i].mode[0] = '\0';
                vex->mode[vex->nmode].mode_track[i].id[0] = '\0';
            }
            for (i = nroll; i < nmax; i++) {
                vex->mode[vex->nmode].mode_roll[i].mode[0] = '\0';
                vex->mode[vex->nmode].mode_roll[i].id[0] = '\0';
            }
            for (i = npcal; i < nmax; i++) {
                vex->mode[vex->nmode].mode_pcal_detect[i].mode[0] = '\0';
                vex->mode[vex->nmode].mode_pcal_detect[i].id[0] = '\0';
            }
            for (i = nif; i < nmax; i++) {
                vex->mode[vex->nmode].mode_if[i].mode[0] = '\0';
                vex->mode[vex->nmode].mode_if[i].id[0] = '\0';
            }
            for (i = nbbc; i < nmax; i++) {
                vex->mode[vex->nmode].mode_bbc[i].mode[0] = '\0';
                vex->mode[vex->nmode].mode_bbc[i].id[0] = '\0';
            }

            vex->nmode++;
            goto loop;
        }
        if(vp->n > 0 && strncmp(vp->param[1],"$FREQ",5)==0){
           i=5;
           while(1){
              // first check already existed id or not
              if(nfreq >1){
                   for(m=0; m<nfreq; m++){
                       if(strncmp(vex->mode[vex->nmode].mode_freq[m].id,vp->param[i],strlen(vp->param[i]))==0){
                           // if u want to overwrite please remove // of strcpy(.... 
                           //strcpy(vex->mode[vex->nmode].mode_freq[m].mode,vp.param[3]);
                           goto next_fid;
                       }
                   }
              }
               // for new id
              strcpy(vex->mode[vex->nmode].mode_freq[nfreq].mode,vp->param[3]); // 
              strcpy(vex->mode[vex->nmode].mode_freq[nfreq].id,vp->param[i]); 
              nfreq++;
next_fid:;
              i+=2;
              if(i >= vp->n) break;
           } 
           if(k > 0) goto loop01;
           goto loop00;
        }

        if(vp->n > 0 && strncmp(vp->param[1],"$TRACKS",7)==0){
           i=5;
           while(1){
              // first check already existed id or not
               if(ntrack >1){
                   for(m=0; m<ntrack; m++){
                       if(strncmp(vex->mode[vex->nmode].mode_track[m].id,vp->param[i],strlen(vp->param[i]))==0){
                           // if u want to overwrite please remove // of strcpy(.... 
                           //strcpy(vex->mode[vex->nmode].mode_track[m].mode,vp.param[3]);
                           goto next_tid;
                       }
                   }
               }
               // for new id
               strcpy(vex->mode[vex->nmode].mode_track[ntrack].mode,vp->param[3]); // 
               strcpy(vex->mode[vex->nmode].mode_track[ntrack].id,vp->param[i]); 
               ntrack++;
next_tid:;
               i+=2;
               if(i >= vp->n) break;
           }
           if(k > 0) goto loop01;
           goto loop00;
        }
        if(vp->n > 0 && strncmp(vp->param[1],"$ROLL",5)==0){
           i=5;
           while(1){
              strcpy(vex->mode[vex->nmode].mode_roll[nroll].mode,vp->param[3]); // 
              strcpy(vex->mode[vex->nmode].mode_roll[nroll].id,vp->param[i]); 
              nroll++;
              i+=2;
              if(i >= vp->n) break;
           }
           if(k > 0) goto loop01;
           goto loop00;
        }

        if(vp->n > 0 && strncmp(vp->param[1],"$PHASE_CAL_DETECT",17)==0){
           i=5;
           while(1){
              // first check already existed id or not
               if(npcal >1){
                   for(m=0; m<npcal; m++){
                       if(strncmp(vex->mode[vex->nmode].mode_pcal_detect[m].id,vp->param[i],strlen(vp->param[i]))==0){
                           // if u want to overwrite please remove // of strcpy(.... 
                           //strcpy(vex->mode[vex->nmode].mode_track[m].mode,vp.param[3]);
                           goto next_pcid;
                       }
                   }
               }
              strcpy(vex->mode[vex->nmode].mode_pcal_detect[npcal].mode,vp->param[3]); // 
              strcpy(vex->mode[vex->nmode].mode_pcal_detect[npcal].id,vp->param[i]); 
              npcal++;
next_pcid:;
              i+=2;
              if(i >= vp->n) break;
           }
           if(k > 0) goto loop01;
           goto loop00;
        } 
        // $IF mode
        if(vp->n > 0 && strncmp(vp->param[1],"$IF",3)==0){
           i=5;
           while(1){
              // first check already existed id or not
               if(nif >1){
                   for(m=0; m<nif; m++){
                       if(strncmp(vex->mode[vex->nmode].mode_if[m].id,vp->param[i],strlen(vp->param[i]))==0){
                           // if u want to overwrite please remove // of strcpy(.... 
                           //strcpy(vex->mode[vex->nmode].mode_track[m].mode,vp.param[3]);
                           goto next_ifid;
                       }
                   }
               }
               // for new id
               strcpy(vex->mode[vex->nmode].mode_if[nif].mode,vp->param[3]); // 
               strcpy(vex->mode[vex->nmode].mode_if[nif].id,vp->param[i]); 
               nif++;
next_ifid:;
               i+=2;
               if(i >= vp->n) break;
           }
           if(k > 0) goto loop01;
           goto loop00;
        }

        // $BBC mode
        if(vp->n > 0 && strncmp(vp->param[1],"$BBC",3)==0){
           i=5;
           while(1){
              // first check already existed id or not
               if(nbbc >1){
                   for(m=0; m<nbbc; m++){
                       if(strncmp(vex->mode[vex->nmode].mode_bbc[m].id,vp->param[i],strlen(vp->param[i]))==0){
                           // if u want to overwrite please remove // of strcpy(.... 
                           //strcpy(vex->mode[vex->nmode].mode_track[m].mode,vp.param[3]);
                           goto next_bbcid;
                       }
                   }
               }
               // for new id
               strcpy(vex->mode[vex->nmode].mode_bbc[nbbc].mode,vp->param[3]); // 
               strcpy(vex->mode[vex->nmode].mode_bbc[nbbc].id,vp->param[i]); 
               nbbc++;
next_bbcid:;
               i+=2;
               if(i >= vp->n) break;
           }
           if(k > 0) goto loop01;
           goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}


/* PCAL block analysis */
int PCAL_ana(FILE* fin, char *line, vex_t *vex) /* 2004-06-17 */
{
    int i,k,m,mm;
    char cbuf[50];
    //vparam_t vp;
    vparam_t *vp;   // 2020-06-26
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    vex->npcal=0;
    //printf("PCAL ANA\n");
loop:
    i=0;
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
loop1:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp);   // 2020-06-26
    //if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {  // 2020-06-26
        free(vp);
        return 0; //
    }
    if(vp->n > 0 && strncmp(vp->param[0],"def",3)==0){
        strcpy(vex->pcal[vex->npcal].def,vp->param[1]); // save def param
//printf("DEBUG %s\n",vex->pcal[vex->npcal].def);
//printf("k= %d\n",k);
        if(k > 0) goto loop01;
loop00:
        //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
    loop01:
//printf("debug\n");
        //k=vex_parse(line, &vp);
        k=vex_parse(line, vp);  // 2020-06-26
//printf("debug2 k=%d %s\n",k,line);
        if(vp->n>0 && strncmp(vp->param[0],"enddef",6)==0){
            vex->pcal[vex->npcal].npcal=i;
             vex->npcal++;
             //if(vex->nifb >= MAXIF_B){   // 2016-09-14
             if(vex->npcal > vex->npcal_max){   // 2016-09-14
                 printf(" PCAL_ana: Warning!!  # of PCAL blocks exceeds limit (%d)\n",vex->npcal_max);
                 vex->npcal--;
             }
             goto loop;
        }
        if(vp->n>0 && strncmp(vp->param[0],"phase_cal_detect",16)==0){ // 
             strcpy(vex->pcal[vex->npcal].pcal[i].pcal_id,vp->param[2]);
//printf("debug3 %s\n",vp.param[2]);
//printf("debug4 %s\n",vex->pcal[vex->npcal].pcal[i].pcal_id);
            m=0;
            mm=0;
            strcpy(cbuf,vp->param[2]);
            if(vp->n > 4){     /* 2004-06-17 */
//printf("debug40 %d\n",vp.n);
            while(1){
//printf("debug41\n");

               strcpy(cbuf,vp->param[m*2+4]);
//printf("debug42\n");
               if(atoi(cbuf) != 0) {    // 2003-10-28
                  vex->pcal[vex->npcal].pcal[i].tone[mm]=atoi(cbuf);
                  mm++;
               } // 2003-10-28
               //vex->pcal[vex->npcal].pcal[i].tone[m]=atoi(cbuf);
//printf("debug %d\n",vex->pcal[vex->npcal].pcal[i].tone[m]);
               m++;
//printf("m,vp.n %d %d\n",m,vp.n);
               if(m*2+4 >=vp->n) {
                   //vex->pcal[vex->npcal].pcal[i].ntone=m;
                   vex->pcal[vex->npcal].pcal[i].ntone=mm;  // 2003-10-28
                   break;
               }
            }
            } else {
                vex->pcal[vex->npcal].pcal[i].ntone=0;  /* 2004-06-17 */
            }
            i++;
//printf("i,k %d %d\n",i,k);
            if(k > 0) goto loop01;
            goto loop00;
        }
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}


/* BBC block analysis */
int BBC_ana(FILE* fin, char *line, vex_t *vex) //2003-07-17
{
    int i,k;
    char cbuf[50];
    //vparam_t vp;
    vparam_t *vp;  // 2020-06-26
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    vex->nbbc=0;
//printf("BBC ANA\n");
loop:
    i=0;
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
    if (line[0] == '*') goto loop;  // 2019-05-22
loop1:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp);  // 2020-06-26
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {
        /*
        for (k = 0; k < 64; k++) {  // debug
            printf("  Debug k %d  %s %s\n", k, vex->bbc[0].bbc_def[k].bbc_id,
                vex->bbc[0].bbc_def[k].if_id);  // debug
        }
        */
        free(vp);  // 2020-06-26
        return 0; //
    }
    if(vp->n > 0 && strncmp(vp->param[0],"def",3)==0){
        strcpy(vex->bbc[vex->nbbc].def,vp->param[1]); // save def param

        if(k > 0) goto loop01;
loop00:
        //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
        if (line[0] == '*') goto loop00;  // 2019-05-22
loop01:
        //k=vex_parse(line, &vp);
        k=vex_parse(line, vp);  // 2020-06-26
        if(vp->n>0 && strncmp(vp->param[0],"enddef",6)==0){
            vex->bbc[vex->nbbc].nbbc=i;
             vex->nbbc++;
             //if(vex->nbbc >= MAXBBC_B){   // 2016-09-14
             if(vex->nbbc > vex->nbbc_max){   // 2016-09-16
                 printf(" BBC_ana: Warning!!  # of BBC blocks exceeds limit (%d)\n",vex->nbbc_max);
                 vex->nbbc--;
             }

             goto loop;
        }
        if(vp->n>0 && strncmp(vp->param[0],"BBC_assign",10)==0){ // 
             strcpy(vex->bbc[vex->nbbc].bbc_def[i].bbc_id,vp->param[2]);
             strcpy(cbuf,vp->param[4]);
             vex->bbc[vex->nbbc].bbc_def[i].bbcch=atoi(cbuf);
             strcpy(vex->bbc[vex->nbbc].bbc_def[i].if_id,vp->param[6]);
             i++;
             if (i > MAXCH) {   // 2019-05-23
                 printf(" BBC_ana: Warning!!  # of BBC_assign field exceeds limit (%d)\n", MAXCH);
                 i--;
             }
        }
        if(k > 0) goto loop01;
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* IF block analysis */
int IF_ana(FILE* fin, char *line, vex_t *vex) //2003-07-17
{
    int i,k;
    //vparam_t vp;
    vparam_t *vp;  // 2020-06-26
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    vex->nifb=0;
//printf("IF ANA\n");
loop:
    i=0;
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
loop1:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp);  // 2020-06-26
    //if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {  // 2020-06-26
        free(vp);
        return 0; //
    }
    if(vp->n > 0 && strncmp(vp->param[0],"def",3)==0){
        strcpy(vex->ifb[vex->nifb].def,vp->param[1]); // save def param
//printf("DEBUG %s\n",vex->ifb[vex->nifb].def);

        if(k > 0) goto loop01;
loop00:
        //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
    loop01:
        //k=vex_parse(line, &vp);
        k=vex_parse(line, vp);  // 2020-06-26
        if(vp->n>0 && strncmp(vp->param[0],"enddef",6)==0){
             vex->ifb[vex->nifb].nif=i;
             vex->nifb++;
             //if(vex->nifb >= MAXIF_B){   // 2016-09-14
             if(vex->nifb > vex->nifb_max){   // 2016-09-16
                 printf(" IF_ana: Warning!!  # of IF blocks exceeds limit (%d)\n",vex->nifb_max);
                 vex->nifb--;
             }
             goto loop;
        }
        if(vp->n>0 && strncmp(vp->param[0],"if_def",6)==0){ // 
             strcpy(vex->ifb[vex->nifb].if_def[i].if_id,vp->param[2]);
             strcpy(vex->ifb[vex->nifb].if_def[i].pol,vp->param[6]);

             vex->ifb[vex->nifb].if_def[i].lo=atof(vp->param[8]);
             if(strncmp(vp->param[9],"GHz",3)==0) {
                  vex->ifb[vex->nifb].if_def[i].lo*=1.0e9;
             }
             if(strncmp(vp->param[9],"MHz",3)==0) {
                  vex->ifb[vex->nifb].if_def[i].lo*=1.0e6;
             }
             if(strncmp(vp->param[9],"kHz",3)==0) {
                  vex->ifb[vex->nifb].if_def[i].lo*=1.0e3;
             }
             strcpy(vex->ifb[vex->nifb].if_def[i].sideband,vp->param[11]);
             vex->ifb[vex->nifb].if_def[i].pcal_space=0.0;
             if(vp->n >= 13){
                 vex->ifb[vex->nifb].if_def[i].pcal_space=atof(vp->param[13]);
                 if(strncmp(vp->param[14],"GHz",3)==0) {
                    vex->ifb[vex->nifb].if_def[i].pcal_space*=1.0e9;
                 }
                 if(strncmp(vp->param[14],"MHz",3)==0) {
                    vex->ifb[vex->nifb].if_def[i].pcal_space*=1.0e6;
                 }
                 if(strncmp(vp->param[14],"kHz",3)==0) {
                    vex->ifb[vex->nifb].if_def[i].pcal_space*=1.0e3;
                 }
             }
             vex->ifb[vex->nifb].if_def[i].pcal_base=0.0;
             if(vp->n >= 16){
                 vex->ifb[vex->nifb].if_def[i].pcal_base=atof(vp->param[16]);
                 if(strncmp(vp->param[17],"GHz",3)==0) {
                    vex->ifb[vex->nifb].if_def[i].pcal_base*=1.0e9;
                 }
                 if(strncmp(vp->param[17],"MHz",3)==0) {
                    vex->ifb[vex->nifb].if_def[i].pcal_base*=1.0e6;
                 }
                 if(strncmp(vp->param[17],"kHz",3)==0) {
                    vex->ifb[vex->nifb].if_def[i].pcal_base*=1.0e3;
                 }
             }
             i++;

        }
//printf("i,k %d %d\n",i,k);
        if(k > 0) goto loop01;
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
}

/* DAS block analysis */
int DAS_ana(FILE* fin, char *line, vex_t *vex) 
{
    int k;
    das_def_t das;
    int reach_flag; //2009-11-19
    //char cbuf[50];
    //vparam_t vp;
    //char line[500];
    // return -1 for exitend   0 for goto loop1
    vex->ndas=0;
    vex->ndas_true=0; //2009-11-19
	reach_flag=0; //2009-11-19
//printf("DAS ANA\n");
loop:
    //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        return -1; //goto exitend;  // 1 line read 2020-06-26
    }
    /* renewed on 2009-11-19 */
	k=sub_das_ana(fin, line, &das);
	if(k == -1) return -1; // EOF encountered
	if(k == 1) return 0; // "$" encountered
	if(k == 0){
		//if(vex->ndas < MAXDAS){
        if(vex->ndas < vex->ndas_max){    // 2016-09-16
			strcpy(vex->das[vex->ndas].def,das.def);
			vex->das[vex->ndas].earlysec=das.earlysec;
            vex->ndas++;
		} else {
			if(reach_flag==0){
               //printf("DAS_ana: Number of data ack terminals reaches limit (%d)\n",MAXDAS);
               printf("DAS_ana: Number of data ack terminals reaches limit (%d)\n",vex->ndas_max);  //2016-09-16
			   reach_flag=1;
			}
		}
        vex->ndas_true++;
	}
    goto loop;

/*
loop1:
    k=vex_parse(line, &vp);
    if(vp.n>0 && strncmp(vp.param[0],"$",1)==0) return 0; //
    if(vp.n > 0 && strncmp(vp.param[0],"def",3)==0){
        strcpy(vex->das[vex->ndas].def,vp.param[1]); // save def param
        vex->das[vex->ndas].earlysec=0;              // default set
//printf("DEBUG %s\n",vex->das[vex->ndas].def);

        if(k > 0) goto loop01;
loop00:
        if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
loop01:
        k=vex_parse(line, &vp);
        if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0){
             vex->ndas++;
			 // added on 2009-11-19
             vex->ndas_true++;
             if(vex->ndas >= MAXDAS) {
                printf("DAS_ana: Number of data ack terminals exceeds limit (%d)\n",MAXDAS);
                vex->ndas -= 1;    // to avoid array overflow
             }
			 //
             goto loop;
        }
        if(vp.n>0 && strncmp(vp.param[0],"tape_motion",11)==0){ // 
            if(strncmp(vp.param[2],"start&stop",10)==0 ||
                  strncmp(vp.param[2],"adaptive",8)==0){ // get an early parameter
                vex->das[vex->ndas].earlysec=atoi(vp.param[4]);
                if(strncmp(vp.param[5],"min",3)==0) {
                    vex->das[vex->ndas].earlysec*=60;
                }
            }
        }
        if(k > 0) goto loop01;
        goto loop00;
    }
    if(k > 0) goto loop1;
    goto loop;
 */
}


int sub_das_ana(FILE* fin, char *line, das_def_t *das)  //2009-11-19
{
/* 
     return   0: one "def .... enddef" block is sucessfully analyzed
	         -1: eof encountered before reach enddef
			  1: encounter "$" which means the start of next block
	          
*/

	int k;
    //vparam_t vp;
    vparam_t *vp;  // 2020-06-26

    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26
    das->def[0]='\0';
	das->earlysec=0;

loop1:
    //k=vex_parse(line, &vp);
    k=vex_parse(line, vp); // 2020-06-26
    //if(vp.n > 0 && strncmp(vp.param[0],"$",1)==0) return 1; 
    if (vp->n > 0 && strncmp(vp->param[0], "$", 1) == 0) {  // 2020-06-26
        free(vp);
        return 1;
    }
    if(vp->n > 0 && strncmp(vp->param[0],"def",3)==0){
        strcpy(das->def,vp->param[1]); // save def param
        das->earlysec=0;              // default set
        if(k > 0) goto loop01;
loop00:
        //if(fgets(line, 500, fin ) == NULL) return -1; //goto exitend;  // 1 line read
        if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
            free(vp);
            return -1; //goto exitend;  // 1 line read 2020-06-26
        }
    loop01:
        //k=vex_parse(line, &vp);
        k=vex_parse(line, vp);  // 2020-06-26
        //if(vp.n>0 && strncmp(vp.param[0],"enddef",6)==0) return 0;
        if (vp->n > 0 && strncmp(vp->param[0], "enddef", 6) == 0) {  // 2020-06-26
            free(vp);
            return 0;
        }
        if(vp->n>0 && strncmp(vp->param[0],"tape_motion",11)==0){ // 
           if(strncmp(vp->param[2],"start&stop",10)==0 ||
                  strncmp(vp->param[2],"adaptive",8)==0){ // get an early parameter
               das->earlysec=atoi(vp->param[4]);
               if(strncmp(vp->param[5],"min",3)==0) das->earlysec*=60;
           }
		}
        if(k > 0) goto loop01;
        goto loop00;
    }
    if(k > 0) goto loop1;
    //if(fgets(line, 500, fin ) == NULL) return -1;
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        free(vp);
        return -1; 
    }
    goto loop1;
}

void MODE_mon(vex_t *vex)  //2003-03-21
{
/*      $MODE block monitor  */
    int i,k;
    printf("MODE %d\n",vex->nmode);
    for (k=0; k<vex->nmode; k++){  
        printf(" %s\n",vex->mode[k].def);
        for (i=0;i<vex->mode[k].nmode;i++){
          printf("%d %s %s\n",i+1,vex->mode[k].mode_freq[i].mode, vex->mode[k].mode_freq[i].id);
          printf("%d %s %s\n",i+1,vex->mode[k].mode_track[i].mode, vex->mode[k].mode_track[i].id);
          printf("%d %s %s\n",i+1,vex->mode[k].mode_roll[i].mode, vex->mode[k].mode_roll[i].id);
          printf("%d %s %s\n",i+1,vex->mode[k].mode_pcal_detect[i].mode, vex->mode[k].mode_pcal_detect[i].id);
        } 
    } 
}

void SKED_mon(vex_t *vex)  //2003-03-21
{
/*      $SCHED block monitor  */
    int i,k;
    printf("NUMOBS  %d\n",vex->numobs);
    for (i=0; i<vex->numobs; i++){
        
        printf("%d %s %s %s %d %d %d %d %d",
            vex->scan[i].number,vex->scan[i].scanid, vex->scan[i].mode,
            vex->scan[i].source,
            vex->scan[i].start[0],
            vex->scan[i].start[1],
            vex->scan[i].start[2],
            vex->scan[i].start[3],
            vex->scan[i].start[4]);
        //printf("numstat = %d\n", vex->scan[i].numstat);
        for(k=0; k< vex->scan[i].numstat; k++){
            printf("  %s ",vex->scan[i].station[k].id);
            printf("  %d %d  ",
                vex->scan[i].station[k].data_start,vex->scan[i].station[k].data_stop);
        }
        printf("\n");
    }
}

void TRACKS_mon(vex_t *vex)  //2003-03-21
{
/*      $TRACKS block monitor  */
    // 2020-04-13  add bit/sampl
    // 2020-06-15  minor revision

    int i,k,m;
    printf("NUMTRACK  %d\n",vex->ntrack_def);
    for (i=0; i<vex->ntrack_def; i++){
        printf("%d %s %s\n",i+1,vex->track_def[i].def,vex->track_def[i].format);
        //printf("%d\n",vex->track_def[i].nfanout_def);
        printf("   data modulation : %d\n",vex->track_def[i].data_modulation);  // 2020-05-13
        for(k=0; k<vex->track_def[i].nfanout_def; k++){
            printf("   %s ",vex->track_def[i].fanout_def[k].sub_pass);
            printf("   %d",vex->track_def[i].fanout_def[k].fanout);
            printf(" %s ",vex->track_def[i].fanout_def[k].chan_id);
            //printf(" %s ",vex->track_def[i].fanout_def[k].stream_info);
            printf(" %4s ",vex->track_def[i].fanout_def[k].stream_info);
            for(m=0; m< vex->track_def[i].fanout_def[k].fanout; m++){
                printf(" %2d",vex->track_def[i].fanout_def[k].track[m]);
            }
            printf("\n");
        }
        printf("   bit/sampl = %d\n", vex->track_def[i].bitssampl);  // 2020-04-13
    }
}


void TRACKS_check_adbit(vex_t *vex)  //2020-06-15
{
    /*      $TRACKS block check adbit for both sign and mag tracks are the same  */
        // 2020-06-15

    int i, k, m, kkai;
    int tr1, tr2;
    int flag_1bit;
    char chan1[20], chan2[20];
    char strem1[10], strem2[10];
    for (i = 0; i < vex->ntrack_def; i++) {
        flag_1bit = 0;
        if (vex->track_def[i].bitssampl == 1) continue;  // this adbit=1, so nothing to do

        kkai = vex->track_def[i].nfanout_def;
        //printf("%d %s %s\n", i + 1, vex->track_def[i].def, vex->track_def[i].format);
        //printf("%d\n",vex->track_def[i].nfanout_def);
        //printf("   data modulation : %d\n", vex->track_def[i].data_modulation);  // 2020-05-13
        for (k = 0; k < kkai/2; k++) {
            strcpy(chan1, vex->track_def[i].fanout_def[2*k].chan_id);
            strcpy(chan2, vex->track_def[i].fanout_def[2*k+1].chan_id);
            strcpy(strem1, vex->track_def[i].fanout_def[2 * k].stream_info);
            strcpy(strem2, vex->track_def[i].fanout_def[2 * k+1].stream_info);
            tr1 = vex->track_def[i].fanout_def[2 * k].track[0];
            tr2 = vex->track_def[i].fanout_def[2 * k+1].track[0];
            if (strncmp(chan1, chan2, strlen(chan2)) == 0) {
                if (strncmp(strem1, strem2, strlen(strem2)) != 0) {
                    if (tr1 == tr2) flag_1bit = 1;
                }
            }
        }
        if (flag_1bit == 1) {  // edit info
            // edit info
            vex->track_def[i].bitssampl = 1;
            vex->track_def[i].nfanout_def=kkai/2;
            for (k = 0; k < kkai / 2; k++) {
                strcpy(vex->track_def[i].fanout_def[k].sub_pass,
                    vex->track_def[i].fanout_def[2 * k].sub_pass);
                vex->track_def[i].fanout_def[k].fanout = vex->track_def[i].fanout_def[2 * k].fanout;
                strcpy(vex->track_def[i].fanout_def[k].chan_id,
                    vex->track_def[i].fanout_def[2 * k].chan_id);
                strcpy(vex->track_def[i].fanout_def[k].stream_info,
                    vex->track_def[i].fanout_def[2 * k].stream_info);
                for (m = 0; m < vex->track_def[i].fanout_def[k].fanout; m++) {
                    vex->track_def[i].fanout_def[k].track[m] = vex->track_def[i].fanout_def[2 * k].track[m];
                }
            }
        }
    }
}

void SITE_mon(vex_t *vex)  //2003-03-21
{
/*      $SITE block monitor  */
    // 2020-04-12  add mk4_id

    int i,k;
    printf("NUMSITE  %d\n",vex->numstat);
    for (i=0; i<vex->numstat; i++){
        //printf("%d %s %s %s\n",i+1,vex->site[i].def,vex->site[i].name,vex->site[i].id);
        printf("%d %s %s %s %s\n",i+1,
            vex->site[i].def,vex->site[i].name,vex->site[i].id, vex->site[i].mk4_id); // 2020-04-12
        for(k=0; k<3; k++){
            printf(" %f\n",vex->site[i].position[k]);
        }
    }
}

void STATION_mon(vex_t *vex)  //2009-11-19
{
/*      $STATION block monitor  */
    int i,n;
    printf("NUMSTATION  %d\n",vex->nstation);
    for (i=0; i<vex->nstation; i++){
        //printf("%d %s %s %s %s %s\n",i+1,vex->station[i].def,vex->station[i].site,
            //vex->station[i].antenna,vex->station[i].clock,vex->station[i].das);
        printf("%d %s %s %s %s",i+1,vex->station[i].def,vex->station[i].site,
            vex->station[i].antenna,vex->station[i].clock);//2009-11-19
		/* added on 2009-11-19 */
		//for(n=0; n<MAXDAS-1; n++){
        for(n=0; n<vex->ndas_max-1; n++){  // 2016-09-16
			if(vex->station[i].das[n][0] != '\0'){
				printf(" %s",vex->station[i].das[n]);
			} else {
				printf("\n");
				break;
			}
		}
		/* add end */
    }
}

void SOURCE_mon(vex_t *vex)  //2003-03-21
{
/*      $SOURCE block monitor  */
    int i;
    printf("NUMSTAR  %d\n",vex->numstar);
    for (i=0; i<vex->numstar; i++){
        printf("%d %s %s %s ",i+1,vex->source[i].def,vex->source[i].name,vex->source[i].IAU_name);
        printf(" %f %f %s \n",vex->source[i].ra,vex->source[i].dec,vex->source[i].epoch);
    }
}

void FREQ_mon(vex_t *vex)  //2003-03-21
{
/*      $FREQ block monitor  */
    // 2020-04-13 revised

    int i,k;
    printf("NUMFREQ def %d\n",vex->nfreq_def);
    for (i=0; i<vex->nfreq_def; i++){
        printf("%d %s %f\n",i+1,vex->freq_def[i].def,vex->freq_def[i].samplerate);
        for(k=0; k<vex->freq_def[i].numch; k++){
           //printf(" %f %s %f %s %s %s\n",vex->freq_def[i].chan_def[k].rf,
           printf(" %s %f %s %f %s %s %s\n",vex->freq_def[i].chan_def[k].chan_id0,  // 2020-04-13
               vex->freq_def[i].chan_def[k].rf,
               vex->freq_def[i].chan_def[k].sideband,
               vex->freq_def[i].chan_def[k].vbw,
               vex->freq_def[i].chan_def[k].chan_id,
               vex->freq_def[i].chan_def[k].bbc_id,
               vex->freq_def[i].chan_def[k].pcal_id);
        }
    }
}

void CLOCK_mon(vex_t *vex)  //2003-03-21   revised 2018-02-02 
{
/*      $CLOCK block monitor  */
    int i,k;
    printf("NUMCLOCK def %d\n",vex->nclock);
    if(vex->nclock <= 0){                                        // 2018-02-02
        printf(" No clock information in the VEX file!\n");      // 2018-02-02
    } else {                                                     // 2018-02-02
        for (i=0; i<vex->nclock; i++){
            printf("%d %s\n",i+1,vex->siteclock[i].def);
            printf(" validity epoch: ");
            for(k=0; k<5; k++) printf(" %d",vex->siteclock[i].validity_epoch[k]);
            printf("\n");
            printf(" clock early : %e\n",vex->siteclock[i].clock_offset);
            printf(" clock epoch : ");
            for(k=0; k<5; k++) printf(" %d",vex->siteclock[i].clock_epoch[k]);
            printf("\n");
            printf(" clock rate  : %e\n",vex->siteclock[i].clock_rate);
        }
    }                                                            // 2018-02-02
}


void CLOCK_mon2(vex_t *vex, char *stat_id)  //2003-07-13
{
/*      $CLOCK block monitor only for stat_id */
    int i,k;
    char cw1[20];
    strcpy(cw1,"--");
    k=-1;
    //find station key for stat_id
    for(i=0; i<vex->nstation; i++){
       if(strncmp(stat_id,vex->station[i].def,strlen(stat_id))==0){
            strcpy(cw1,vex->station[i].clock);
            break;
       }
    }
    for(i=0; i<vex->nclock; i++){
       if(strncmp(cw1,vex->siteclock[i].def,strlen(cw1))==0){
           k=i;
           break;
       }
    }
    if(k < 0){
        printf("Clock parameters not found for %s\n",stat_id);
        goto exit2return;
    }
    //printf("NUMCLOCK def %d\n",vex->nclock);
    printf("  %s %s\n",stat_id,vex->siteclock[k].def);
    printf("   validity epoch: ");
    for(i=0; i<5; i++) printf("   %d",vex->siteclock[k].validity_epoch[i]);
    printf("\n");
    printf("   clock early : %e\n",vex->siteclock[k].clock_offset);
    printf("   clock epoch : ");
    for(i=0; i<5; i++) printf("   %d",vex->siteclock[k].clock_epoch[i]);
    printf("\n");
    printf("   clock rate  : %e\n",vex->siteclock[k].clock_rate);
exit2return:;
}

//void CLOCK_get(vex_t *vex, char *stat_id, siteclock_t *clock)  //2017-09-04
int CLOCK_get(vex_t *vex, char *stat_id, siteclock_t *clock)  //2017-09-04
{
/*      Get clock parameter */
    int i,k;
    int ierr;    // 2018-02-02
    char cw1[20];

    ierr=0;   // 2018-02-02
    strcpy(cw1,"--");
    strcpy(clock->def,"--");
    for(i=0; i<5; i++){
       clock->clock_epoch[i]=0;
       clock->validity_epoch[i]=0;
    }
    clock->clock_offset=0.0;
    clock->clock_rate=0.0;
    k=-1;
    //find station key for stat_id  // obsoleted on 2017-09-04
    //for(i=0; i<vex->nstation; i++){
    //   if(strncmp(stat_id,vex->station[i].def,strlen(stat_id))==0){
    //        strcpy(cw1,vex->station[i].clock);
    //        break;
    //   }
    //}
    strcpy(cw1,stat_id);  // 2017-09-04
    for(i=0; i<vex->nclock; i++){
       if(strncmp(cw1,vex->siteclock[i].def,strlen(cw1))==0){
           k=i;
           break;
       }
    }
    if(k < 0){
        printf("   CLOCK_get: Clock parameters not found for %s\n",stat_id);
        ierr=-1;              // 2018-02-02
        goto exit2return;
    }
    //printf("NUMCLOCK def %d\n",vex->nclock);
    strcpy(clock->def,vex->siteclock[k].def);
    for(i=0; i<5; i++) {
        clock->validity_epoch[i]=vex->siteclock[k].validity_epoch[i];
        clock->clock_epoch[i]=vex->siteclock[k].clock_epoch[i];
    }
    clock->clock_offset=vex->siteclock[k].clock_offset;
    clock->clock_rate=vex->siteclock[k].clock_rate;
exit2return:;
    return ierr;    // 2018-02-02
}

void EOP_mon(vex_t *vex)  //2003-03-21
{
/*      $EOP block monitor  */
    int i,k,kk;
    printf("NUMEOP def %d\n",vex->neop);
    for (i=0; i<vex->neop; i++){
        printf("%d %s\n",i+1,vex->eop[i].def);
        printf(" eop ref epoch   : ");
        for(k=0; k<5; k++) printf(" %d",vex->eop[i].epoch[k]);
        printf("\n");
        printf(" eop_interval(hr): %d\n",vex->eop[i].interval_hr);
        printf(" eop_ponts : %d\n",vex->eop[i].points);
        kk=vex->eop[i].points;
        printf(" ut1-utc  : ");
        for(k=0; k<kk; k++) printf(" %f",vex->eop[i].ut1[k]);
        printf("\n");
        printf(" x_wobble : ");
        for(k=0; k<kk; k++) printf(" %f",vex->eop[i].xwob[k]);
        printf("\n");
        printf(" y_wobble : ");
        for(k=0; k<kk; k++) printf(" %f",vex->eop[i].ywob[k]);
        printf("\n");
    }
}

void EOP_mon2(vex_t *vex)  //2003-03-21
{
/*      $EOP block monitor  */
    int i,k,kk;
    k=-1;
    //find EOP data set
    for(i=0; i<vex->neop; i++){
       if(strncmp(vex->global.eop,vex->eop[i].def,strlen(vex->global.eop))==0){
           k=i;
           break;
       }
    }
    if(k<0) {
        printf("   No EOP data for this experiment\n");
        goto exit2return;
    }

    //printf("NUMEOP def %d\n",vex->neop);
    printf("   %s\n",vex->eop[k].def);
    printf("   eop ref epoch   : ");
    printf("  %d/%03d %02d:%02d:%02d\n",
                vex->eop[k].epoch[0],
                vex->eop[k].epoch[1],
                vex->eop[k].epoch[2],
                vex->eop[k].epoch[3],
                vex->eop[k].epoch[4]);
    printf("   eop_interval(hr): %d\n",vex->eop[k].interval_hr);
    printf("   eop_points      : %d\n",vex->eop[k].points);
    kk=vex->eop[k].points;
    printf("   ut1-utc (s)     : ");
    for(i=0; i<kk; i++) printf(" %f",vex->eop[k].ut1[i]);
    printf("\n");
    printf("   x_wobble (asec) : ");
    for(i=0; i<kk; i++) printf(" %f",vex->eop[k].xwob[i]);
    printf("\n");
    printf("   y_wobble (asec) : ");
    for(i=0; i<kk; i++) printf(" %f",vex->eop[k].ywob[i]);
    printf("\n");
exit2return:;
}

void PCAL_mon(vex_t *vex)  //2003-07-16
{
/*      $PCAL block monitor  */
    int i,kk,m;
    printf("vex.npcal %d\n",vex->npcal);
    for(i=0; i<vex->npcal; i++){

        printf("PCAL def %s\n",vex->pcal[i].def);
        for(m=0; m< vex->pcal[i].npcal; m++){
            printf("  %s\n",vex->pcal[i].pcal[m].pcal_id);
            for(kk=0; kk< vex->pcal[i].pcal[m].ntone; kk++){
                printf(" %d\n",vex->pcal[i].pcal[m].tone[kk]);
            }
        }
    }
}

void BBC_mon(vex_t *vex)  //2003-07-16
{
/*      $BBC block monitor  */
    int i,m;
    printf("vex.nbbc %d\n",vex->nbbc);
    for(i=0; i<vex->nbbc; i++){

        printf("BBC def %s\n",vex->bbc[i].def);
        for(m=0; m< vex->bbc[i].nbbc; m++){
            printf("  %s ",vex->bbc[i].bbc_def[m].bbc_id);
            printf("  %d ",vex->bbc[i].bbc_def[m].bbcch);
            printf("  %s\n",vex->bbc[i].bbc_def[m].if_id);
        }
    }
}

void IF_mon(vex_t *vex)  //2003-07-16
{
/*      $IF block monitor  */
    int i,m;
    printf("vex.nifb %d\n",vex->nifb);
    for(i=0; i<vex->nifb; i++){

        printf("IF def %s\n",vex->ifb[i].def);
        for(m=0; m< vex->ifb[i].nif; m++){
            printf("  %s ",vex->ifb[i].if_def[m].if_id);
            printf("  %s ",vex->ifb[i].if_def[m].pol);
            printf("  %f ",vex->ifb[i].if_def[m].lo);
            printf("  %s ",vex->ifb[i].if_def[m].sideband);
            printf("  %f ",vex->ifb[i].if_def[m].pcal_space);
            printf("  %f\n",vex->ifb[i].if_def[m].pcal_base);
        }
    }
}

void DAS_mon(vex_t *vex)  //2005-07-25
{
/*      $DAS block monitor  */
    int k;
    printf("DAS %d\n",vex->ndas);
    for (k=0; k<vex->ndas; k++){  
        printf(" %s : earlystart %d sec\n",vex->das[k].def,vex->das[k].earlysec);
    } 
}

void  vex_read(vex_t *vex)  
{
    /*
       VEX file read and analyze by $BLOCK
       2018-02-02 update
       2019-10-11 revised to avoid $SCHEDULING_PARAMS; keyword to $SCHED;
    */
     int k,loop;
     //char line[500];
     char *line;   // 2020-06-26
     FILE *fin;
     //vparam_t vp;
     vparam_t *vp;  // 2020-06-26


     if( (fin  = fopen(vex->vex_file_name, "r" )) == NULL ){
        printf("vex_read: File (%s) not found or Open Error! \n",vex->vex_file_name);
        exit(1);
     } 

     line = malloc(_LINE_SIZE); // 2020-06-26
     vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26

	 /* initialize variables */
	 vex->numobs=0;       //2009-11-19
	 vex->numobs_true=0;  //2009-11-19
	 vex->numstar=0;      //2009-11-19
	 vex->numstar_true=0; //2009-11-19
	 vex->numstat=0;      //2009-11-19
	 vex->numstat_true=0; //2009-11-19
	 vex->ndas=0;         //2009-11-19
	 vex->ndas_true=0;    //2009-11-19
     vex->neop=0;       // 2018-02-02
     vex->nclock=0;     // 2018-02-02

     /* initialize counter */
     vex->nblock=0;
     loop=0;
     // first line analysis to identify VEX or OVEX
     //if( fgets(line, 500, fin ) == NULL) goto exitend;
     if( fgets(line, _LINE_SIZE, fin ) == NULL) goto exitend;  // 2020-06-26
     //k=vex_parse(line, &vp);
     k=vex_parse(line, vp);  // 2020-06-26
     if(strncmp(vp->param[0],"$OVEX",5)==0){  // This is OVEX format
         strcpy(vex->block[vex->nblock],vp->param[0]); vex->nblock++;
         strcpy(vex->flag,"OVEX");
     } else {
         strcpy(vex->flag,"VEX");
     }

mainloop:
     //if( fgets( line, 500, fin ) == NULL) goto exitend;
     if( fgets( line, _LINE_SIZE, fin ) == NULL) goto exitend;  // 2020-06-26
     //printf("%s",line);
loop1:
     //k=vex_parse(line, &vp);
     k=vex_parse(line, vp);  // 2020-06-26
     //if(line[0]=='$') printf("%s\n",line);

     //block analysis
     if(vp->n <= 0) goto mainloop;
     if(strncmp(vp->param[0],"$",1)==0){  // found block name
         strcpy(vex->block[vex->nblock],vp->param[0]); 
         vex->nblock++;
     }

     /* $GLOBAL analysis */
     if(strncmp(vp->param[0],"$GLOBAL",7)==0){  // GLOBAL
         //printf("GLOBAL analysis start\n");
         if(GLOBAL_ana(fin, line, vex)==-1) goto exitend;
         //printf("GLOBAL analysis end\n");
         //printf("%s %s\n",vex->global.exper, vex->global.eop);
         goto loop1;
     } 

     /* $EXPER analysis */
     if(strncmp(vp->param[0],"$EXPER",7)==0){  // EXPER
        //printf("EXPER analysis start\n");
        if(EXPER_ana(fin, line, vex)==-1) goto exitend;
        //printf("EXPER analysis end\n");
        //printf("exper = %s\n",vex->exper[0].exper_name);
        goto loop1;
     }

     /* $MODE analysis */
     if(strncmp(vp->param[0],"$MODE",5)==0){  // MODE
        //printf("MODE analysis start\n");
        if(MODE_ana(fin, line, vex)==-1) goto exitend;
        //printf("MODE analysis end\n");
        goto loop1;
     }

     /* $SOURCE analysis (moved from jst below $station) 2016-1012 */
     if(strncmp(vp->param[0],"$SOURCE",7)==0){  // SOURCE
         //printf("SOURCE analysis start\n");
         if(SOURCE_ana(fin, line, vex)==-1) goto exitend;
         //printf("SOURCE analysis end\n");
         goto loop1;
     }

     /* $SCHED analysis */
     //if(strncmp(vp.param[0],"$SCHED",6)==0){  // SCHED
     if((strncmp(vp->param[0],"$SCHED",6)==0) && strlen(vp->param[0])==6){  // SCHED;  
                                                           // to avoid $SCHEDULING_PARAMS; 2019-10-11
         //printf("SCHED analysis start\n");
         if(SKED_ana(fin, line, vex)==-1) goto exitend;
         //printf("SCHED analysis end\n");
         goto loop1;
     }

     /* $TRACKS analysis */
     if(strncmp(vp->param[0],"$TRACKS",7)==0){  // TRACKS
         //printf("TRACK analysis start\n");
         if(TRACKS_ana(fin, line, vex)==-1) goto exitend;
         //TRACKS_mon(vex);  // debug
         //printf("TRACK analysis end\n");
         goto loop1;
     }

     /* $SITE analysis */
     if(strncmp(vp->param[0],"$SITE",5)==0){  // SITE
         //printf("SITE analysis start\n");
         if(SITE_ana(fin, line, vex)==-1) goto exitend;
         //printf("SITE analysis end\n");
         goto loop1;
     }

     /* $STATION analysis */
     if(strncmp(vp->param[0],"$STATION",8)==0){  // SITE
         //printf("STATION analysis start\n");
         if(STATION_ana(fin, line, vex)==-1) goto exitend;
         //printf("STATION analysis end\n");
         goto loop1;
     }

     /* $SOURCE analysis */
     /*  moved to just before $SCHED analysis    2016-10-12
     if(strncmp(vp.param[0],"$SOURCE",7)==0){  // SOURCE
         //printf("SOURCE analysis start\n");
         if(SOURCE_ana(fin, line, vex)==-1) goto exitend;
         //printf("SOURCE analysis end\n");
         goto loop1;
     }
     */

     /* $FREQ analysis */
     if(strncmp(vp->param[0],"$FREQ",5)==0){  // FREQ
         //printf("FREQ analysis start\n");
         if(FREQ_ana(fin, line, vex)==-1) goto exitend;
         //printf("FREQ analysis end\n");
         goto loop1;
     }

     /* $CLOCK analysis */
     if(strncmp(vp->param[0],"$CLOCK",6)==0){  // CLOCK
         //printf("CLOCK analysis start\n");
         if(CLOCK_ana(fin, line, vex)==-1) goto exitend;
         //printf("CLOCK analysis end\n");
         goto loop1;
     }

     /* $EOP analysis */
     if(strncmp(vp->param[0],"$EOP",4)==0){  // EOP
         //printf("EOP analysis start\n");
         if(EOP_ana(fin, line, vex)==-1) goto exitend;
         //printf("EOP analysis end\n");
         goto loop1;
     }

     /* $ROLL analysis */
     if(strncmp(vp->param[0],"$ROLL",5)==0){  // ROLL
         //printf("ROLL analysis start\n");
         if(ROLL_ana(fin, line, vex)==-1) goto exitend;
         //printf("ROLL analysis end\n");
         //printf("ROLL %s %s\n",vex->roll[0].def,vex->roll[0].rolldef);
         goto loop1;
     }


     /* $PHASE_CAL_DETECT analysis */   //added on 2003-07-17
     if(strncmp(vp->param[0],"$PHASE_CAL_DETECT",17)==0){  // PHASE_CAL
         //printf("PHASE_CAL analysis start\n");
         if(PCAL_ana(fin, line, vex)==-1) goto exitend;
         //printf("PHASE_CAL analysis end\n");
         //PCAL_mon(vex);
         goto loop1;
     }

     /* $BBC analysis */   //added on 2003-07-17

     if(strncmp(vp->param[0],"$BBC",4)==0){  // BBC
         //printf("BBC analysis start\n");
         if(BBC_ana(fin, line, vex)==-1) goto exitend;
         //printf("BBC analysis end\n");
         //printf(" BBC debug %d\n",vex->nbbc);
         
         //for (k = 0; k < 64; k++) {  // debug
         //    printf("  Debug k %d  %s %s\n", k, vex->bbc[0].bbc_def[k].bbc_id,
         //        vex->bbc[0].bbc_def[k].if_id);  // debug
         //}
         
         //BBC_mon(vex);
         goto loop1;
     }

     /* $IF analysis */  

     if(strncmp(vp->param[0],"$IF",3)==0){  // IF
         //printf("IF analysis start\n");
         if(IF_ana(fin, line, vex)==-1) goto exitend;
         //printf("IF analysis end\n");
         //printf(" IF debug %d\n",vex->nifb);
         //IF_mon(vex);
         goto loop1;
     }

     /* $DAS analysis */  // added on 2005-07-25

     if(strncmp(vp->param[0],"$DAS",4)==0){  // IF
         //printf("DAS analysis start\n");
         if(DAS_ana(fin, line, vex)==-1) goto exitend;
         //printf("DAS analysis end\n");
         //printf(" DAS debug %d\n",vex->ndas);
         //DAS_mon(vex);
         goto loop1;
     }
             
     if(k==1) goto loop1;
     loop++;
     //if(loop==50) goto exitend; // for check
     goto mainloop;
exitend:;

     k=fclose(fin); // 2016-09-14
     free(line);  // 2020-06-26
     free(vp);    // 2020-06-26

     /*      $SCHED block monitor  */
  //   SKED_mon(vex);

     /* $TRACKS monitor */
  //   TRACKS_mon(vex);

    /* $STATION monitor */
  //   STATION_mon(vex);

    /* $SITE monitor */
  //   SITE_mon(vex);

    /* $SOURCE monitor */
  //   SOURCE_mon(vex);

    /* $FREQ monitor */
  //   FREQ_mon(vex);

    /* $MODE monitor */
  //   MODE_mon(vex);

    /* $CLOCK monitor */
  //   CLOCK_mon(vex);

    /* $EOP monitor */
  //   EOP_mon(vex);
}


int search_site(vex_t *vex, char *key, site_t *site)  /* 2017-09-04 */
{
/* 
   Get site info correspond to 'key' in $MODE (station id)
       return=-1; not found   =0; found
*/

    int i,id,id2,iok;
    id=-1;
    for(i=0; i<vex->nstation; i++){
       if(strncmp(vex->station[i].def,key,strlen(key))==0){  // STATION found
              //printf("%s %s %s %s\n",vex.station[i].def,vex.station[i].site,
              //         vex.station[i].antenna,vex.station[i].clock);
          id=i;
          break;
       }
    }
    if(id < 0) return -1;

    // site position get
    id2=-1;
    for(i=0; i<vex->numstat; i++){
       if(strncmp(vex->station[id].site,vex->site[i].def,strlen(vex->site[i].def))==0){  // STATION found
            //printf("%s %s %s\n",vex.site[i].def,vex.site[i].name,
            //           vex.site[i].id);
            //printf("%f %f %f\n",vex.site[i].position[0],
            //            vex.site[i].position[1],vex.site[i].position[2]);
            *site=vex->site[i];
            id2=i;
            break;
       }
    }
    if(id2<0) return -1;

    // site clock info get
    iok=-1;
    for(i=0; i<vex->nclock; i++){
       //if(strncmp(vex->station[id].clock,vex->siteclock[i].def,strlen(vex->siteclock[i].def))==0){  // STATION found
       if(strncmp(vex->siteclock[i].def,key,strlen(key))==0){  // STATION found   2017-09-04
            //printf("%s %s %s\n",vex.site[i].def,vex.site[i].name,
            //           vex.site[i].id);
            //printf("%f %f %f\n",vex.site[i].position[0],
            //            vex.site[i].position[1],vex.site[i].position[2]);
            site->sclock=vex->siteclock[i];
            iok=0;
            break;
       }
    }
    if(iok<0) {
        printf("search_site: No CLOCK info for %s included in the VEX FILE.\n",
            site->name);
        printf("search_site: So all 0 for clock information was set.\n");
        for(i=0; i<5; i++) site->sclock.clock_epoch[i]=0;
        for(i=0; i<5; i++) site->sclock.validity_epoch[i]=0;
        site->sclock.clock_offset=0.0;
        site->sclock.clock_rate=0.0;
        site->sclock.def[0]='\0';
    }
    return 0;
}


int search_site2(vex_t *vex, char *key, site_t *site)  /* 2017-09-04 */
{
/*    as same as search_site but suppress unnecessary warning message
   Get site info correspond to 'key' in $MODE (station id)
       return=-1; not found   =0; found
*/

    int i,id,id2,iok;
    id=-1;
    for(i=0; i<vex->nstation; i++){
       if(strncmp(vex->station[i].def,key,strlen(key))==0){  // STATION found
              //printf("%s %s %s %s\n",vex.station[i].def,vex.station[i].site,
              //         vex.station[i].antenna,vex.station[i].clock);
          id=i;
          break;
       }
    }
    if(id < 0) return -1;

    // site position get
    id2=-1;
    for(i=0; i<vex->numstat; i++){
       if(strncmp(vex->station[id].site,vex->site[i].def,strlen(vex->site[i].def))==0){  // STATION found
            //printf("%s %s %s\n",vex.site[i].def,vex.site[i].name,
            //           vex.site[i].id);
            //printf("%f %f %f\n",vex.site[i].position[0],
            //            vex.site[i].position[1],vex.site[i].position[2]);
            *site=vex->site[i];
            id2=i;
            break;
       }
    }
    if(id2<0) return -1;

    // site clock info get
    iok=-1;
    for(i=0; i<vex->nclock; i++){
       //if(strncmp(vex->station[id].clock,vex->siteclock[i].def,strlen(vex->siteclock[i].def))==0){  // STATION found
       if(strncmp(vex->siteclock[i].def,key,strlen(key))==0){  // STATION found   2017-09-04
            //printf("%s %s %s\n",vex.site[i].def,vex.site[i].name,
            //           vex.site[i].id);
            //printf("%f %f %f\n",vex.site[i].position[0],
            //            vex.site[i].position[1],vex.site[i].position[2]);
            site->sclock=vex->siteclock[i];
            iok=0;
            break;
       }
    }
    if(iok<0) {
        //printf("search_site: No CLOCK info for %s included in the VEX FILE.\n",
        //    site->name);
        //printf("search_site: So all 0 for clock information was set.\n");
        for(i=0; i<5; i++) site->sclock.clock_epoch[i]=0;
        for(i=0; i<5; i++) site->sclock.validity_epoch[i]=0;
        site->sclock.clock_offset=0.0;
        site->sclock.clock_rate=0.0;
        site->sclock.def[0]='\0';
    }
    return 0;
}


freq_info_t vex_freq_get(vex_t *vex, char *mode, char *sid, char *chanid)
//2003-10-28
{
    int i,ipos,k,kf,iok, kp,j,kbbc,kif;
    char freqid[30];//, chanid[30];
    char pcalid[30], pcalmode[50], bbcid[30],bbcmode[50],ifid[30],ifmode[50];
    int tones[10],ntone;
    int nji;
    double pcalspace,pcalbase, pcalf;
    freq_info_t freq;

    // initialize
    freq.npcal=0;
    freq.pcalf[0]=-1.0;
    freq.npcal=0;
    freq.rf=0.0;
    freq.sideband[0]='\0';
    freq.bbcid[0]='\0';
    freq.vbw=0.0;
    strcpy(freq.chanid,chanid);
    freq.nch=0;
    freq.chmax=0;
    // first mode selection
    ipos=-1;
    for(i=0; i<vex->nmode; i++){
        if(strncmp(vex->mode[i].def,mode,strlen(mode))==0) {  // MODE found
           ipos=i;
           break;
        }
    }
    if(ipos<0) return freq;
    // freq mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
//printf("pas1 %d %s\n",i,vex->mode[ipos].mode_freq[i].id);
        if(strncmp(vex->mode[ipos].mode_freq[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(freqid,vex->mode[ipos].mode_freq[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    // bbc mode finding for sid

    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
//printf("pas2 %d %s\n",i,vex->mode[ipos].mode_bbc[i].id);
        if(strncmp(vex->mode[ipos].mode_bbc[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(bbcmode,vex->mode[ipos].mode_bbc[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    // ifmode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
//printf("pas3 %d %s\n",i,vex->mode[ipos].mode_if[i].id);
        if(strncmp(vex->mode[ipos].mode_if[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(ifmode,vex->mode[ipos].mode_if[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    // pcal_detection mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
//printf("pas4 %d %s\n",i,vex->mode[ipos].mode_pcal_detect[i].id);
        if(strncmp(vex->mode[ipos].mode_pcal_detect[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(pcalmode,vex->mode[ipos].mode_pcal_detect[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    /*freq data set up*/
    iok=-1;
    for(i=0; i<vex->nfreq_def; i++){
        if(strncmp(vex->freq_def[i].def,freqid,strlen(freqid))==0) {  // MODE found
            kf=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    // get frequency info from chanid
    //printf("debug chan id %s\n",chanid);//debug
    iok=-1;
    freq.chmax=vex->freq_def[kf].numch;
    for(k=0; k<vex->freq_def[kf].numch; k++){
        if(strncmp(vex->freq_def[kf].chan_def[k].chan_id,chanid,strlen(chanid))==0) {  // MODE found
            freq.rf=vex->freq_def[kf].chan_def[k].rf;
            strcpy(freq.sideband,vex->freq_def[kf].chan_def[k].sideband);
            freq.vbw=vex->freq_def[kf].chan_def[k].vbw;
 //printf("debug chan id  k freq.rf %s %d %f\n",chanid,k,freq.rf);//debug
            // pcal id get
            strcpy(pcalid,vex->freq_def[kf].chan_def[k].pcal_id);
            // bbc id get
            strcpy(bbcid,vex->freq_def[kf].chan_def[k].bbc_id);
            strcpy(freq.bbcid,bbcid); //2004-12-25
            freq.nch=k+1;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    iok=-1;

    //pcal tone info get
    for(i=0; i<vex->npcal; i++){
        if(strncmp(vex->pcal[i].def,pcalmode,strlen(pcalmode))==0) {  // MODE found
            kp=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    iok=-1;
    for(k=0; k< vex->pcal[kp].npcal; k++){
        if(strncmp(vex->pcal[kp].pcal[k].pcal_id,pcalid,strlen(pcalid))==0){
            ntone=vex->pcal[kp].pcal[k].ntone;
            for(j=0; j<ntone; j++){
                tones[j]=vex->pcal[kp].pcal[k].tone[j];
            }
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    //pcal freq info get through BBC info
    iok=-1;
    for(i=0; i<vex->nbbc; i++){
        if(strncmp(vex->bbc[i].def,bbcmode,strlen(bbcmode))==0) {  // MODE found
            kbbc=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    iok=-1;
    for(k=0; k< vex->bbc[kbbc].nbbc; k++){
        if(strncmp(vex->bbc[kbbc].bbc_def[k].bbc_id,bbcid,strlen(bbcid))==0){
            strcpy(ifid,vex->bbc[kbbc].bbc_def[k].if_id);  //if_id get
            freq.bbc=vex->bbc[kbbc].bbc_def[k].bbcch;  // BBC# get 2004-12-25
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    //if_id analyze
    iok=-1;
    for(i=0; i<vex->nifb; i++){
        if(strncmp(vex->ifb[i].def,ifmode,strlen(ifmode))==0) {  // MODE found
            kif=i;
            iok=0;
            break;
        }
    }
    for(k=0; k< vex->ifb[kif].nif; k++){
        if(strncmp(vex->ifb[kif].if_def[k].if_id,ifid,strlen(ifid))==0){
            pcalspace=vex->ifb[kif].if_def[k].pcal_space;
            pcalbase =vex->ifb[kif].if_def[k].pcal_base;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    //pcal frequency calc

    if(fabs(pcalspace) < 0.1 ) return freq;
    // following is valid for USB  
    if(freq.sideband[0]=='U' || freq.sideband[0]=='u'){
       /* new algorithm was applied on 2003-10-28 */
       if(ntone > 0){
           for(i=0; i<ntone; i++){
               k=tones[i];
               if(k > 0) {
                   nji=(int)(freq.rf/pcalspace);
                   pcalf=pcalspace*(double)(nji+k);
                   freq.pcalf[i]=pcalf-freq.rf;
               }
               if(k < 0) {
                   nji=(int)((freq.rf+freq.vbw)/pcalspace);
                   pcalf=pcalspace*(double)(nji+k+1);
                   freq.pcalf[i]=pcalf-freq.rf;
               }
               if(k == 0) {  // nothing to do
                   freq.pcalf[i]=0.0;   
               }
           }
       }
    }
    // following is valid for LSB  
    if(freq.sideband[0]=='L' || freq.sideband[0]=='l'){
       /* new algorithm was applied on 2003-10-28 */
       if(ntone > 0){
           for(i=0; i<ntone; i++){
               k=tones[i];
               if(k > 0) {
                   nji=(int)(freq.rf/pcalspace);
                   pcalf=pcalspace*(double)(nji-k+1);
                   freq.pcalf[i]=freq.rf-pcalf;
               }
               if(k < 0) {
                   nji=(int)((freq.rf-freq.vbw)/pcalspace);
                   pcalf=pcalspace*(double)(nji-k);
                   freq.pcalf[i]=freq.rf-pcalf;
               }
               if(k == 0) {  // nothing to do
                   freq.pcalf[i]=0.0;   
               }
           }
       }
    }
    freq.npcal=ntone;

    return freq;
}


freq_info_t vex_chinfo_get(vex_t *vex, char *mode, char *sid, int nch)
// 2003-07-17
// 2019-06-11
// 2020-01-11 revise to handle the case that $PHASE_CAL_DETECT is missed in a VEX file
// 2020-10-10 revised
//  
{
    int i,ipos,k,kf,iok, kp,j,kbbc,kif;
    char cbuf[10];  // 2019-06-11
    char freqid[30];//, chanid[30];
    //char pcalid[30], pcalmode[50], bbcid[30],bbcmode[50],ifid[30],ifmode[50];
    char pcalmode[50], bbcmode[50],ifmode[50];
    char trackmode[50]; // 2020-10-10
    int tones[10],ntone;
    int nji;
    int ch_table[64];
    double pcalspace,pcalbase, pcalf;
    freq_info_t freq;

    // initialize
    // initialize
    freq.npcal=0;
    freq.pcalf[0]=-1.0;
    freq.npcal=0;
    freq.rf=0.0;
    freq.sideband[0]='\0';
    freq.vbw=0.0;
    freq.chanid[0]='\0';
    freq.nch=nch;
    freq.ch = 0; // 2019-06-13
    freq.chmax=0;
    freq.pol[0] = '-';  // 2019-06-14
    freq.pol[1] = '\0';  // 2019-06-14
    freq.bbcid[0] = '\0';  // 2019-06-11
    freq.pcalid[0] = '\0';  // 2019-06-11
    freq.ifid[0] = '\0';  // 2019-06-11
    freq.thread = 0;      // 2019-06-11
    // first mode selection
    ipos=-1;
    for(i=0; i<vex->nmode; i++){
        if(strncmp(vex->mode[i].def,mode,strlen(mode))==0) {  // MODE found
           ipos=i;
           break;
        }
    }
    if(ipos<0) return freq;
    // freq mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
//printf("pas1 %d %s\n",i,vex->mode[ipos].mode_freq[i].id);
        if(strncmp(vex->mode[ipos].mode_freq[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(freqid,vex->mode[ipos].mode_freq[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    // bbc mode finding for sid

    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
//printf("pas2 %d %s\n",i,vex->mode[ipos].mode_bbc[i].id);
        if(strncmp(vex->mode[ipos].mode_bbc[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(bbcmode,vex->mode[ipos].mode_bbc[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    // ifmode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_if[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(ifmode,vex->mode[ipos].mode_if[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    // pcal_detection mode finding for sid
    pcalmode[0] = '\0';  // 2020-01-11
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_pcal_detect[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(pcalmode,vex->mode[ipos].mode_pcal_detect[i].mode);
            iok=0;
            break;
        }
    }
    //if(iok<0) return freq;   // obsoleted on 2020-01-11

    /*freq data set up*/
    iok=-1;
    for(i=0; i<vex->nfreq_def; i++){
        if(strncmp(vex->freq_def[i].def,freqid,strlen(freqid))==0) {  // MODE found
            kf=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    freq.chmax=vex->freq_def[kf].numch;
    //chanid get for nch
    if(nch <=0 || nch > vex->freq_def[kf].numch) return freq;
    strcpy(freq.chanid,vex->freq_def[kf].chan_def[nch-1].chan_id);
    // get CH# and thread # from chanid 2019-06-11
    if (strlen(freq.chanid) == 6) {  // thiis is "&CHAXX" type  2019-06-11
        cbuf[0] = freq.chanid[4]; cbuf[1] = freq.chanid[5]; cbuf[2] = '\0'; // 2019-06-13
        freq.ch = atoi(cbuf);  // 2019-06-13
        if (freq.chanid[3] == 'A' || freq.chanid[3] == 'a') freq.thread = 0;
        if (freq.chanid[3] == 'B' || freq.chanid[3] == 'b') freq.thread = 1;
        if (freq.chanid[3] == 'C' || freq.chanid[3] == 'c') freq.thread = 2;
        if (freq.chanid[3] == 'D' || freq.chanid[3] == 'd') freq.thread = 3;
        if (freq.chanid[3] == 'E' || freq.chanid[3] == 'e') freq.thread = 4;
        if (freq.chanid[3] == 'F' || freq.chanid[3] == 'f') freq.thread = 5;
        if (freq.chanid[3] == 'G' || freq.chanid[3] == 'g') freq.thread = 6;
        if (freq.chanid[3] == 'H' || freq.chanid[3] == 'h') freq.thread = 7;  
    }
    else if (strlen(freq.chanid) == 5) {
        cbuf[0] = freq.chanid[3]; cbuf[1] = freq.chanid[4]; cbuf[2] = '\0'; // 2019-06-13
        freq.ch = atoi(cbuf);  // 2019-06-13
    }
    freq.rf=vex->freq_def[kf].chan_def[nch-1].rf;
    strcpy(freq.sideband,vex->freq_def[kf].chan_def[nch-1].sideband);
    freq.vbw=vex->freq_def[kf].chan_def[nch-1].vbw;
    // pcal id get
    //strcpy(pcalid,vex->freq_def[kf].chan_def[nch-1].pcal_id);
    strcpy(freq.pcalid,vex->freq_def[kf].chan_def[nch-1].pcal_id);  // 2019-06-11
    // bbc id get
    //strcpy(bbcid,vex->freq_def[kf].chan_def[nch-1].bbc_id);
    strcpy(freq.bbcid,vex->freq_def[kf].chan_def[nch-1].bbc_id);  // 2019-06-11

    // ** re-setup freq.ch using $TRACKS data  (not implemented as of 2020-10-10)  **

    // track mode finding for sid
    iok = -1;
    for (i = 0; i < vex->mode[ipos].nmode; i++) {
        if (strncmp(vex->mode[ipos].mode_track[i].id, sid, strlen(sid)) == 0) {  // TRACK MODE found
            strcpy(trackmode, vex->mode[ipos].mode_track[i].mode);
            iok = 0;
            break;
        }
    }
    //if (iok < 0) return freq;
    if (iok == 0) { // re-setup ch infor using track number
         anal_track_then_conv_to_ch(trackmode, vex, ch_table);
         freq.ch = ch_table[nch - 1];
    }
    //  *****  add end  *****

    iok=-1;

    //pcal tone info get

    if (pcalmode[0] == '\0') {  // 2020-01-11
        // set default (not implemented) should be properly set to pretend as follows
        //
        // $PHASE_CAL_DETECT;
        // def Standard;
        // phase_cal_detect = &U_cal : 1;
        // phase_cal_detect = &L_cal : 1;
        // enddef;

        vex->npcal = 1;
        vex->pcal[0].npcal = 2;
        strcpy(vex->pcal[0].pcal[0].pcal_id, "&U_cal");
        vex->pcal[0].pcal[0].ntone=1;
        vex->pcal[0].pcal[0].tone[0]=1;
        strcpy(vex->pcal[0].pcal[1].pcal_id, "&L_cal");
        vex->pcal[0].pcal[1].ntone=1;
        vex->pcal[0].pcal[1].tone[0]=1;
        kp = 0;
        goto bypass_pcalmode;
    }


    for(i=0; i<vex->npcal; i++){
        if(strncmp(vex->pcal[i].def,pcalmode,strlen(pcalmode))==0) {  // MODE found
            kp=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

bypass_pcalmode:;  // 2020-01-11

    iok=-1;
    for(k=0; k< vex->pcal[kp].npcal; k++){
        //if(strncmp(vex->pcal[kp].pcal[k].pcal_id,pcalid,strlen(pcalid))==0){
        if(strncmp(vex->pcal[kp].pcal[k].pcal_id,freq.pcalid,strlen(freq.pcalid))==0){ //2019-06-11
           ntone=vex->pcal[kp].pcal[k].ntone;
            for(j=0; j<ntone; j++){
                tones[j]=vex->pcal[kp].pcal[k].tone[j];
            }
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;

    //pcal freq info get through BBC info
    iok=-1;
    for(i=0; i<vex->nbbc; i++){
        if(strncmp(vex->bbc[i].def,bbcmode,strlen(bbcmode))==0) {  // MODE found
            kbbc=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    iok=-1;
    for(k=0; k< vex->bbc[kbbc].nbbc; k++){
        //if(strncmp(vex->bbc[kbbc].bbc_def[k].bbc_id,bbcid,strlen(bbcid))==0){
        if(strncmp(vex->bbc[kbbc].bbc_def[k].bbc_id,freq.bbcid,strlen(freq.bbcid))==0){ // 2019-06-11
            //strcpy(ifid,vex->bbc[kbbc].bbc_def[k].if_id);  //if_id get
            strcpy(freq.ifid,vex->bbc[kbbc].bbc_def[k].if_id);  //if_id get  2019-06-11
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    //if_id analyze
    iok=-1;
    for(i=0; i<vex->nifb; i++){
        if(strncmp(vex->ifb[i].def,ifmode,strlen(ifmode))==0) {  // MODE found
            kif=i;
            iok=0;
            break;
        }
    }
    for(k=0; k< vex->ifb[kif].nif; k++){
        //if(strncmp(vex->ifb[kif].if_def[k].if_id,ifid,strlen(ifid))==0){
        if(strncmp(vex->ifb[kif].if_def[k].if_id,freq.ifid,strlen(freq.ifid))==0){  // 2019-06-11
            pcalspace=vex->ifb[kif].if_def[k].pcal_space;
            pcalbase =vex->ifb[kif].if_def[k].pcal_base;
            strcpy(freq.pol, vex->ifb[kif].if_def[k].pol);     // 2019-06-11
            iok=0;
            break;
        }
    }
    if(iok<0) return freq;
    //pcal frequency calc
    iok=-1;

    if(fabs(pcalspace) < 0.1 ) return freq;
    // following is valid for USB now 
    if(freq.sideband[0]=='U' || freq.sideband[0]=='u'){
       nji=(int)(freq.rf/pcalspace)-1;
       k=0;
       i=0;
       if(ntone > 0) {
          while(1){
             pcalf=pcalspace*(double)nji;
             if(pcalf > freq.rf) k++;
             if(k==tones[i]){
                freq.pcalf[i]=pcalf-freq.rf;
                i++;
                if(i >= ntone) break;
             } 
            nji++;
          } 
       } 
    }
    // following is valid for LSB now 
    if(freq.sideband[0]=='L' || freq.sideband[0]=='l'){
       nji=(int)(freq.rf/pcalspace)+2;
       k=0;
       i=0;
       if(ntone > 0) {
          while(1){
             pcalf=pcalspace*(double)nji;
             if(pcalf < freq.rf) k++;
             if(k==tones[i]){
                freq.pcalf[i]=freq.rf-pcalf;
                i++;
                if(i >= ntone) break;
             } 
            nji--;
          } 
       } 
    }
    freq.npcal=ntone;

    return freq;
}



int search_track_freq(vex_t *vex, char *mode, char *sid, track_freq_t *trfreq)
//2004-12-25  
{
/*
       Get track and frequency info correspond to 'def' mode in $MODE for 
       station id sid
       return=-1; not found   =0; found
       pcal frequency info was added to search_track_freq2
       
       2004-06-29 sub-pass was added in track_freq_sub
       2004-12-25 BBC# was added in track_freq_sub

*/

    int i,ipos,k,kk,kt,kf,iok,m;
    char freqid[30], trackid[30], rollid[30], chanid[30];
    freq_info_t freq_info;

    trfreq->ntrack=0;
    // first mode selection
    ipos=-1;
    for(i=0; i<vex->nmode; i++){
        if(strncmp(vex->mode[i].def,mode,strlen(mode))==0) {  // MODE found
           ipos=i;
           break;
        }
    }
    if(ipos<0) return -1;
    // freq mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_freq[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(freqid,vex->mode[ipos].mode_freq[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;
    strcpy(trfreq->freqdef,freqid);
 
    // track mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_track[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(trackid,vex->mode[ipos].mode_track[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;
    strcpy(trfreq->trackdef,trackid);

    // roll mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_roll[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(rollid,vex->mode[ipos].mode_roll[i].mode);
            //printf("DEBUG!!: %s\n",rollid);
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;


    //track data set up
    iok=-1;
    for(i=0; i<vex->ntrack_def; i++){
        if(strncmp(vex->track_def[i].def,trackid,strlen(trackid))==0) {  // MODE found
            kt=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;

    trfreq->data_modulation = vex->track_def[kt].data_modulation;  // 2020-05-13
    strcpy(trfreq->frame_format, vex->track_def[kt].format);       // 2020-05-13

    //freq data set up
    iok=-1;
    for(i=0; i<vex->nfreq_def; i++){
        if(strncmp(vex->freq_def[i].def,freqid,strlen(freqid))==0) {  // MODE found
            kf=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;

    //found roll mode and value
    for(i=0; i<vex->nroll; i++){
        if(strncmp(vex->roll[i].def,rollid,strlen(rollid))==0) {  // MODE found
           strcpy(trfreq->roll,vex->roll[i].rolldef);
           break;
        }
    }

    trfreq->samplrate=vex->freq_def[kf].samplerate;
    trfreq->adbit=1;
    trfreq->nfreq=vex->freq_def[kf].numch;

    m=0;
    trfreq->fanout=vex->track_def[kt].fanout_def[0].fanout;
    //printf("Debug 1 %d \n",trfreq->fanout);
    for(i=0; i<vex->track_def[kt].nfanout_def; i++){
        for(k=0; k<vex->track_def[kt].fanout_def[i].fanout; k++){
            strcpy(trfreq->trk[m].chan_id,vex->track_def[kt].fanout_def[i].chan_id);
            strcpy(chanid,vex->track_def[kt].fanout_def[i].chan_id);
            //printf("%s\n",vex->track_def[kt].fanout_def[m].stream_info);
            strcpy(trfreq->trk[m].bitstream,vex->track_def[kt].fanout_def[i].stream_info);
            if(strncmp(trfreq->trk[m].bitstream,"mag",3)==0) trfreq->adbit=2;

            trfreq->trk[m].fan_out_pos=k+1;
            trfreq->trk[m].track=vex->track_def[kt].fanout_def[i].track[k];
            //printf("%d %d %d\n",i,k,vex->track_def[kt].fanout_def[m].track[k]);
            strcpy(trfreq->trk[m].sub_pass,vex->track_def[kt].fanout_def[i].sub_pass); //2004-06-29
            trfreq->trk[m].headstack=vex->track_def[kt].fanout_def[i].headstack; // added 2004-09-13
            
            // get frequency info from chan id
            //printf("debug mode,sid,chan id %s %s %s\n",mode,sid,chanid);//debug
            freq_info=vex_freq_get(vex,mode,sid,chanid);
            //printf("i,k %d %d\n",i,k);  //debug
            trfreq->trk[m].rf=freq_info.rf;
            strcpy(trfreq->trk[m].sideband,freq_info.sideband);
            trfreq->trk[m].vbw=freq_info.vbw;
            trfreq->trk[m].npcal=freq_info.npcal;
            trfreq->trk[m].bbc  =freq_info.bbc; //2004-12-25
            strcpy(trfreq->trk[m].bbcid,freq_info.bbcid); //2004-12-25
            for(kk=0; kk< freq_info.npcal; kk++){
                trfreq->trk[m].pcalf[kk]=freq_info.pcalf[kk];
            }
            m++;
        }
    }
    trfreq->ntrack=m;


    return 0;
}

int search_track_freq2(vex_t *vex, char *mode, char *sid, track_freq_t *trfreq)
//2003-07-12 debug
{
/*
       Get track and frequency info correspond to 'def' mode in $MODE for 
       station id sid
       return=-1; not found   =0; found
       pcal frequency info was addwd to search_track_freq2
*/

    int i,ipos,k,kk,kt,kf,iok,m;
    char freqid[30], trackid[30], rollid[30], chanid[30];

    trfreq->ntrack=0;
    // first mode selection
    ipos=-1;
    for(i=0; i<vex->nmode; i++){
        if(strncmp(vex->mode[i].def,mode,strlen(mode))==0) {  // MODE found
           ipos=i;
           break;
        }
    }
    if(ipos<0) return -1;
    // freq mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_freq[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(freqid,vex->mode[ipos].mode_freq[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;
    strcpy(trfreq->freqdef,freqid);
 
    // track mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_track[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(trackid,vex->mode[ipos].mode_track[i].mode);
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;
    strcpy(trfreq->trackdef,trackid);

    // roll mode finding for sid
    iok=-1;
    for(i=0; i<vex->mode[ipos].nmode; i++){
        if(strncmp(vex->mode[ipos].mode_roll[i].id,sid,strlen(sid))==0) {  // MODE found
            strcpy(rollid,vex->mode[ipos].mode_roll[i].mode);
            //printf("DEBUG!!: %s\n",rollid);
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;


    //track data set up
    iok=-1;
    for(i=0; i<vex->ntrack_def; i++){
        if(strncmp(vex->track_def[i].def,trackid,strlen(trackid))==0) {  // MODE found
            kt=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;

    //freq data set up
    iok=-1;
    for(i=0; i<vex->nfreq_def; i++){
        if(strncmp(vex->freq_def[i].def,freqid,strlen(freqid))==0) {  // MODE found
            kf=i;
            iok=0;
            break;
        }
    }
    if(iok<0) return -1;

    //found roll mode and value
    for(i=0; i<vex->nroll; i++){
        if(strncmp(vex->roll[i].def,rollid,strlen(rollid))==0) {  // MODE found
           strcpy(trfreq->roll,vex->roll[i].rolldef);
           break;
        }
    }

    trfreq->samplrate=vex->freq_def[kf].samplerate;
    trfreq->adbit=1;
    trfreq->nfreq=vex->freq_def[kf].numch;

    m=0;
    trfreq->fanout=vex->track_def[kt].fanout_def[0].fanout;
    for(i=0; i<vex->track_def[kt].nfanout_def; i++){
        for(k=0; k<vex->track_def[kt].fanout_def[i].fanout; k++){
            strcpy(trfreq->trk[m].chan_id,vex->track_def[kt].fanout_def[i].chan_id);
            strcpy(chanid,vex->track_def[kt].fanout_def[i].chan_id);
            //printf("%s\n",vex->track_def[kt].fanout_def[m].stream_info);
            strcpy(trfreq->trk[m].bitstream,vex->track_def[kt].fanout_def[i].stream_info);
            if(strncmp(trfreq->trk[m].bitstream,"mag",3)==0) trfreq->adbit=2;

            trfreq->trk[m].fan_out_pos=k+1;
            trfreq->trk[m].track=vex->track_def[kt].fanout_def[i].track[k];
            //printf("%d %d %d\n",i,k,vex->track_def[kt].fanout_def[m].track[k]);
            
            // get frequency info from chan id
            //printf("debug chan id %s\n",chanid);//debug
            iok=-1;
            for(kk=0; kk<vex->freq_def[kf].numch; kk++){
               if(strncmp(vex->freq_def[kf].chan_def[kk].chan_id,chanid,strlen(chanid))==0) {  // MODE found
                   //printf("debug chan id  kk %s %d %f\n",chanid,kk,);//debug
                   iok=0;
                   //trfreq->trk[m].rf=vex->freq_def[kf].chan_def[k].rf;
                   //strcpy(trfreq->trk[m].sideband,vex->freq_def[kf].chan_def[k].sideband);
                   //trfreq->trk[m].vbw=vex->freq_def[kf].chan_def[k].vbw;
                   trfreq->trk[m].rf=vex->freq_def[kf].chan_def[kk].rf;                     //2003-07-12
                   strcpy(trfreq->trk[m].sideband,vex->freq_def[kf].chan_def[kk].sideband); //2003-07-12
                   trfreq->trk[m].vbw=vex->freq_def[kf].chan_def[kk].vbw;                   //2003-07-12
               } 
            } 
            if(iok<0) return -1;
            m++;
        }
    }
    trfreq->ntrack=m;
    return 0;
}


void create_siteid_table(vex_t *vex, int *nstation, sidtable_t *siteid)  //2003-03-21
{
/*
    Create site id (corresponds to 'def' keyword in $STAION) table
*/

    int i;
    *nstation=vex->nstation;
    for(i=0; i<vex->nstation; i++){
       strcpy(siteid[i].sid,vex->station[i].def);
       strcpy(siteid[i].site,vex->station[i].site);
    }
}



int check_block(vex_t *vex)  //2003-03-21
{
/* 
     Check $BLOCK names that contains all blocks necessary for
     correlation processing or not

     return 0: OK,  -1 for no good

*/

    int i,k,iok,iok2;
    char kijun1[][20]={"$GLOBAL","$MODE","$STATION","$SITE","$TRACKS",
        "$FREQ","$ROLL","$SCHED","$SOURCE","$CLOCK","$EOP",
        "$BBC","$IF","$PHASE_CAL_DETECT"};
    int nkijun1=14;
    char *kijun2="$EOP";
    iok2=0;
    for(k=0; k<nkijun1; k++){
       iok=-1;
       for(i=0; i<vex->nblock; i++){
           if(strncmp(vex->block[i],kijun1[k],strlen(kijun1[k]))==0) {
               iok=0;
               break;
           }
       }
       if(iok<0){
           printf("%s is not found in VEX file\n",kijun1[k]);
           iok2=-1;
       }
    }
    if(iok2<0) return -1;

    return 0;
}


/* added on 2013-01-28 */

int TrimTK(char *s) {    
	/* 
	   trimg initial and trail spaces and 
	   trail CRLF if existed
	*/ 
    int i;
    int count = 0;

    /* NULL pointer? */
    if ( s == NULL ) { /* yes */
        return -1;
    }

    /* get string length */
    i = strlen(s);

    /* find last non space position */
	while ( --i >= 0){
		if(s[i]==' ' || s[i]=='\r' ||  s[i]=='\n' ){
			count++;
		} else {
			break;
		}
	}

    /* add NULL at last */
    s[i+1] = '\0';

    /* find first non space position */
    i = 0;
    while ( s[i] != '\0' && s[i] == ' ' ) i++;
    strcpy(s, &s[i]);

    return i + count;
}

void pick_up_sked_codes_section(char *skdname, codes_t *code)  //2013-01-28   
{
	/*
	  pick up $CODES section in a sked file

	*/
    //char line[1000];
    char *line;  // 2020-06-26
    int i;
    FILE *stream;

    //printf("%s \n",skdname);

    if( (stream  = fopen(skdname, "r" )) == NULL ){
       printf("pick_up_sked_codes_section: File (%s) not found or Open Error! \n",skdname);
       exit(1);
       //goto exitend2;
    }
    //printf("File %s opened\n",>skdname);
    line = malloc(_LINE_SIZE); // 2020-06-26

	// initialize code data
	code->ncode=0;
	//for(i=0; i<MAXCODES; i++){
    for(i=0; i<code->ncode_max; i++){ // 2016-09-16
		code->ctab[i].nfreq=0;
		code->ctab[i].nstation=0;
	}
	code->fsample=0.0;

    /* Schedule File $CODES section Analysis */    


loop:
     //if( fgets( line, 1000, stream ) == NULL) goto exitend;
     if( fgets( line, _LINE_SIZE, stream ) == NULL) goto exitend;  // 2020-06-26
     if(line[0]=='*') goto loop;  // this is comment line (nothing to do)  
       
     if(line[0]=='$'){    // this is command line  
       if(strncmp(line,"$CODES",5)==0){       // $CODES block (FREQ info)   
          //printf("readskd: $CODES Detected! \n");
          i=readskd_codes(stream, line, code);
		  goto exitend;
       } 
     } 
     goto loop;
 exitend:
     i=fclose( stream ) ;
     free(line);
}



int readskd_codes(FILE *fin, char *line, codes_t *code)
{
    // 2016-09-29 revised
    // 2019-12-19 renewed

    int k;
    int ch;
    int tus, tls;
    int tum, tlm, adbit; 
    int flagRead;
    int ierr;
    double d1;
    double pcal, vbw;
    double temp_lsb_freq[16], temp_lsb_pcal[16], temp_lsb_vbw[16];
    int    temp_lsb_adbit[16];  // 2018-08-15
    int nlsb, nfreq, ncode, nstat;
    //char s1[21],s2[21],s3[21],s4[21];
    //char line[500];
    //vparam_t vparam;
    vparam_t *vparam;  // 2020-06-26

    vparam = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26

    nfreq = 0;
    ncode = 0;
    nfreq = 0;
    nstat = 0;
    nlsb = 0;
    flagRead = 0;
    ierr = 0;
loop:
    //if (fgets(line, 500, fin) == NULL) {
    if (fgets(line, _LINE_SIZE, fin) == NULL) {  // 2020-06-26
        ierr = -1;
        goto exit2return;  // no more line
    }
    if (line[0] == '*') goto loop;  // this is comment line (nothing to do)  
    if (line[0] == '$')  goto exit2return;   // next declaration line 
    if (line[0] == 'F') {    // frequency table (including next) start
        if (flagRead == 1) {  // if read frequency info at least 1 line then
            // add LSB info if there are 
            if (nlsb > 0) {
                for (k = 0; k < nlsb; k++) {
                    code->ctab[ncode].freqtab[nfreq] = temp_lsb_freq[k];
                    code->ctab[ncode].vbw[nfreq] = temp_lsb_vbw[k];
                    //code->ctab[ncode].pcalf[0][nfreq] = temp_lsb_pcal[k];
                    //code->ctab[ncode].pcalf[1][nfreq] = 0.0;
                    code->ctab[ncode].pcalf[nfreq][0] = temp_lsb_pcal[k]; // 2019-12-21
                    code->ctab[ncode].pcalf[nfreq][1] = 0.0; // 2019-12-21
                    code->ctab[ncode].adbit[nfreq] = temp_lsb_adbit[k];  // 2019-12-19
                    strcpy(code->ctab[ncode].sideband[nfreq], "L");
                    nfreq++;
                }
            }
            code->ctab[ncode].nfreq = nfreq;
            nlsb = 0;
            nfreq = 0;
            flagRead = 0;
            ncode++;
        }

        // get station name
        TrimTK(line);
        //line_parse(line, &vparam);
        line_parse(line, vparam);  // 2020-06-26
        code->ctab[ncode].nstation = 0;
        if (vparam->n > 3) {   // there is station name
            for (k = 0; k < vparam->n - 3; k++) {
                strcpy(code->ctab[ncode].statnam[k], vparam->param[k + 3]);
            }
            code->ctab[ncode].nstation = vparam->n - 3;
        }
    }
    if (line[0] == 'C') {   // Frequency info
       //k=sscanf(line, "%*s %*s %*s %s %s %d %*s %s %s", s1,s2,&i1,s3,s4); 
       //pcal=atof(s2);
       //vbw=atof(s3);
       //parseSBinfo(s4, &tus, &tls);    // libskdvex function
        //line_parse(line, &vparam);
        line_parse(line, vparam);  // 2020-06-26
        pcal = atof(vparam->param[4]);
        vbw = atof(vparam->param[7]);
        ch = atoi(vparam->param[5]);  // channel #
        //parseSBinfo(vparam.param[8],&tus, &tls);    // libskdvex function
        parseSBinfo(vparam->param[8], &tus, &tls, &tum, &tlm);    // libskdvex function  2016-08-15


        //printf("%d %f %f %d %d\n",ch, pcal,vbw,tus,tls);   // debug
        //if(s1[0] != 'x' && s1[0] != 'X') {    // to avoid  xxxx.xx or XXXX.XX
        if (vparam->param[3][0] != 'x' && vparam->param[3][0] != 'X') {    // to avoid  xxxx.xx or XXXX.XX
            flagRead = 1;
            //d1=atof(s1);
            d1 = atof(vparam->param[3]);
            //printf("d1= %f\n",d1); //debug
            code->ctab[ncode].freqtab[ch - 1] = d1;
            code->ctab[ncode].vbw[ch - 1] = vbw;
            //code->ctab[ncode].pcalf[0][ch - 1] = pcal;
            //code->ctab[ncode].pcalf[1][ch - 1] = 0.0;
            code->ctab[ncode].pcalf[ch - 1][0] = pcal;  // 2019-12-21
            code->ctab[ncode].pcalf[ch - 1][1] = 0.0;   // 2019-12-21
            if (tus != -99) {                                 // 2016-08-15
                adbit = 1;
                if (tum != -99) adbit = 2;
                code->ctab[ncode].adbit[ch - 1] = adbit;
                if (tls != -99) {
                    adbit = 1;
                    if (tlm != -99) adbit = 2;
                }
            }

            if (code->usb == 0) { // compulsory set side band to LSB mode
                strcpy(code->ctab[ncode].sideband[ch - 1], "L");
            }
            else {
                if (tus != -99) strcpy(code->ctab[ncode].sideband[ch - 1], "U");
                if (tus == -99 && tls != -99) strcpy(code->ctab[ncode].sideband[ch - 1], "L");
                if (tus != -99 && tls != -99) {       //
                   // save LSB data
                    temp_lsb_freq[nlsb] = d1;
                    temp_lsb_pcal[nlsb] = 1.0e6 - pcal;  // valid only in case of 1 MHz PCAL
                    temp_lsb_vbw[nlsb] = vbw;
                    temp_lsb_adbit[nlsb] = adbit;    // 2016-08-15
                    nlsb++;
                }
            }
            nfreq++;
        }
    }
    if (line[0] == 'R') {    // Sampling frequecny 
        // get sampling frequency
        //line_parse(line, &vparam);
        line_parse(line, vparam);  // 2020-06-26
        code->fsample = atof(vparam->param[2])*1.0e6;
    }
    goto loop;
exit2return:

    free(vparam); // 2020-06-26

    // update sampling frequency
    if (code->fsample == 0.0) {                           // 2016-09-29
        code->fsample = code->ctab[0].vbw[0] * 2.0*1.0e6;   // 2016-09-29
    }                                                   // 2016-09-29

    // add LSB info if there are 
    if (flagRead == 1) {
        if (nlsb > 0) {
            for (k = 0; k < nlsb; k++) {
                //if(nfreq >= MAXCH) break;  // 2016-08-10
                if (nfreq >= code->ch_max) break;  // 2016-09-15
                code->ctab[ncode].freqtab[nfreq] = temp_lsb_freq[k];
                code->ctab[ncode].vbw[nfreq] = temp_lsb_vbw[k];
                //code->ctab[ncode].pcalf[0][nfreq] = temp_lsb_pcal[k];
                //code->ctab[ncode].pcalf[1][nfreq] = 0.0;
                code->ctab[ncode].pcalf[nfreq][0] = temp_lsb_pcal[k]; // 2019-12-21
                code->ctab[ncode].pcalf[nfreq][1] = 0.0;              // 2019-12-21
                code->ctab[ncode].adbit[nfreq] = temp_lsb_adbit[k];  // 2016-08-15
                strcpy(code->ctab[ncode].sideband[nfreq], "L");
                // debug
                //printf(" Ncode %d CH# %d  lsb adbit %d\n", ncode, nfreq + 1, code->ctab[ncode].adbit[nfreq]);
                nfreq++;
            }
        }
        code->ctab[ncode].nfreq = nfreq;
        ncode++;
    }
    code->ncode = ncode;
    return ierr;

}


/* added on 2016-09-14 */


//void  get_nelement_VEX(char *vex_file_name, int *nscan,  int *nstation,  int *nstar)
void  get_nelement_VEX(vex_t *vex)  // 2016-09-16
{
/*
      set up maximum size of each element in vex_t type variable

      some are set by default and some are read from a VEX file

      2018-02-08  debug  (no value was substituted to "vex->nstation_max" in some VEX file) 

*/


     int n;
     FILE *fin;


     // set up default not read from a VEX file

     vex->nblock_max=50;
     //vex->nexper_max=10;
     //vex->nmode_max=10;
     //vex->nfreq_def_max=20;
     //vex->nclock_max=vex->nstation_max;
     //vex->neop_max=10;
     //vex->nroll_max=10;
     //vex->npcal_max=MAXPCAL_B;
     //vex->nbbc_max=MAXBBC_B;
     //vex->nifb_max=MAXIF_B;
     //vex->ndas_max=MAXDAS;

     if( (fin  = fopen(vex->vex_file_name, "r" )) == NULL ){
        printf("get_nelement_VEX: File (%s) not found or Open Error! \n",vex->vex_file_name);
        exit(1);
     }

     vex->nexper_max    =get_VEX_nKEY(fin,"$EXPER");
     vex->numobs_max    =get_VEX_nKEY(fin,"$SCHED");
     vex->nstation      =get_VEX_nKEY(fin,"$STATION");
     vex->nstation_max=vex->nstation;                        // 2018-02-08
     vex->numstat_max   =get_VEX_nKEY(fin,"$SITE");
     n=get_VEX_nKEY(fin,"$ANTENNA");
     if(n > vex->numstat_max) vex->numstat_max=n;
     if(n > vex->nstation_max) vex->nstation_max=n;
     vex->numstar_max   =get_VEX_nKEY(fin,"$SOURCE");
     vex->nfreq_def_max =get_VEX_nKEY(fin,"$FREQ");
     vex->nifb_max      =get_VEX_nKEY(fin,"$IF");
     vex->nbbc_max      =get_VEX_nKEY(fin,"$BBC");
     vex->npcal_max     =get_VEX_nKEY(fin,"$PHASE_CAL_DETECT");
     vex->ntrack_def_max=get_VEX_nKEY(fin,"$TRACKS");
     vex->nmode_max     =get_VEX_nKEY(fin,"$MODE");
     vex->ndas_max      =get_VEX_nKEY(fin,"$DAS");
     vex->nroll_max     =get_VEX_nKEY(fin,"$ROLL");
     vex->neop_max      =get_VEX_nKEY(fin,"$EOP");
     vex->nclock_max    =get_VEX_nKEY(fin,"$CLOCK");

     n = get_nIDs_in_VEX_MODE(fin);    // 2019-05-22
     if (n > vex->numstat_max) vex->numstat_max = n; // 2019-05-22

     // zero check to avoid unnecessary error
     if(vex->nexper_max == 0) vex->nexper_max=1;
     if(vex->numobs_max == 0) vex->numobs_max=1;
     if(vex->numstat_max == 0) vex->numstat_max=1;
     if(vex->nstation_max == 0) vex->nstation_max=1;
     if(vex->numstar_max == 0) vex->numstar_max=1;
     if(vex->nfreq_def_max == 0) vex->nfreq_def_max=1;
     if(vex->nifb_max == 0) vex->nifb_max=1;
     if(vex->nbbc_max == 0) vex->nbbc_max=1;
     if(vex->npcal_max == 0) vex->npcal_max=1;
     if(vex->ntrack_def_max == 0) vex->ntrack_def_max=1;
     if(vex->nmode_max == 0) vex->nmode_max=1;
     if(vex->ndas_max == 0) vex->ndas_max=1;
     if(vex->nroll_max == 0) vex->nroll_max=1;
     if(vex->neop_max == 0) vex->neop_max=1;
     if(vex->nclock_max == 0) vex->nclock_max=1;


     //vex->nclock_max=vex->nstation_max;

     n=fclose(fin);
}


/* get # of elements $KEYWORD in VEX file */
int get_VEX_nKEY(FILE* fin, char *keyword)    // 2016-09-14
{
/*
    keyword should be "$XXXXXX"

  examples:
      $SCHED,  $STATION,   $SOURCE,   
      $SITE,  $ANTENNA, $FREQ,
      $IF,     $BBC,      $PHASE_CAL_DETECT, $TRACKS,  $MODE

*/
    int k,n;
    //int k,loop;
    //char line[500];
    char *line;   // 2020-06-26
    char subkey[10];
    //vparam_t vp;
    vparam_t *vp;  // 2020-06-26

    line = malloc(_LINE_SIZE);  // 2020-06-26
    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26

    // rewind file

    rewind(fin);

    if(strncmp(keyword,"$SCHED",6)==0){
        strcpy(subkey,"scan");
    } else {
        strcpy(subkey,"def");
    }

    n=0;

    // find $xxxxx field block

loop:
     //if( fgets( line, 500, fin ) == NULL) goto exitend;
     if( fgets( line, _LINE_SIZE, fin ) == NULL) goto exitend;  // 2020-06-26
     //k=vex_parse(line, &vp);
     k=vex_parse(line, vp);  // 2020-06-26
     if(vp->n <= 0) goto loop;

     //if(strncmp(vp.param[0],keyword,strlen(keyword))!=0) goto loop;
     if(strncmp(vp->param[0],keyword,strlen(vp->param[0]))!=0) goto loop;  // 2016-10-12

     /* $xxxxxx found */

loop2:
    
     //if(fgets(line, 500, fin ) == NULL) goto exitend;
     if(fgets(line, _LINE_SIZE, fin ) == NULL) goto exitend;  // 2020-06-26
     //k=vex_parse(line, &vp);
     k=vex_parse(line, vp);  // 2020-06-26
     if(vp->n <= 0) goto loop2;
     if(strncmp(vp->param[0],"$",1)==0) goto exitend;
     if(strncmp(vp->param[0],subkey,strlen(subkey))==0){
         n++;
     }
     goto loop2;

 exitend:
     free(line);  // 2020-06-26
     free(vp);    // 2020-06-26
     return n;
}



void  get_nelement_SKED(sked_data_t *skd)
{
     int n;
     FILE *fin;
     if( (fin  = fopen(skd->skdname, "r" )) == NULL ){
        printf("get_nelement_SKED File (%s) not found or Open Error! \n",skd->skdname);
        exit(1);
     }

     skd->nobs_max=get_SKED_nKEY(fin,"$SKED");
     skd->nstation_max=get_SKED_nKEY(fin,"$STATION");
     skd->nstar_max=get_SKED_nKEY(fin,"$SOURCE");
     skd->code.ncode_max=get_SKED_nKEY(fin,"$CODE");

     skd->ch_max=MAXCH;

     n=fclose(fin);
}


/* get # of elements $KEYWORD in SKED file */
int get_SKED_nKEY(FILE* fin, char *keyword)    // 2016-09-14
{
/*
    keyword should be either "$SKED" "SOURCES" "STATIONS"

*/
    int k,n;
    //int k,loop;
    //char line[500];
    char *line;    // 2020-06-26
    char subkey[10];
    //vparam_t vp;
    vparam_t *vp;

    line = malloc(_LINE_SIZE);  // 2020-06-26
    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26

    // rewind file

    rewind(fin);

    if(strncmp(keyword,"$STATION",8)==0){
        strcpy(subkey,"A");
    }else if(strncmp(keyword,"$CODE",5)==0){
        strcpy(subkey,"F");
    } else {
        subkey[0]='\0';
    }

    n=0;

    // find $xxxxx field block

loop:
     //if( fgets( line, 500, fin ) == NULL) goto exitend;
     if( fgets( line, _LINE_SIZE, fin ) == NULL) goto exitend;  // 2020-06-26
     //k=vex_parse(line, &vp);
     k=vex_parse(line, vp);  // 2020-06-26
     if(vp->n <= 0) goto loop;

     if(strncmp(vp->param[0],keyword,strlen(keyword))!=0) goto loop;

     /* $xxxxxx found */

loop2:
    
     //if( fgets( line, 500, fin ) == NULL) goto exitend;
     if (fgets(line, _LINE_SIZE, fin) == NULL) goto exitend;  // 2020-06-26
     //k=vex_parse(line, &vp);
     k = vex_parse(line, vp);  // 2020-06-26
     if(vp->n <= 0) goto loop2;
     if(strncmp(vp->param[0],"$",1)==0) goto exitend;
     if(subkey[0] != '\0'){
       if(strncmp(vp->param[0],subkey,strlen(subkey))==0){
           n++;
       }
     } else {
         n++;
     }

     goto loop2;

exitend:
     free(line);  // 2020-06-26
     free(vp);    // 2020-06-26
     return n;
}



void SKED_malloc(sked_data_t *skd)
{
/* 
    malloc elements of sked data type variable 
    2017-09-01
    2018-02-02
*/
    int n,k;

    skd->code.nstation_max=skd->nstation_max;
    skd->code.ch_max=skd->ch_max;


    /* element size MAXSTAR */
    //skd->starnam1=malloc(skd->nstar_max*11);    
    //skd->starnam2=malloc(skd->nstar_max*11);
    skd->starnam1=malloc(skd->nstar_max*MAX_SRC_NAME);    // 2017-08-28   
    skd->starnam2=malloc(skd->nstar_max*MAX_SRC_NAME);    // 2017-08-28

    skd->ra   =(double *)malloc(skd->nstar_max*sizeof(double));
    skd->dec  =(double *)malloc(skd->nstar_max*sizeof(double));
    skd->epo  =(double *)malloc(skd->nstar_max*sizeof(double));

    /* element size MAXSTAT */
    //skd->statnam = malloc(skd->nstation_max*11);
    skd->statnam = malloc(skd->nstation_max*MAX_STAT_NAME);  // 2017-08-28
    skd->statid  = malloc(skd->nstation_max*3);
    skd->statid2 = malloc(skd->nstation_max*3);
    skd->xyz = malloc(skd->nstation_max*3*sizeof(double));
    //skd->st_freq = malloc(skd->nstation_max * sizeof(st_freq_t));              // 2019-06-12
    skd->st_freq = (st_freq_t *)malloc(skd->nstation_max * sizeof(st_freq_t));              // 2019-06-12
    for (k = 0; k < skd->nstation_max; k++) {                                  // 2019-06-12
        skd->st_freq[k].ch = (int *)malloc(skd->ch_max * sizeof(int));         // 2019-06-12
        skd->st_freq[k].ul = (int *) malloc(skd->ch_max * sizeof(int));        // 2019-06-12
        skd->st_freq[k].freq = (double *)malloc(skd->ch_max * sizeof(double)); // 2019-06-12
        skd->st_freq[k].pcalf = malloc(skd->ch_max * 10 * sizeof(double));     // 2019-06-12
        skd->st_freq[k].vbw = (double *)malloc(skd->ch_max * sizeof(double));  // 2019-06-12
        skd->st_freq[k].thread =(int *) malloc(skd->ch_max * sizeof(int));     // 2019-06-12
        skd->st_freq[k].pol = malloc(skd->ch_max * 4);                         // 2019-06-12
    }                                                                          // 2019-06-12

    /* element size MAXOBS */
    //skd->obsstar=malloc(skd->nobs_max*11);
    skd->obsstar=malloc(skd->nobs_max*MAX_SRC_NAME);  // 2017-08-28
    skd->obsid  =malloc(skd->nobs_max*21);
    skd->scanid  =malloc(skd->nobs_max*21);
    skd->starid=(int *)malloc(skd->nobs_max*sizeof(int));
    skd->dura =(int *)malloc(skd->nobs_max*sizeof(int));
    skd->start=malloc(skd->nobs_max*5*sizeof(int));

    /* element size MAXOBS*MAXSTAT */
    skd->datastart=malloc(skd->nobs_max*sizeof(int *));
    //for(n=0; n<skd->nobs_max; n++) skd->datastart[n]=malloc(skd->nstation_max*sizeof(int));
    for(n=0; n<skd->nobs_max; n++) skd->datastart[n]=(int *)malloc(skd->nstation_max*sizeof(int)); // 2019-06-16
    skd->datastop=malloc(skd->nobs_max*sizeof(int *));
    //for(n=0; n<skd->nobs_max; n++) skd->datastop[n]=malloc(skd->nstation_max*sizeof(int));
    for(n=0; n<skd->nobs_max; n++) skd->datastop[n]=(int *)malloc(skd->nstation_max*sizeof(int)); // 2019-06-16
    skd->station_obs=malloc(skd->nobs_max*sizeof(int *));
    //for(n=0; n<skd->nobs_max; n++) skd->station_obs[n]=malloc(skd->nstation_max*sizeof(int));
    for(n=0; n<skd->nobs_max; n++) skd->station_obs[n]=(int *)malloc(skd->nstation_max*sizeof(int)); // 2019-06-16

    /* element size MAXCH */
    skd->freqtab =(double *)malloc(skd->ch_max*sizeof(double));
    //for(n=0; n<10; n++) skd->pcalf[n]=(double *)malloc(skd->ch_max*sizeof(double));
    //for(n=0; n<10; n++) skd->pcalf[n]=malloc(skd->ch_max*sizeof(double));
    //for(n=0; n<10; n++) skd->pcalf[n]=(double *)malloc(skd->ch_max*sizeof(double)); // 2019-06-16
    skd->pcalf = malloc(skd->ch_max * 10 * sizeof(double)); // 2019-12-21

    skd->sideband=malloc(skd->ch_max*4);
    skd->pol=malloc(skd->ch_max*4);               // 2019-06-11
    skd->thread=(int *)malloc(skd->ch_max*sizeof(int));               // 2019-06-11


    /* element size MAXCH*MAXSTAT */  // 2017-09-01
    skd->ch_by_stat=(int *)malloc(skd->ch_max*skd->nstation_max*sizeof(int));  // 2017-09-01
    skd->ul_by_stat=(int *)malloc(skd->ch_max*skd->nstation_max*sizeof(int));  // 2018-02-02
    skd->freq_by_stat=(double *)malloc(skd->ch_max*skd->nstation_max*sizeof(double));  // 2018-02-02
    skd->pol_by_stat=malloc(skd->ch_max*skd->nstation_max*4);  // 2019-06-11)
    skd->thread_by_stat=(int *)malloc(skd->ch_max*skd->nstation_max*sizeof(int));  // 2019-06-11
    skd->vbw_by_stat=(double *)malloc(skd->ch_max*skd->nstation_max*sizeof(double));  // 2019-06-12

    /* malloc skd->code.ctab   */
    //for(n=0; n<MAXCODES; n++){
    skd->code.ctab=(code_ele_t *)malloc(skd->code.ncode_max*sizeof(code_ele_t));
    for(n=0; n<skd->code.ncode_max; n++){
        //skd->code.ctab[n].statnam =malloc(skd->nstation_max*11);
        skd->code.ctab[n].statnam =malloc(skd->nstation_max*MAX_STAT_NAME);  // 2017-08-28
        skd->code.ctab[n].sideband=malloc(skd->ch_max*4);
        skd->code.ctab[n].freqtab=(double *)malloc(skd->ch_max*sizeof(double));
        skd->code.ctab[n].vbw    =(double *)malloc(skd->ch_max*sizeof(double));
        skd->code.ctab[n].adbit  =(int *)malloc(skd->ch_max*sizeof(int));
        //for(k=0; k<10; k++) skd->code.ctab[n].pcalf[k]=(double *)malloc(skd->ch_max*sizeof(double));
        skd->code.ctab[n].pcalf = malloc(skd->ch_max * 10 * sizeof(double)); // 2019-12-21
    }
}




void SKED_free(sked_data_t *skd)
{
/* 
    free elements of sked data type variable 
    2017-09-01
*/
    int n,k;

    /* element size MAXSTAR */
    free(skd->starnam1);
    free(skd->starnam2);
    free(skd->ra);
    free(skd->dec);
    free(skd->epo);

    /* element size MAXSTAT */
    free(skd->statnam);
    free(skd->statid);
    free(skd->statid2);
    free(skd->xyz);
    for (k = 0; k < skd->nstation_max; k++) {  // 2019-06-12
        free(skd->st_freq[k].ch);              // 2019-06-12
        free(skd->st_freq[k].ul);              // 2019-06-12
        free(skd->st_freq[k].freq);            // 2019-06-12
        free(skd->st_freq[k].pcalf);           // 2019-06-12
        free(skd->st_freq[k].vbw);             // 2019-06-12
        free(skd->st_freq[k].thread);          // 2019-06-12
        free(skd->st_freq[k].pol);             // 2019-06-12
    }                                          // 2019-06-12
    free(skd->st_freq);                        // 2019-06-12


    /* element size MAXCH */
    free(skd->freqtab);
    //for(n=0; n<10; n++) free(skd->pcalf[n]);
    free(skd->pcalf);  // 2019-12-21
    free(skd->sideband);
    free(skd->pol);      // 2019-06-11
    free(skd->thread);      // 2019-06-11

    /* element size MAXCH*MAXSTAT */  // 2017-09-01
    free(skd->ch_by_stat);
    free(skd->ul_by_stat);    // 2018-02-02
    free(skd->freq_by_stat);   // 2018-02-02
    free(skd->pol_by_stat);   // 2019-06-11
    free(skd->thread_by_stat);   // 2019-06-11
    free(skd->vbw_by_stat);   // 2019-06-12

    /* element size MAXOBS */
    free(skd->obsstar);
    free(skd->obsid);
    free(skd->scanid);
    free(skd->starid);
    free(skd->dura);
    free(skd->start);

    /* element size MAXOBS*MAXSTAT */
    for(n=0; n<skd->nobs_max; n++) free(skd->datastart[n]);
    free(skd->datastart);
    for(n=0; n<skd->nobs_max; n++) free(skd->datastop[n]);
    free(skd->datastop);
    for(n=0; n<skd->nobs_max; n++) free(skd->station_obs[n]);
    free(skd->station_obs);

    for(n=0; n<skd->code.ncode_max; n++){
        free(skd->code.ctab[n].statnam);
        free(skd->code.ctab[n].sideband);
        free(skd->code.ctab[n].freqtab);
        free(skd->code.ctab[n].vbw);
        free(skd->code.ctab[n].adbit);
        //for(k=0; k<10; k++) free(skd->code.ctab[n].pcalf[k]);
        free(skd->code.ctab[n].pcalf);
    }
    free(skd->code.ctab);


}





void VEX_malloc(vex_t *vex)
{
/* 
    malloc elements of vex data type variable 
    2016-09-16
*/
    int n;

    /* set $BLOCK related element */
    //vex->nblock_max=50;
    vex->block=malloc(vex->nblock_max*20);

    /* set $EXPER related element */
    //vex->nexper_max=10;
    vex->exper=(exper_t *)malloc(vex->nexper_max*sizeof(exper_t));

    /* set $MODE related element */
    //vex->nmode_max=10;
    vex->mode=(vexmode_t *)malloc(vex->nmode_max*sizeof(vexmode_t));
    for(n=0; n<vex->nmode_max; n++){
        vex->mode[n].mode_freq       =(mode_item_t *)malloc(vex->numstat_max*sizeof(mode_item_t));
        vex->mode[n].mode_track      =(mode_item_t *)malloc(vex->numstat_max*sizeof(mode_item_t));
        vex->mode[n].mode_roll       =(mode_item_t *)malloc(vex->numstat_max*sizeof(mode_item_t));
        vex->mode[n].mode_pcal_detect=(mode_item_t *)malloc(vex->numstat_max*sizeof(mode_item_t));
        vex->mode[n].mode_if         =(mode_item_t *)malloc(vex->numstat_max*sizeof(mode_item_t));
        vex->mode[n].mode_bbc        =(mode_item_t *)malloc(vex->numstat_max*sizeof(mode_item_t));
        vex->mode[n].def[0]='\0';  // 2016-09-30
    }

    /* set $SITE related element */
    vex->site=(site_t *)malloc(vex->numstat_max*sizeof(site_t));

    /* set $SOURCE related element */
    vex->source=(source_t *)malloc(vex->numstar_max*sizeof(source_t));

    /* set $FREQ related element */
    //vex->nfreq_def_max=20;
    vex->freq_def=(freq_t *)malloc(vex->nfreq_def_max*sizeof(freq_t));

    /* set $TRACK related element */
    //vex->ntrack_def_max=10;
    vex->track_def=(tracks_t *)malloc(vex->ntrack_def_max*sizeof(tracks_t));

    /* set $SCHED related element */
    vex->scan=(scan_t *)malloc(vex->numobs_max*sizeof(scan_t));
    for(n=0; n<vex->numobs_max; n++){
        vex->scan[n].nstation_max=vex->nstation_max;
        vex->scan[n].station=(station_in_scan_t *)malloc(vex->scan[n].nstation_max*sizeof(station_in_scan_t));
    }

    //printf("Debug VEX_malloc 1 \n");                                       // debug
    //printf("  block,exper,mode,site,source,freq_def,track_def,scan\n");    // debug
    //printf(" %X %X %X %X  %X %X %X %X\n",vex->block,vex->exper,vex->mode,vex->site,vex->source,vex->freq_def,vex->track_def,vex->scan);


    /* set $STATION related element */
    vex->station=(station_def_t *)malloc(vex->nstation_max*sizeof(station_def_t));
    //printf("Debug VEX_malloc 2  nstation_max station %d %X\n",vex->nstation_max,vex->station);
    for(n=0; n<vex->nstation_max; n++){
    //printf("Debug VEX_malloc 3a %d %d\n",n,vex->ndas_max);
        vex->station[n].ndas_max=vex->ndas_max;
    //printf("Debug VEX_malloc 3b %d\n",n);
        vex->station[n].das=malloc(vex->station[n].ndas_max*20);
    }

    /* set $CLOCK related element */
    //vex->nclock_max=vex->nstation_max;
    vex->siteclock=(siteclock_t *)malloc(vex->nclock_max*sizeof(siteclock_t));

    /* set $EOP related element */
    //vex->neop_max=10;
    vex->eop=(eop_t *)malloc(vex->neop_max*sizeof(eop_t));

    /* set $ROLL related element */
    //vex->nroll_max=10;
    vex->roll=(roll_t *)malloc(vex->nroll_max*sizeof(roll_t));

    /* set $PCAL related element */
    //vex->npcal_max=MAXPCAL_B;
    vex->pcal=(pcal_block_t *)malloc(vex->npcal_max*sizeof(pcal_block_t));

    /* set $BBC related element */
    //vex->nbbc_max=MAXBBC_B;
    vex->bbc=(bbc_block_t *)malloc(vex->nbbc_max*sizeof(bbc_block_t));

    /* set $IF related element */
    //vex->nifb_max=MAXIF_B;
    vex->ifb=(if_block_t *)malloc(vex->nifb_max*sizeof(if_block_t));

    /* set $DAS related element */
    //vex->ndas_max=MAXDAS;
    vex->das=(das_def_t *)malloc(vex->ndas_max*sizeof(das_def_t));

}




void VEX_free(vex_t *vex)
{
/* 
    malloc elements of vex data type variable 
    2016-09-16
*/
    int n;

    free(vex->block);
    free(vex->exper);
    for(n=0; n<vex->nmode_max; n++){
        free(vex->mode[n].mode_freq);
        free(vex->mode[n].mode_track);
        free(vex->mode[n].mode_roll);
        free(vex->mode[n].mode_pcal_detect);
        free(vex->mode[n].mode_if);
        free(vex->mode[n].mode_bbc);
    }
    free(vex->mode);
    free(vex->site);
    free(vex->source);
    free(vex->freq_def);
    free(vex->track_def);
    for(n=0; n<vex->numobs_max; n++){
        free(vex->scan[n].station);
    }
    free(vex->scan);
    for(n=0; n<vex->nstation_max; n++){
        free(vex->station[n].das);
    }
    free(vex->station);
    free(vex->siteclock);
    free(vex->eop);
    free(vex->roll);
    free(vex->pcal);
    free(vex->bbc);
    free(vex->ifb);
    free(vex->das);

}

// add on 2018-02-01

int setup_freq_table_for_sid_vex(vex_t *vex, sked_data_t *skd, char *stat_id)
{
/*
    set up frequency table 
    condition :  use mode information in the scan information where given station attends firstly.
                 If given station has no scan to attend,  -1 will be returned, but 
                 frequency is set up by using the 1st mode.    
*/
    // 2018-02-01
    // 2019-06-11
         
    int i,n;
    //int index_id;
    //int v_mode;
    int scan_num;
    //int nch;
    char mode[30];
    //char freq_code[50];
    scan_t  *scan;
    freq_t *freq_def; 
    freq_info_t freq_info;
    int ierr;


    ierr=0;
    scan=vex->scan;
    freq_def=vex->freq_def;
    // search mode should be used

    scan_num=-1;
    mode[0]='\0';
    for(n=0; n<vex->numobs; n++){
        for(i=0; i<scan[n].numstat; i++){
            if(strncmp(scan[n].station[i].id, stat_id, strlen(stat_id))==0){
                strcpy(mode,scan[n].mode);
                scan_num=n;
                goto exit1;
            }
        }
    }
    // not attend the obs, so use 1st mode
    strcpy(mode,vex->mode[0].def);
    ierr=-1;
exit1:

    // mode is fixed


    for(n=0; n<skd->ch_max; n++) { 
        skd->freqtab[n]=0.0;
        //skd->sideband[n][0]='\0';
        strcpy(skd->sideband[n],"U");  /* default */
        //for(i=0; i<10; i++) skd->pcalf[i][n]=0.0;
        for(i=0; i<10; i++) skd->pcalf[n][i]=0.0;  // 2019-12-21
        strcpy(skd->pol[n], " ");     // 2019-06-11
    }

    freq_info=vex_chinfo_get(vex,mode,stat_id,1);
    skd->nfreq=freq_info.chmax;
    for(n=0; n< skd->nfreq; n++){
        freq_info=vex_chinfo_get(vex,mode,stat_id,n+1);
        skd->freqtab[n]=freq_info.rf/1.0e6;  // MHz unit in SKED
        //for(i=0; i<10; i++) skd->pcalf[i][n]=0.0;
        for(i=0; i<10; i++) skd->pcalf[n][i]=0.0;  // 2019-12-21
        //printf("freq_info.npcal  %d\n",freq_info.npcal);
        for(i=0; i<freq_info.npcal; i++){
           //skd->pcalf[i][n]=freq_info.pcalf[i]; 
           skd->pcalf[n][i]=freq_info.pcalf[i];    // 2019-12-21
           //printf("skd->pcalf  %d\n",skd->pcalf[i][n]);
        }
        strcpy(skd->sideband[n],freq_info.sideband);
        strcpy(skd->pol[n],freq_info.pol);    // 2019-06-11
        skd->thread[n] = freq_info.thread;    // 2019-06-11
    }

    setup_ch_alloc_table_by_station(vex, mode, skd);  // 2017-09-03


    return ierr;
}









        

    // set up frequency table again

    // kokokara kaihatutyuu
/*
    for(n=0; n<16; n++) {   
        skd->freqtab[n]=0.0;
        //skd->sideband[n][0]='\0';
        strcpy(skd->sideband[n],"U");  
        for(i=0; i<10; i++) skd->pcalf[i][n]=0.0;
    }

    // check mode 

    freq_info=vex_chinfo_get(vex,mode,ssta_id,1);
    skd->nfreq=freq_info.chmax;
    //printf("Chmax %d\n",skd->nfreq);
    for(n=0; n< skd->nfreq; n++){
        freq_info=vex_chinfo_get(vex,mode,sid,n+1);
        skd->freqtab[n]=freq_info.rf/1.0e6;  // MHz unit in SKED
        for(i=0; i<10; i++) skd->pcalf[i][n]=0.0;
        //printf("freq_info.npcal  %d\n",freq_info.npcal);
        for(i=0; i<freq_info.npcal; i++){
           skd->pcalf[i][n]=freq_info.pcalf[i]; 
           //printf("skd->pcalf  %d\n",skd->pcalf[i][n]);
        }
        strcpy(skd->sideband[n],freq_info.sideband);
    }


    setup_ch_alloc_table_by_station(vex, mode, skd);  // 2017-09-03
*/


/** added on 2018-10-12  **/

void setup_ch_alloc_table_by_station_for_sked(sked_data_t *skd)
// 2018-10-12
// 2019-06-11
{
    /*
      set up channle allocation table in SKED type data for SKED file
      (Note that "setup_ch_alloc_table_by_station" function is for VEX!!)
     [in]
       skd->nfreq
       skd->freqtab[]
       skd->sideband[]
     [return]
       skd->ch_by_stat[] --  channel allocation table
       skd->ul_by_stat[] --  USB/LSB (1/0) table
       skd->freq_by_stat[] --  Frequency table
*/

    int n,k;
    int uorl;   // 1: usb  0: lsb

    // initialize ch_by_stat witn default
    for(k=0; k<skd->nstation; k++){
        for(n=0; n<skd->ch_max; n++){
           skd->ch_by_stat[k*skd->ch_max+n]=n+1;
           skd->freq_by_stat[k*skd->ch_max+n]=0.0;
           skd->ul_by_stat[k*skd->ch_max+n]=1;
           skd->pol_by_stat[k*skd->ch_max + n][0] = '\0';  // 2019-06-11
           skd->thread_by_stat[k*skd->ch_max + n] = 0;  // 2019-06-11
        }
    }


    for(n=0; n<skd->nfreq; n++){
        uorl=1;    // default is USB
        if(skd->sideband[n][0]=='L') uorl=0;   // LSB

        for(k=0; k<skd->nstation; k++){
            skd->freq_by_stat[k*skd->ch_max+n]=skd->freqtab[n];
            skd->ul_by_stat[k*skd->ch_max+n]=uorl;
        }
    }
}


/* added on 2019-05-22 */
/* get maximum number of ids used for $XXXXXXX under $MODE block in VEX file */
int get_nIDs_in_VEX_MODE(FILE* fin)    // 2019-05-22
{
    /*
        keyword should be "$XXXXXX"

      Key # of IDs under $MODE block:
          

    */
    int k, n;
    //int k,loop;
    //char line[500];
    char *line;  // 2020-06-26
    char subkey[10];
    char itemkey[20];
    //vparam_t vp;
    vparam_t *vp;  // 2020-06-26
    char item[10][20] = { "$FREQ", "$BBC", "$IF", "$TRACKS", "$HEAD_POS",
                        "$PASS_ORDER", "$ROLL", "$PHASE_CAL_DETECT", "", "" };
    int num[10];
    int nid,nmax;
    int nitem = 8;

    line = malloc(_LINE_SIZE);  // 2020-06-26
    vp = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26

    for (n = 0; n < nitem; n++) num[n] = 0;

    // rewind file
    rewind(fin);

    //if (strncmp(keyword, "$SCHED", 6) == 0) {
    //    strcpy(subkey, "scan");
    //}
    //else {
    strcpy(subkey, "def");
    //}

    nmax = 0;

    // find $MODE field block

loop:
    //if (fgets(line, 500, fin) == NULL) goto exitend;
    if (fgets(line, _LINE_SIZE, fin) == NULL) goto exitend;  // 2020-06-26
    //k = vex_parse(line, &vp);
    k = vex_parse(line, vp);  // 2020-06-26
    if (vp->n <= 0) goto loop;

    if (strncmp(vp->param[0], "$MODE", strlen(vp->param[0])) != 0) goto loop; 

    /* analysis every items */

loop2:

    //if (fgets(line, 500, fin) == NULL) goto exitend;
    if (fgets(line, _LINE_SIZE, fin) == NULL) goto exitend;  // 2020-06-26
    //k = vex_parse(line, &vp);
    k = vex_parse(line, vp);  // 2020-06-26
    if (vp->n <= 0) goto loop2;
    if (strncmp(vp->param[0], "$", 1) == 0) goto exitend;
    if (strncmp(vp->param[0], subkey, strlen(subkey)) == 0) {
loop3:
        //if (fgets(line, 500, fin) == NULL) goto exitend;
        if (fgets(line, _LINE_SIZE, fin) == NULL) goto exitend;  // 2020-06-26
        //k = vex_parse(line, &vp);
        k = vex_parse(line, vp);  // 2020-06-26
        if (vp->n <= 0) goto loop3;
        if (strncmp(vp->param[0], "$", 1) == 0) goto exitend;
        if (strncmp(vp->param[0], subkey, strlen(subkey)) == 0) {  // next def
            // find max and initialization of num[] for new statistics
            for (n = 0; n < nitem; n++) {
                if (num[n] > nmax) nmax = num[n];
                num[n] = 0;
            }
            goto loop3;
        }

        if (vp->n > 5) {
            if (strncmp(vp->param[0], "ref", 3) == 0) {
                for (n = 0; n < nitem; n++) {
                    strcpy(itemkey, item[n]);
                    nid = (vp->n - 5+1) / 2;
                    if (strncmp(vp->param[1], itemkey, strlen(itemkey)) == 0) {
                        if (nid > 0) num[n] += nid;
                        //printf("   %s  %d\n", itemkey, num[n]);  // debug
                    }
                }
            }
        }
        goto loop3;
    }
    goto loop2;

exitend:
    free(line);  // 2020-06-26
    free(vp);    // 2020-06-26
    for (n = 0; n < nitem; n++) {
        if (num[n] > nmax) nmax = num[n];
    }
    return nmax;
}

// add on 2019-06-14

void setup_st_freq(sked_data_t *skd, vex_t *vex, char *mode_id)
{
/*
    set up skd->st_freq data
      2019-06-14
*/
    int i,n,k;
    int imulti, iold;
    freq_info_t freq_info; 

    for (n = 0; n < skd->nstation; n++) {
        freq_info = vex_chinfo_get(vex, mode_id, skd->statid[n], 1);
        skd->nfreq = freq_info.chmax;
        strcpy(skd->st_freq[n].sid, skd->statid[n]);
        skd->st_freq[n].nch = skd->nfreq;
        imulti = 0;
        for (k = 0; k < skd->nfreq; k++) {
            freq_info = vex_chinfo_get(vex, vex->mode[0].def, skd->statid[n], k + 1);
            if(k==0) iold= freq_info.thread;
            //skd->st_freq[n].freq[k] =  skd->freq_by_stat[n*skd->ch_max + k];
            skd->st_freq[n].freq[k] = freq_info.rf / 1.0e6;  // MHz unit in SKED

            for (i = 0; i < 10; i++) skd->st_freq[n].pcalf[k][i] = 0.0;
            for (i = 0; i < 10; i++) {
                //skd->st_freq[n].pcalf[k][i] = skd->pcalf[i][k];
                skd->st_freq[n].pcalf[k][i] = freq_info.pcalf[i];
            }
            skd->st_freq[n].ul[k] = 1;
            if (freq_info.sideband[0] == 'L' || freq_info.sideband[0] == 'l') skd->st_freq[n].ul[k] = 0;
            strcpy(skd->st_freq[n].pol[k], freq_info.pol);
            skd->st_freq[n].vbw[k] = freq_info.vbw;
            skd->st_freq[n].thread[k] = freq_info.thread;
            skd->st_freq[n].ch[k] = freq_info.ch; 
            if (freq_info.thread != iold) imulti = 1;
            iold=freq_info.thread;
        }
        skd->st_freq[n].flag_multi_thread = imulti;

    }
}



// added on 2019-12-19

void mon_readskd_codes(codes_t *code)
{
/*
     monitor code (frequency field) data for debug
       2019-12-19
*/
    int n,k,i;


    for (n = 0; n < code->ncode; n++) {
        printf(" Code %d\n", n + 1);
        // "F" field
        printf(" Station: ");
        for (k = 0; k < code->ctab[n].nstation; k++) printf(" %s", code->ctab[n].statnam[k]);
        printf("\n");
        // "C" field
        for (k = 0; k < code->ctab[n].nfreq; k++) {
            printf(" CH# %2d  Freq(MHz)=%9.2f U/L= %s  AD= %d  PCAL(kHz)= ",
                k + 1, code->ctab[n].freqtab[k],
                code->ctab[n].sideband[k], code->ctab[n].adbit[k]);
            i = 0;
            while (1) {
                //if (code->ctab[n].pcalf[i][k] < 0.1) break;
                if (code->ctab[n].pcalf[k][i] < 0.1) break;
                //printf(" %8.1f", code->ctab[n].pcalf[i][k] / 1.0e3);
                printf(" %8.1f", code->ctab[n].pcalf[k][i] / 1.0e3);
                i++;
                if (i >= 10) break;
            }
            printf("\n");
        }
    }

}

// added on 2019-12-20

void set_up_codes(codes_t *code, int np, char **cod, int nline)
{
    /*
        analsysis each code field line
          2019-12-19
          2019-12-25

          not completed

    */
    int n, k;
    int trmax, trmin, trstep;
    int ch;
    int tus, tls, tum, tlm;
    //double freq, pcal, vbw;
    int nvc;  // # of Video Converters
    //int vch;  // VC#
    int n_vch; // # of video channels
    int flag_mix;  // flag for odd even mixture
    int flag_even, flag_odd;
    int flag_1bit;  // 1bit AD flag  2019-12-25
    //char line[100];
    char *line;  // 2020-06-26
    //vparam_t vparam;
    vparam_t *vparam;  // 2020-06-26
    code_C_line_t *cele;

    cele = (code_C_line_t *)malloc((nline - 1) * sizeof(code_C_line_t));  // # of C lines is nline-1
    line = malloc(_LINE_SIZE);  // 2020-06-26
    vparam = (vparam_t *)malloc(sizeof(vparam_t));  // 2020-06-26


    // at 1st  get total number of tracks
    /*
      Example of F and C lines   inside of ( ) is track#

        F AOVSX16N SX HOBART26 WARK12M YARRA12M
        C SX X 8208.99 10000.000  1 Mk341:1 16.00 1(-1, 15, 0, 16)
        C SX X 8228.99 10000.000  2 Mk341:1 16.00 1(1, , 2)
        .....
    */
    flag_1bit = 1;  // dfault is 1bit AD
    nvc = 0;
    for (n = 0; n < nline; n++) {
        strcpy(line, cod[n]);
        if (line[0] == 'F') {   // Station Names 
            //line_parse(line, &vparam);
            line_parse(line, vparam);  // 2020-06-26
            code->ctab[np].nstation = vparam->n - 3;
            for (k = 0; k < code->ctab[np].nstation; k++) {
                strcpy(code->ctab[np].statnam[k], vparam->param[k + 3]);
            }
        }
        if (line[0] == 'C') {   // Frequency info
            //line_parse(line, &vparam);
            line_parse(line, vparam);  // 2020-06-26
            strcpy(cele[nvc].code, vparam->param[1]);
            strcpy(cele[nvc].subgrp, vparam->param[2]);
            cele[nvc].freq = atof(vparam->param[3]);
            cele[nvc].pcal = atof(vparam->param[4]);
            cele[nvc].ch = atoi(vparam->param[5]);   // VC#
            strcpy(cele[nvc].mode, vparam->param[6]);  // Formatter mode
            cele[nvc].bw = atof(vparam->param[7]);  // video band width
            parseSBinfo(vparam->param[8], &tus, &tls, &tum, &tlm); // libskdvex function
            cele[nvc].tus = tus;
            cele[nvc].tls = tls;
            cele[nvc].tum = tum;
            cele[nvc].tlm = tlm;
            // analyze AD bits
            cele[nvc].adu = 0;   // means no data 
            cele[nvc].adl = 0;   // means no data 
            if (tus != -99) {
                cele[nvc].adu = 1;   // 1-bit AD
                if (tum != -99) {
                    cele[nvc].adu = 2;  // 2-bit AD
                    flag_1bit = 0;  // 2019-12-25
                }
            }
            if (tls != -99) {
                cele[nvc].adl = 1;   // 1-bit AD
                if (tlm != -99) {
                    cele[nvc].adl = 2;  // 2-bit AD
                    flag_1bit = 0;  // 2019-12-25
                }
            }
            cele[nvc].chu = 0;   // set default  (fixed later)
            cele[nvc].chl = 0;   // set default  (fixed later)
            nvc++;
            if (nvc > nline - 1) nvc--;  // to avoid error
        }
    }

    flag_mix = 0;
    flag_even = 0;
    flag_odd = 0;
    n_vch = 0;
    trmax = -10; trmin = 100;
    for (n = 0; n < nvc; n++) {
        if (cele[n].tus != -99) {
            n_vch++;
            if (cele[n].tus > trmax) trmax = cele[n].tus;
            if (cele[n].tus < trmin) trmin = cele[n].tus;
            if ((abs(cele[n].tus) % 2) == 0) flag_even++;     // even
            if ((abs(cele[n].tus) % 2) == 1) flag_odd++;     // odd
        }
        if (cele[n].tum != -99) {
            if (cele[n].tum > trmax) trmax = cele[n].tum;
            if (cele[n].tum < trmin) trmin = cele[n].tum;
        }
        if (cele[n].tls != -99) {
            n_vch++;
            if (cele[n].tls > trmax) trmax = cele[n].tls;
            if (cele[n].tls < trmin) trmin = cele[n].tls;
            if ((abs(cele[n].tls) % 2) == 0) flag_even++;     // even
            if ((abs(cele[n].tls) % 2) == 1) flag_odd++;     // odd
        }
        if (cele[n].tlm != -99) {
            if (cele[n].tlm > trmax) trmax = cele[n].tlm;
            if (cele[n].tlm < trmin) trmin = cele[n].tlm;
        }
    }
    trstep = (trmax - trmin) / (n_vch - 1);  // usually 2
    // now n_vch is total # of video channels (nvc + alpha)
    if (flag_odd > 0 && flag_even > 0) flag_mix = 1;
    if (flag_1bit == 1) flag_mix = 0;  // 2019-12-25

    // setup VC CH#
    for (n = 0; n < nvc; n++) {
        if (cele[n].tus != -99) {
            if (flag_mix == 0) {
                ch = (cele[n].tus - trmin) / trstep + 1;
            }
            else {
                if ((abs(cele[n].tus) % 2) == 1) {
                    ch = (cele[n].tus - trmin) / 4 + 1;
                }
                else {
                    ch = cele[n].tus / 4 + 1 + 8;
                }
            }
            cele[n].chu = ch;
        }
        if (cele[n].tls != -99) {
            if (flag_mix == 0) {
                ch = (cele[n].tls - trmin) / trstep + 1;
            }
            else {
                if ((abs(cele[n].tls) % 2) == 1) {
                    ch = (cele[n].tls - trmin) / 4 + 1;
                }
                else {
                    ch = cele[n].tls / 4 + 1 + 8;
                }
            }
            cele[n].chl = ch;
        }
    }



    // debug
    /*
    for (n = 0; n < nvc; n++) {
        printf(" %d %s %s %f %f %d %s %f (%d %d %d %d) (%d %d)\n",
            n, cele[n].code, cele[n].subgrp, cele[n].freq, cele[n].pcal, cele[n].ch,
            cele[n].mode, cele[n].bw, cele[n].tus, cele[n].tls, cele[n].tum, cele[n].tlm,
            cele[n].chu,cele[n].chl);
    }
    */

    // set up code data
    code->ctab[np].nfreq = n_vch;
    for (n = 0; n < nvc; n++) {
        ch = cele[n].chu;    // USB
        if (ch > 0) {
            code->ctab[np].freqtab[ch - 1] = cele[n].freq;
            code->ctab[np].vbw[ch - 1] = cele[n].bw;
            //code->ctab[np].pcalf[0][ch - 1] = cele[n].pcal;
            //code->ctab[np].pcalf[1][ch - 1] = 0.0;
            code->ctab[np].pcalf[ch - 1][0] = cele[n].pcal;
            code->ctab[np].pcalf[ch - 1][1] = 0.0;
            code->ctab[np].adbit[ch - 1] = cele[n].adu;
            strcpy(code->ctab[np].sideband[ch - 1], "U");
        }
        ch = cele[n].chl;   // LSB
        if (ch > 0) {
            code->ctab[np].freqtab[ch - 1] = cele[n].freq;
            code->ctab[np].vbw[ch - 1] = cele[n].bw;
            //code->ctab[np].pcalf[0][ch - 1] = 1.0e6 - cele[n].pcal; // valid only in case of 1 MHz PCAL
            //code->ctab[np].pcalf[1][ch - 1] = 0.0;
            code->ctab[np].pcalf[ch - 1][0] = 1.0e6 - cele[n].pcal; // valid only in case of 1 MHz PCAL
            code->ctab[np].pcalf[ch - 1][1] = 0.0;
            code->ctab[np].adbit[ch - 1] = cele[n].adl;
            strcpy(code->ctab[np].sideband[ch - 1], "L");
        }
    }


    free(cele);
    free(line);  // 2020-06-26
    free(vparam);  // 2020-06-26
}

int anal_skd_codes(FILE *fi, char *line, codes_t *code)
{
    /*
       analyze SKED file $CODES

       return +VE:  # of codes
              <0 :  error ocurred
    */
    //char line[128];
    char old1stch;
    //int numC[10];
    int flag_line;
    int n;
    int ncount;
    int cp;
    //int iok;
    //int iflag;
    int ierr;
    int len;
    int numF;
    int flag_code, flag_F;
    int flag_finish;
    char *lbuf;
    int lsize = 6400;
    //char *cod[64];
    char **cod;
    //int nsize = 64;
    //vparam_t vparam;
    vparam_t *vparam;  // 2020-06-26

    lbuf = (char *)malloc(lsize * sizeof(char));
    lbuf[0] = '\0';
    //cod = (char **)malloc(code->ch_max * sizeof(char *));
    cod = malloc(code->ch_max * sizeof(char *));
    cod[0] = lbuf;

    vparam = (vparam_t *)malloc(sizeof(vparam_t)); // 2020-06-26

    ncount = 0;
    cp = 0;
    ierr = 0;
    numF = 0;
    //flag_code = 0;  
    flag_code = 1;  // in case $CODES is already selected in caller side
    flag_F = 0;
    flag_line = 0;
    flag_finish = 0;
    //rewind(fi);

    line[0] = '\0';

loop:;
    old1stch = line[0];
    //if (fgets(line, 500, fi) == NULL) {
    if (fgets(line, _LINE_SIZE, fi) == NULL) {  // 2020-06-26
        ierr = -1;
        goto exithere;  // no more line
    }
    if (line[0] == '*') goto loop;  // this is comment line (nothing to do)  
    if (line[0] == '$')  goto exithere;   // next declaration line 
    //if (flag_code == 1 && line[0] == '$') goto exithere; // next declaration line 

    if (flag_code == 0) {
        if (strncmp(line, "$CODES", 6) == 0) {    // this should be done at a caller side
            flag_code = 1;
            goto loop;  // to read next line
        }
    }
    if (flag_code == 0) goto loop;

    // judge completion of F-C block 
    if (flag_F == 1 && line[0] != 'F' && old1stch == 'C') {  //  1 F-C block completed no more new F-C block
        if (line[0] != 'C') {  //  1 F-C block completed no more new F-C block
            set_up_codes(code, numF, cod, ncount);
            goto exithere;
        }
    }
    if (line[0] == 'F' && old1stch == 'C') {  //  1 F-C block completed and start new F-C block
            // 1 block completed
        set_up_codes(code, numF, cod, ncount);
        //for (n = 0; n < ncount; n++) printf("** %s\n", cod[n]);  // KOKO
        numF++;
        flag_F = 1;
        ncount = 0;
        lbuf[0] = '\0';
        cp = 0;
        flag_line = 1;  // flag to copy line to lbuf
    }
    if (line[0] == 'F' && old1stch != 'C') {  //  1 F-C block just started
        flag_F = 1;
        flag_line = 1;  // flag to copy line to lbuf
    }
    if (line[0] == 'C') {  //   C line 
        flag_line = 1;
    }

    // copy line to lbuf

    if (flag_line == 1) {
        len = strlen(line);
        // remove CR/LF code
        for (n = 0; n < len; n++) {
            if (line[n] == 0x0a || line[n] == 0x0d) {
                line[n] = '\0';
                break;
            }
        }
        len = strlen(line);

        for (n = 0; n < len; n++) {
            lbuf[cp] = line[n];
            cp++;
        }
        lbuf[cp] = '\0';
        cp++;
        ncount++;
        cod[ncount] = lbuf + cp;
        //printf(" %d", cp);
    }
    goto loop;

loop2:;

    //if (fgets(line, 500, fi) == NULL) {
    if (fgets(line, _LINE_SIZE, fi) == NULL) {  // 2020-06-26
        ierr = -1;
        goto exithere2;  // no more line
    }
    if (line[0] == '*') goto loop2;  // this is comment line (nothing to do)  
    //if (line[0] == '$')  goto exithere;   // next declaration line 
    if (flag_finish == 1) {
        if (line[0] == '$') goto exithere2; // next declaration line 
    }

exithere:;

    if (line[0] == 'R') {  //   R line 
        // get sampling frequency
        //line_parse(line, &vparam);
        line_parse(line, vparam);  // 2020-06-26
        code->fsample = atof(vparam->param[2])*1.0e6;
        flag_finish = 1;
        //goto exithere2;
    }
    goto loop2;

exithere2:;
    numF++;
    code->ncode = numF;

    // debug
    //mon_readskd_codes(code);

    free(cod);
    free(lbuf);
    free(vparam);  // 2020-06-26

    if (ierr < 0) return ierr;
    return 0;

}

void init_skd_st_freq(sked_data_t *skd)
{
/*
     initialize skd->st_freq data
       2019-12-21

*/

    int k, n;
    for (n = 0; n < skd->nstation_max; n++) {
        skd->st_freq[n].sid[0] = '\0';
        skd->st_freq[n].nch = 0;
        skd->st_freq[n].flag_multi_thread = 0;

        for (k = 0; k < skd->ch_max; k++) {
            skd->st_freq[n].ch[k] = -1;
            skd->st_freq[n].ul[k] = -1;
            skd->st_freq[n].freq[k] = 0.0;
            //skd->st_freq[n].pcalf[0][k] = 0.0;
            //skd->st_freq[n].pcalf[1][k] = 0.0;
            skd->st_freq[n].pcalf[k][0] = 0.0;
            skd->st_freq[n].pcalf[k][1] = 0.0;
            skd->st_freq[n].vbw[k] = 0.0;
            skd->st_freq[n].thread[k] = -1;
            skd->st_freq[n].pol[k][0] = '\0';
        }
    }
}                                                                          // 2019-06-12

void setup_sked_to_st_freq(sked_data_t *skd)
{
/*
     set-up   skd->st_freq data from skd->code data
      2019-12-21
      2020-09-30 revised a bit

*/
    int i,k,n;
    int iok;
    int nch;
    int kpos;
    char stat1[20], stat2[20];

    init_skd_st_freq(skd);  // initialize skd->st_freq data

    for (n = 0; n < skd->nstation; n++) {
        strcpy(skd->st_freq[n].sid, skd->statid[n]);

        strcpy(stat1, skd->statnam[n]);
        // search same station name in code data
        iok = 0;
        kpos = 0;
        for (k = 0; k < skd->code.ncode; k++) {
            for (i = 0; i < skd->code.ctab[k].nstation; i++) {
                strcpy(stat2, skd->code.ctab[k].statnam[i]);
                if (strncmp(stat1, stat2, strlen(stat2)) == 0) {
                    iok = 1;
                    kpos = k;
                    break;
                }
            }
        }
        if (skd->flag_code_new_anal == 1) {  // 2020-09-30
            if (iok == 0) {  // station name not match with code data
                printf(" /Warning!: station name %s has no frequency data. Anyway use 1st station data instead.\n",
                    stat1);
            }
        }
        skd->st_freq[n].nch = skd->code.ctab[kpos].nfreq;
        nch = skd->st_freq[n].nch;
        for (i = 0; i < nch; i++) {
            skd->st_freq[n].ch[i] = i + 1;
            skd->st_freq[n].ul[i] = 1;
            if(skd->code.ctab[kpos].sideband[i][0]=='L') skd->st_freq[n].ul[i] = 0;
            skd->st_freq[n].freq[i] =  skd->code.ctab[kpos].freqtab[i];
            //skd->st_freq[n].pcalf[0][i] =  skd->code.ctab[k].pcalf[0][i];
            //skd->st_freq[n].pcalf[1][i] =  skd->code.ctab[k].pcalf[1][i];
            skd->st_freq[n].pcalf[i][0] =  skd->code.ctab[kpos].pcalf[i][0];
            skd->st_freq[n].pcalf[i][1] =  skd->code.ctab[kpos].pcalf[i][1];

            //printf(" %d  %f %f\n",
            //    i, skd->st_freq[n].pcalf[1][0], skd->st_freq[n].pcalf[i][1]); // debug
            skd->st_freq[n].vbw[i] =  skd->code.ctab[kpos].vbw[i];
        }
    }

    //mon_st_freq(skd); // debug
}



void mon_st_freq(sked_data_t *skd)
{
    /*
         monitor code (frequency field) data for debug
           2019-12-19
    */
    int n, k, i;
    st_freq_t *sf;

    for (n = 0; n < skd->nstation; n++) {
        printf(" Station: ");
        printf(" %s %s\n", skd->statnam[n], skd->statid[n]);
        sf = &skd->st_freq[n];
        for (k = 0; k < sf->nch; k++) {
            printf(" CH# %2d  Freq(MHz)=%9.2f U/L= %d PCAL(kHz)= ",
                sf->ch[k], sf->freq[k], sf->ul[k]);
            i = 0;
            while (1) {
                if (sf->pcalf[k][i] < 0.1) break;
                printf(" %8.1f", sf->pcalf[k][i]);
                i++;
                if (i >= 10) break;
            }
            printf("\n");
        }
    }

}



void anal_track_then_conv_to_ch(char *track_code, vex_t *vex, int *ch_table)
{
/*
      Pick up track info then convert it channel info
        2020-10-10

    [in]
       track_code --- pick up $TRACK field def code
       vex  --  vex_t type variable



*/

    char cbuf[60]; 
    char ch_id[10];
    int i, n;
    int ch;
    int stab[64], mtab[64];
    int flag2bit;
    int wmin, chmax;

    for (n = 0; n < 64; n++) ch_table[n] = n+1;  // set default

    // pick up track 
    for (n = 0; n < 64; n++) {
        stab[n] = -1; mtab[n] = -1;
    }
    flag2bit = 0;
    wmin = 99;
    chmax = 0;
    for (n = 0; n < vex->ntrack_def; n++) {
        if (strncmp(track_code, vex->track_def[n].def, strlen(vex->track_def[n].def)) == 0 &&
            strlen(track_code) == strlen(vex->track_def[n].def)) {
            if (strncmp("Mark4", vex->track_def[n].format, strlen(vex->track_def[n].format)) == 0) {  
                           // Mark 4 is supposed to be converted to Mark5B outside
                           // so pick up channel should be 1,2,3,.....
                           // so keep default setting
                goto exithere;
            }

            // copy track table
            for (i = 0; i < vex->track_def[n].nfanout_def; i++) {
                strcpy(ch_id, vex->track_def[n].fanout_def[i].chan_id);  // copy CH id
                if (strlen(ch_id) == 5) {    // &CHXX
                    cbuf[0] = ch_id[3]; cbuf[1] = ch_id[4]; cbuf[2] = '\0';       // pick up channel #
                }
                else if (strlen(ch_id) == 6) { // &CHAXX
                    cbuf[0] = ch_id[4]; cbuf[1] = ch_id[5]; cbuf[2] = '\0';       // pick up channel #

                }
                ch = atoi(cbuf);
                if (ch > chmax) chmax = ch;
                if (strncmp(vex->track_def[n].fanout_def[i].stream_info, "mag", 3) == 0) {  // 'mag'
                    mtab[ch - 1] = vex->track_def[n].fanout_def[i].track[0];        // set track# of 'mag'
                    flag2bit = 1;
                }
                else {
                    stab[ch - 1] = vex->track_def[n].fanout_def[i].track[0];        // set track# of 'sign'
                }
                if (vex->track_def[n].fanout_def[i].track[0] < wmin) wmin = vex->track_def[n].fanout_def[i].track[0];
            }

            // debug
            /*
            printf("%s stab:",sid);
            for(i=0; i<chmax; i++) printf(" %d",stab[i]);
            printf("\n");
            printf("%s mtab:",sid);
            for(i=0; i<chmax; i++) printf(" %d",mtab[i]);
            printf("\n");
            */

            // convert track table to pick up channel position
            if (vex->track_def[n].fanout_def[0].fanout == 2) {  // in case of fanout=2)
               // nothing to do right now (use default)
                if (flag2bit == 1) {     // 2 bit AD

                }
                else {

                }
            }
            else {     // fanout=1
                if (flag2bit == 1) {    // 2bit AD
                    for (i = 0; i < chmax; i++) {
                        //skd->ch_by_stat[k*skd->ch_max + i] = (stab[i] - wmin) / 2 + 1;
                        ch_table[i] = (stab[i] - wmin) / 2 + 1;
                    }
                }
                else {    // 1bit AD
                    for (i = 0; i < chmax; i++) {
                        //skd->ch_by_stat[k*skd->ch_max + i] = (stab[i] - wmin) + 1;
                        ch_table[i] = (stab[i] - wmin) + 1;
                    }
                }
            }
            continue;
        }


    }
exithere:;
    //debug
    /*
    printf("CHinWord ");
    for(i=0; i<chmax; i++) printf(" %d",ch_table[i]);
    printf("\n");
    */
}